@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,19 +33,34 @@
 
 #pragma once
 
+/////////////////////// stdlib includes
+
 
 /////////////////////// Qt includes
 #include <QDialog>
 #include <QGraphicsScene>
 
 
-/////////////////////// Local includes
-#include "ui_AboutDlg.h"
+/////////////////////// pappsomspp includes
 
 
+/////////////////////// libXpertMass includes
+
+
+/////////////////////// libXpertMassGui includes
+
+
+/////////////////////// Local includes
+
+
+namespace Ui
+{
+class AboutDlg;
+}
+
 namespace MsXpS
 {
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -53,11 +68,11 @@ class AboutDlg : public QDialog
 {
   Q_OBJECT
 
-		friend class ProgramWindow;
+  friend class ProgramWindow;
 
-  private:
   public:
-  AboutDlg(QWidget *parent, const QString &applicationName = "massXpert");
+  explicit AboutDlg(QWidget *parent,
+                    const QString &application_name = "MassXpert3");
 
   ~AboutDlg();
 
@@ -68,7 +83,7 @@ class AboutDlg : public QDialog
   protected:
   QString m_applicationName;
 
-  Ui::AboutDlg m_ui;
+  ::Ui::AboutDlg *m_ui;
 
   // Allocated but ownership taken by the graphicsView.
   QGraphicsScene *mp_graphicsScene;
@@ -78,7 +93,6 @@ class AboutDlg : public QDialog
 };
 
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
-
