@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,8 @@
  * END software license
  */
 
+/////////////////////// stdlib includes
+
 
 /////////////////////// Qt includes
 #include <QApplication>
@@ -38,21 +40,30 @@
 #include <QMouseEvent>
 #include <QMessageBox>
 #include <QFileInfo>
+#include <QHeaderView>
+#include <QMenu>
+
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Oligomer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
 
 
 /////////////////////// Local includes
 #include "../nongui/globals.hpp"
 #include "CleaveOligomerTableView.hpp"
 #include "CleaveOligomerTableViewSortProxyModel.hpp"
-#include <libXpertMass/Oligomer.hpp>
 #include "CleavageDlg.hpp"
 #include "CleaveOligomerTableViewMimeData.hpp"
 
 
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -104,16 +115,17 @@ CleaveOligomerTableView::~CleaveOligomerTableView()
 
 
 void
-CleaveOligomerTableView::setOligomerList(OligomerList *oligomerList)
+CleaveOligomerTableView::setOligomerCollection(
+  libXpertMassCore::OligomerCollection *oligomers_p)
 {
-  mp_oligomerList = oligomerList;
+  mp_oligomers = oligomers_p;
 }
 
 
-const OligomerList *
-CleaveOligomerTableView::oligomerList()
+const libXpertMassCore::OligomerCollection *
+CleaveOligomerTableView::getOligomers()
 {
-  return mp_oligomerList;
+  return mp_oligomers;
 }
 
 
@@ -130,28 +142,26 @@ CleaveOligomerTableView::setParentDlg(CleavageDlg *dlg)
   mp_parentDlg = dlg;
 }
 
-
+// The oligomers are deep-copied from this object's mp_oligomers container
+// into the oligomers argument to this function.
 int
-CleaveOligomerTableView::selectedOligomers(OligomerList *oligomerList,
-                                           int index) const
+CleaveOligomerTableView::selectedOligomers(
+  libXpertMassCore::OligomerCollection &oligomers, int index) const
 {
-  if(!oligomerList)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-
   int count = 0;
 
   int localIndex = 0;
 
   // How many oligomers are there in the list passed as argument?
-  int oligomerCount = oligomerList->size();
+  int oligomerCount = oligomers.size();
 
   if(index > oligomerCount)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+    qFatal() << "Programming error.";
 
   // If index is -1 , then we are asked to append the oligomers to
   // the list.
   if(index == -1)
-    localIndex = oligomerList->size();
+    localIndex = oligomers.size();
 
   // For each selected oligomer, duplicate it and append to the list
   // passed as argument.
@@ -203,27 +213,30 @@ CleaveOligomerTableView::selectedOligomers(OligomerList *oligomerList,
         processedRowList.append(row);
 
       // Older version
-      // CleaveOligomer *oligomer =
-      // static_cast<CleaveOligomer *>(mp_oligomerList->at(row).get());
-      // CleaveOligomerSPtr new_oligomer_sp =
-      // std::make_shared<CleaveOligomer>(*oligomer);
+      // Oligomer *oligomer =
+      // static_cast<Oligomer *>(mp_oligomers.at(row).get());
+      // OligomerSPtr new_oligomer_sp =
+      // std::make_shared<Oligomer>(*oligomer);
 
-      CleaveOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<CleaveOligomer>(mp_oligomerList->at(row));
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersRef().at(row);
 
-      CleaveOligomerSPtr new_oligomer_sp =
-        std::make_shared<CleaveOligomer>(*oligomer_sp);
+      libXpertMassCore::OligomerSPtr new_oligomer_sp =
+        std::make_shared<libXpertMassCore::Oligomer>(*oligomer_sp);
 
       // Create a NoDeletePointerProp, which might be used later by
       // the user of the list of oligomers to highlight regions in
       // the sequence editor.
 
-      libXpertMass::NoDeletePointerProp *prop = new libXpertMass::NoDeletePointerProp(
-        "SEQUENCE_EDITOR_WND", static_cast<void *>(mp_parentDlg->editorWnd()));
+      libXpertMassCore::NoDeletePointerProp *prop =
+        new libXpertMassCore::NoDeletePointerProp(
+          "SEQUENCE_EDITOR_WND",
+          static_cast<void *>(mp_parentDlg->editorWnd()));
 
       new_oligomer_sp->appendProp(prop);
 
-      oligomerList->insert(localIndex, new_oligomer_sp);
+      oligomers.getOligomersRef().insert(
+        oligomers.getOligomersRef().begin() + localIndex, new_oligomer_sp);
 
       ++localIndex;
       ++count;
@@ -234,22 +247,22 @@ CleaveOligomerTableView::selectedOligomers(OligomerList *oligomerList,
 
 
 QString *
-CleaveOligomerTableView::selectedOligomersAsPlainText(
+CleaveOligomerTableView::describeSelectedOligomersAsPlainText(
   const QString &pattern,
   QString delimiter,
-  bool withSequence,
-  bool forXpertMiner,
-  libXpertMass::MassType massType) const
+  bool with_sequence,
+  bool for_xpert_miner,
+  libXpertMassCore::Enums::MassType mass_type) const
 {
-  // Let's get all the currently selected oligomers in one list.
-  OligomerList oligomerList;
-
-  // Append the selected oligomers to the empty list.
-  int appendedOligomerCount = selectedOligomers(&oligomerList, -1);
+  // Let's get all the currently selected oligomers in one container. Note that
+  //  the Oligomer instances in oligomers are allocated fully for us. We will
+  //  thefore free them upon use below.
+  libXpertMassCore::OligomerCollection oligomers;
+  std::size_t oligomers_count = selectedOligomers(oligomers, -1);
 
   // Sanity check
-  if(appendedOligomerCount != oligomerList.size())
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+  if(oligomers_count != oligomers.size())
+    qFatal() << "Programming error.";
 
   // There are two possibilities:
   // 1. The pattern string is not empty, in which case it should be used to
@@ -259,7 +272,7 @@ CleaveOligomerTableView::selectedOligomersAsPlainText(
 
   bool withPattern;
 
-  if(pattern.isEmpty() || forXpertMiner)
+  if(pattern.isEmpty() || for_xpert_miner)
     {
       withPattern = false;
 
@@ -278,45 +291,48 @@ CleaveOligomerTableView::selectedOligomersAsPlainText(
   // Slot to get the text from the called functions.
   QString *text = nullptr;
 
-  while(!oligomerList.isEmpty())
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_iterator =
+    oligomers.getOligomersRef().begin();
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_end_iterator =
+    oligomers.getOligomersRef().end();
+
+  while(the_iterator != the_end_iterator)
     {
-      // libXpertMass::OligomerSPtr oligomer_sp = oligomerList.takeFirst();
-
-      CleaveOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<CleaveOligomer>(oligomerList.takeFirst());
-
       if(withPattern)
         {
-          text =
-            selectedOligomersAsPlainTextWithPattern(oligomer_sp.get(), pattern);
+          text = describeOligomerAsPlainTextWithPattern((*the_iterator).get(),
+                                                        pattern);
+
           if(text == nullptr)
-            qFatal(
-              "Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+            qFatal() << "Programming error.";
 
           *output += *text;
+
           delete text;
         }
       else
         {
-          text = selectedOligomersAsPlainTextWithDelimiter(oligomer_sp.get(),
-                                                           delimiter,
-                                                           withSequence,
-                                                           forXpertMiner,
-                                                           massType);
+          text = describeOligomerAsPlainTextWithDelimiter((*the_iterator).get(),
+                                                          delimiter,
+                                                          with_sequence,
+                                                          for_xpert_miner,
+                                                          mass_type);
           if(text == nullptr)
-            qFatal(
-              "Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+            qFatal() << "Programming error.";
 
           *output += *text;
+
           delete text;
         }
 
-      // At this point we have done the work for the current oligomer, make
-      // sure we delete it now.
+      ++the_iterator;
 
-      oligomer_sp.reset();
+      // The version below is buggy. See oligomers.clear() below.
+      // the_iterator = oligomers.getOligomersRef().erase(the_iterator);
     }
 
+  oligomers.clear();
+
   // Terminate the string with a new line.
   *output += QString("\n");
 
@@ -325,68 +341,73 @@ CleaveOligomerTableView::selectedOligomersAsPlainText(
 
 
 QString *
-CleaveOligomerTableView::selectedOligomersAsPlainTextWithDelimiter(
-  libXpertMass::Oligomer *p_oligomer,
+CleaveOligomerTableView::describeOligomerAsPlainTextWithDelimiter(
+  libXpertMassCore::Oligomer *oligomer_p,
   const QString &delimiter,
-  bool withSequence,
-  bool forXpertMiner,
-  libXpertMass::MassType massType) const
+  bool with_sequence,
+  bool for_xpert_miner,
+  libXpertMassCore::Enums::MassType massType) const
 {
-  if(p_oligomer == nullptr)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+  if(oligomer_p == nullptr)
+    qFatal() << "Programming error.";
 
   // For export to XpertMiner, we only want the masses asked for:
-  // libXpertMass::MassType::MASS_MONO or MASS_AVG. Also, we want the format to be
-  // :
-
-  // mass <delim> charge <delim> name <delim> coords
+  // libXpertMassCore::Enums::MassType::MONO or AVG. Also, we want the format to
+  // be: mass <delim> charge <delim> name <delim> coords
 
   QString *text = new QString;
 
-  if(!forXpertMiner)
-    *text += QString("\n%1%2").arg(p_oligomer->description()).arg(delimiter);
+  if(!for_xpert_miner)
+    *text += QString("\n%1%2").arg(oligomer_p->getDescription()).arg(delimiter);
 
-  if(forXpertMiner && massType == libXpertMass::MassType::MASS_AVG)
+  if(for_xpert_miner && massType == libXpertMassCore::Enums::MassType::AVG)
     {
     }
   else
     {
       *text += QString("%1%2")
-                 .arg(p_oligomer->monoString(libXpertMass::OLIGOMER_DEC_PLACES))
+                 .arg(oligomer_p->getMass(libXpertMassCore::Enums::MassType::MONO),
+                      0,
+                      'f',
+                      libXpertMassCore::OLIGOMER_DEC_PLACES)
                  .arg(delimiter);
     }
 
-  if(forXpertMiner && massType == libXpertMass::MassType::MASS_MONO)
+  if(for_xpert_miner && massType == libXpertMassCore::Enums::MassType::MONO)
     {
     }
   else
     {
       *text += QString("%1%2")
-                 .arg(p_oligomer->avgString(libXpertMass::OLIGOMER_DEC_PLACES))
+                 .arg(oligomer_p->getMass(libXpertMassCore::Enums::MassType::AVG),
+                      0,
+                      'f',
+                      libXpertMassCore::OLIGOMER_DEC_PLACES)
                  .arg(delimiter);
     }
 
-  *text += QString("%1%2").arg(p_oligomer->charge()).arg(delimiter);
-
-  *text += QString("%1%2").arg(p_oligomer->name()).arg(delimiter);
-
   *text +=
-    QString("%1%2")
-      .arg(
-        static_cast<libXpertMass::CoordinateList *>(p_oligomer)->positionsAsText())
-      .arg(delimiter);
+    QString("%1%2").arg(oligomer_p->getIonizerCstRef().charge()).arg(delimiter);
 
-  if(!forXpertMiner)
-    *text += QString("%1%2").arg(p_oligomer->isModified()).arg(delimiter);
+  *text += QString("%1%2").arg(oligomer_p->getName()).arg(delimiter);
+
+  *text += QString("%1%2")
+             .arg(oligomer_p->getCalcOptionsCstRef()
+                    .getIndexRangeCollectionCstRef()
+                    .positionsAsText())
+             .arg(delimiter);
+
+  if(!for_xpert_miner)
+    *text += QString("%1%2").arg(oligomer_p->isModified()).arg(delimiter);
 
   // We cannot export the sequence if data are for XpertMiner
-  if(!forXpertMiner && withSequence)
+  if(!for_xpert_miner && with_sequence)
     {
-      QString *sequence = p_oligomer->monomerText();
+      const QString sequence =
+        oligomer_p->getPolymerCstSPtr()->getSequenceCstRef().getSequence(
+          oligomer_p->getCalcOptionsCstRef().getIndexRangeCollectionCstRef());
 
-      *text += QString("\n%1").arg(*sequence);
-
-      delete sequence;
+      *text += QString("\n%1").arg(sequence);
     }
 
   // Terminate the stanza
@@ -397,8 +418,8 @@ CleaveOligomerTableView::selectedOligomersAsPlainTextWithDelimiter(
 
 
 QString *
-CleaveOligomerTableView::selectedOligomersAsPlainTextWithPattern(
-  libXpertMass::Oligomer *p_oligomer, const QString &pattern) const
+CleaveOligomerTableView::describeOligomerAsPlainTextWithPattern(
+  libXpertMassCore::Oligomer *oligomer_p, const QString &pattern) const
 {
   // We receive a pattern that we use to craft the output. The following
   // shortcuts are used:
@@ -414,8 +435,14 @@ CleaveOligomerTableView::selectedOligomersAsPlainTextWithPattern(
   // %p - partial
   // %i - polymer sequence name (identity)
 
-  if(p_oligomer == nullptr)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+  if(oligomer_p == nullptr)
+    qFatal() << "Programming error.";
+
+  // qDebug() << oligomer_p->getCalcOptionsCstRef()
+  //               .getIndexRangeCollectionCstRef()
+  //               .positionsAsText()
+  //          << "with name:" << oligomer_p->getName();
+
 
   QString *text = new QString;
 
@@ -489,48 +516,63 @@ CleaveOligomerTableView::selectedOligomersAsPlainTextWithPattern(
           if(curChar == 'f')
             {
               QFileInfo fileInfo(
-                mp_parentDlg->editorWnd()->polymer()->filePath());
+                mp_parentDlg->editorWnd()->getPolymer()->getFilePath());
               if(fileInfo.exists())
                 *text += fileInfo.fileName();
               else
                 *text += "Untitled";
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'm')
             {
-              *text += p_oligomer->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
+              *text += QString("%1").arg(
+                oligomer_p->getMass(libXpertMassCore::Enums::MassType::MONO),
+                0,
+                'f',
+                libXpertMassCore::OLIGOMER_DEC_PLACES);
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'a')
             {
-              *text += p_oligomer->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
+              *text += QString("%1").arg(
+                oligomer_p->getMass(libXpertMassCore::Enums::MassType::AVG),
+                0,
+                'f',
+                libXpertMassCore::OLIGOMER_DEC_PLACES);
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'z')
             {
-              *text += QString("%1").arg(p_oligomer->charge());
+              *text +=
+                QString("%1").arg(oligomer_p->getIonizerCstRef().charge());
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 's')
             {
-              QString *sequence = p_oligomer->monomerText();
-
-              *text += *sequence;
-
-              delete sequence;
+              *text += oligomer_p->getPolymerCstSPtr()
+                         ->getSequenceCstRef()
+                         .getSequence(oligomer_p->getCalcOptionsCstRef()
+                                        .getIndexRangeCollectionCstRef());
 
               prevChar = ' ';
               continue;
             }
           if(curChar == 'l')
             {
-              QString *sequence      = p_oligomer->monomerText();
-              QString elidedSequence = libXpertMass::elideText(*sequence);
-              delete sequence;
+              QString elidedSequence = libXpertMassCore::Utils::elideText(
+                oligomer_p->getPolymerCstSPtr()
+                  ->getSequenceCstRef()
+                  .getSequence(oligomer_p->getCalcOptionsCstRef()
+                                 .getIndexRangeCollectionCstRef()));
+
               *text += elidedSequence;
 
               prevChar = ' ';
@@ -538,44 +580,45 @@ CleaveOligomerTableView::selectedOligomersAsPlainTextWithPattern(
             }
           if(curChar == 'c')
             {
-              *text += static_cast<libXpertMass::CoordinateList *>(p_oligomer)
-                         ->positionsAsText();
+              *text += oligomer_p->getCalcOptionsCstRef()
+                         .getIndexRangeCollectionCstRef()
+                         .positionsAsText();
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'e')
             {
-              CleaveOligomer *cleaveOligomer =
-                static_cast<CleaveOligomer *>(p_oligomer);
+              *text += oligomer_p->getDescription();
 
-              *text += cleaveOligomer->description();
               prevChar = ' ';
               continue;
             }
           if(curChar == 'o')
             {
-              *text += QString("%1").arg(p_oligomer->isModified());
+              *text += QString("%1").arg(oligomer_p->isModified());
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'n')
             {
-              *text += p_oligomer->name();
+              *text += oligomer_p->getName();
+
               prevChar = ' ';
               continue;
             }
           if(curChar == 'p')
             {
-              CleaveOligomer *cleaveOligomer =
-                static_cast<CleaveOligomer *>(p_oligomer);
+              *text += QString("%1").arg(oligomer_p->getPartialCleavage());
 
-              *text += QString("%1").arg(cleaveOligomer->partialCleavage());
               prevChar = ' ';
               continue;
             }
           if(curChar == 'i')
             {
-              *text += mp_parentDlg->editorWnd()->polymer()->name();
+              *text += mp_parentDlg->editorWnd()->getPolymer()->getName();
+
               prevChar = ' ';
               continue;
             }
@@ -667,19 +710,25 @@ CleaveOligomerTableView::currentChanged(const QModelIndex &current,
   // tableView (that list actually belongs to the CleavageDlg
   // instance.
 
-  CleaveOligomerSPtr oligomer_sp =
-    std::dynamic_pointer_cast<CleaveOligomer>(mp_oligomerList->at(row));
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersCstRef().at(row);
 
   // If the oligomers obtained with the cleavage are old and the
   // sequence has been changed since the cleavage, then the
   // oligomers might point to a sequence element that is no more. We
   // want to avoid such kind of errors.
 
-  if(oligomer_sp->startIndex() >= oligomer_sp->polymer()->size() ||
-     oligomer_sp->endIndex() >= oligomer_sp->polymer()->size())
+  if(oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .leftMostIndexRangeStart() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+     oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .rightMostIndexRangeStop() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - Cleavage"),
+                           tr("MassXpert3 - Cleavage"),
                            tr("%1@%2\n"
                               "The monomer indices do not correspond "
                               "to a valid polymer sequence range.\n"
@@ -692,31 +741,31 @@ CleaveOligomerTableView::currentChanged(const QModelIndex &current,
       return;
     }
 
-  QString *text = oligomer_sp->monomerText();
+  QString text =
+    oligomer_sp->getPolymerCstSPtr()->getSequenceCstRef().getSequence(
+      oligomer_sp->getCalcOptionsCstRef().getIndexRangeCollectionCstRef());
 
   // We are getting text for an oligomer; it cannot be empty,
   // because that would mean the oligomer has no monomers. In that
   // case it is not conceivable that the oligomer be in the cleavage
   // product list.
 
-  Q_ASSERT(!text->isEmpty());
+  Q_ASSERT(!text.isEmpty());
 
-  *text += QString(" -- %1").arg(oligomer_sp->description());
+  text += QString(" -- %1").arg(oligomer_sp->getDescription());
 
   // Get the formula of the oligomer and display it all along.
 
   QString formula = oligomer_sp->elementalComposition();
 
-  *text += QString(" -- Formula: %1").arg(formula);
+  text += QString(" -- Formula: %1").arg(formula);
 
-  mp_parentDlg->updateOligomerSequence(text);
-
-  delete text;
+  mp_parentDlg->updateOligomerSequence(&text);
 
   // Get the mass calculation engine's options out of the oligomer,
   // so that we can display them correctly.
 
-  libXpertMass::CalcOptions calcOptions = oligomer_sp->calcOptions();
+  libXpertMassCore::CalcOptions calcOptions(oligomer_sp->getCalcOptionsCstRef());
 
   mp_parentDlg->updateCleavageDetails(calcOptions);
 
@@ -741,29 +790,25 @@ CleaveOligomerTableView::itemActivated(const QModelIndex &index)
   // tableView (that list actually belongs to the CleavageDlg
   // instance.
 
-  CleaveOligomer *oligomer =
-    static_cast<CleaveOligomer *>(mp_oligomerList->at(row).get());
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersRef().at(row);
 
-  SequenceEditorWnd *editorWnd = mp_parentDlg->editorWnd();
+  SequenceEditorWnd *seq_editor_wnd = mp_parentDlg->editorWnd();
 
-  libXpertMass::CoordinateList *coordinateList =
-    static_cast<libXpertMass::CoordinateList *>(oligomer);
+  const libXpertMassCore::IndexRangeCollection &index_range_collection =
+    oligomer_sp->getCalcOptionsCstRef().getIndexRangeCollectionCstRef();
 
   // Remove the previous selection, so that we can start fresh.
-  editorWnd->mpa_editorGraphicsView->resetSelection();
+  seq_editor_wnd->mpa_editorGraphicsView->resetSelection();
 
-  for(int iter = 0; iter < coordinateList->size(); ++iter)
+  foreach(const libXpertMassCore::IndexRange *item,
+          index_range_collection.getRangesCstRef())
     {
-      libXpertMass::Coordinates *coordinates = coordinateList->at(iter);
-
-      int start = coordinates->start();
-      int end   = coordinates->end();
-
-      if(start >= oligomer->polymer()->size() ||
-         end >= oligomer->polymer()->size())
+      if(item->m_start >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+         item->m_stop >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
         {
           QMessageBox::warning(this,
-                               tr("massXpert - Cleavage"),
+                               tr("MassXpert3 - Cleavage"),
                                tr("%1@%2\n"
                                   "The monomer indices do not correspond "
                                   "to a valid polymer sequence range.\n"
@@ -776,11 +821,10 @@ CleaveOligomerTableView::itemActivated(const QModelIndex &index)
           return;
         }
 
-      editorWnd->mpa_editorGraphicsView->setSelection(
-        *coordinates, true, false);
+      seq_editor_wnd->mpa_editorGraphicsView->setSelection(*item, true, false);
     }
 
-  editorWnd->updateSelectedSequenceMasses();
+  seq_editor_wnd->updateSelectedSequenceMasses();
 }
 
 
@@ -789,21 +833,21 @@ CleaveOligomerTableView::itemActivated(const QModelIndex &index)
 void
 CleaveOligomerTableView::copyMono()
 {
-  return copyMassList(libXpertMass::MassType::MASS_MONO);
+  return copyMassList(libXpertMassCore::Enums::MassType::MONO);
 }
 
 
 void
 CleaveOligomerTableView::copyAvg()
 {
-  return copyMassList(libXpertMass::MassType::MASS_AVG);
+  return copyMassList(libXpertMassCore::Enums::MassType::AVG);
 }
 
 
 void
-CleaveOligomerTableView::copyMassList(int monoOrAvg)
+CleaveOligomerTableView::copyMassList(libXpertMassCore::Enums::MassType mass_type)
 {
-  QString massList;
+  QString masses_text;
 
   // We want to prepare a textual list of masses (either MONO or
   // AVG) of all the oligomers in the tableview, exactly as they are
@@ -823,33 +867,25 @@ CleaveOligomerTableView::copyMassList(int monoOrAvg)
       QModelIndex proxyIndex  = sortModel->index(iter, 0);
       QModelIndex sourceIndex = sortModel->mapToSource(proxyIndex);
 
-      int sourceRow = sourceIndex.row();
+      int row = sourceIndex.row();
 
       // qDebug() << __FILE__ << __LINE__
-      //          << "sourceRow:" << sourceRow;
+      //          << "row:" << row;
 
-      CleaveOligomer *oligomer =
-        static_cast<CleaveOligomer *>(mp_oligomerList->at(sourceRow).get());
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersRef().at(row);
 
-      if(monoOrAvg == libXpertMass::MassType::MASS_MONO)
-        massList += oligomer->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
-      else if(monoOrAvg == libXpertMass::MassType::MASS_AVG)
-        massList += oligomer->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
-      else
-        qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-
-      // End the mass item with a new line.
-      massList += "\n";
+      masses_text += QString("%1\n").arg(oligomer_sp->getMass(mass_type),
+                                         0,
+                                         'f',
+                                         libXpertMassCore::OLIGOMER_DEC_PLACES);
     }
 
-  if(massList.isEmpty())
+  if(masses_text.isEmpty())
     return;
 
-  QClipboard *clipboard = QApplication::clipboard();
-
-  clipboard->setText(massList, QClipboard::Clipboard);
+  QApplication::clipboard()->setText(masses_text, QClipboard::Clipboard);
 }
 
-} // namespace massxpert
-
+} // namespace MassXpert
 } // namespace MsXpS
