@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,18 +41,18 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
 CleaveOligomerTableViewModel::CleaveOligomerTableViewModel(
-  OligomerList *oligomerList, QObject *parent)
+  libXpertMassCore::OligomerCollection *oligomerList, QObject *parent)
   : QAbstractTableModel(parent)
 {
   if(!oligomerList)
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
 
-  mp_oligomerList = oligomerList;
+  mp_oligomers = oligomerList;
 
   if(!parent)
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
@@ -69,10 +69,10 @@ CleaveOligomerTableViewModel::~CleaveOligomerTableViewModel()
 }
 
 
-const OligomerList *
-CleaveOligomerTableViewModel::oligomerList()
+libXpertMassCore::OligomerCollection *
+CleaveOligomerTableViewModel::getOligomers()
 {
-  return mp_oligomerList;
+  return mp_oligomers;
 }
 
 
@@ -105,7 +105,7 @@ CleaveOligomerTableViewModel::rowCount(const QModelIndex &parent) const
 {
   Q_UNUSED(parent);
 
-  return mp_oligomerList->size();
+  return mp_oligomers->size();
 }
 
 
@@ -181,9 +181,9 @@ CleaveOligomerTableViewModel::data(const QModelIndex &index, int role) const
       // row. Let's find the row first, so that we get to the proper
       // oligomer.
 
-      CleaveOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<CleaveOligomer>(mp_oligomerList->at(row));
-        
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersCstRef().at(row);
+
       // Now see what's the column that is asked for. Prepare a
       // string that we'll feed with the right data before returning
       // it as a QVariant.
@@ -192,47 +192,59 @@ CleaveOligomerTableViewModel::data(const QModelIndex &index, int role) const
 
       if(column == CLEAVE_OLIGO_PARTIAL_COLUMN)
         {
-          int partialCleavage = oligomer_sp->partialCleavage();
+          int partialCleavage = oligomer_sp->getPartialCleavage();
 
           valueString.setNum(partialCleavage);
         }
       else if(column == CLEAVE_OLIGO_NAME_COLUMN)
         {
-          valueString = oligomer_sp->name();
+          valueString = oligomer_sp->getName();
         }
       else if(column == CLEAVE_OLIGO_COORDINATES_COLUMN)
         {
-          int oligomerCount =
-            static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())->size();
+          if(!oligomer_sp->getCalcOptionsRef()
+                .getIndexRangeCollectionCstRef()
+                .getRangesCstRef()
+                .size())
+            qDebug() << "The IndexRangeCollection is empty!!";
 
-
-          for(int jter = 0; jter < oligomerCount; ++jter)
-            {
-              libXpertMass::Coordinates *coordinates =
-                static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())->at(jter);
-
-              valueString += QString("[%1-%2]")
-                               .arg(coordinates->start() + 1)
-                               .arg(coordinates->end() + 1);
-            }
+          foreach(const libXpertMassCore::IndexRange *item,
+                  oligomer_sp->getCalcOptionsRef()
+                    .getIndexRangeCollectionCstRef()
+                    .getRangesCstRef())
+            valueString +=
+              QString("[%1-%2]").arg(item->m_start + 1).arg(item->m_stop + 1);
         }
       else if(column == CLEAVE_OLIGO_MONO_COLUMN)
         {
-          valueString = oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
+          valueString = QString("%1\n").arg(
+            oligomer_sp->getMass(libXpertMassCore::Enums::MassType::MONO),
+            0,
+            'f',
+            libXpertMassCore::OLIGOMER_DEC_PLACES);
         }
       else if(column == CLEAVE_OLIGO_AVG_COLUMN)
         {
-          valueString = oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
+          valueString = QString("%1\n").arg(
+            oligomer_sp->getMass(libXpertMassCore::Enums::MassType::AVG),
+            0,
+            'f',
+            libXpertMassCore::OLIGOMER_DEC_PLACES);
         }
       else if(column == CLEAVE_OLIGO_CHARGE_COLUMN)
         {
-          int charge = oligomer_sp->charge();
-
-          valueString.setNum(charge);
+          valueString.setNum(oligomer_sp->getIonizerCstRef().charge());
         }
       else if(column == CLEAVE_OLIGO_FORMULA_COLUMN)
         {
-          valueString = oligomer_sp->elementalComposition();
+          // elementalComposition() will not yield the right formula
+          // when the cleavage has been performed using a cleavage agent
+          // that has cleavage rules. Indeed, elementalComposition() only
+          // look at "hard" oligomer data : sequence, modifications, cross-links,
+          // but not formulas that are applied as a result of a cleavage rule
+          // application.
+          // valueString = oligomer_sp->elementalComposition();
+          valueString = oligomer_sp->getFormulaCstRef().getActionFormula(false);
         }
       else if(column == CLEAVE_OLIGO_MODIF_COLUMN)
         {
@@ -259,7 +271,8 @@ CleaveOligomerTableViewModel::data(const QModelIndex &index, int role) const
 }
 
 int
-CleaveOligomerTableViewModel::addOligomers(const OligomerList &oligomerList)
+CleaveOligomerTableViewModel::addOligomers(
+  const libXpertMassCore::OligomerCollection &oligomers)
 {
   // We receive a list of oligomers which we are asked to add to the
   // list of oligomers that we actually do not own, since they are
@@ -270,53 +283,45 @@ CleaveOligomerTableViewModel::addOligomers(const OligomerList &oligomerList)
   // addition work here.
 
   // Note that we are acutally *transferring* all the oligomers from
-  // the oligomerList to this model's mp_oligomerList.
+  // the oligomerList to this model's mp_oligomers.
 
-  int oligomerCount = mp_oligomerList->size();
-
-  // qDebug() << __FILE__ << __LINE__
-  //          << "model oligomers:" << oligomerCount;
-
-  int oligomersToAdd = oligomerList.size();
-
-  // qDebug() << __FILE__ << __LINE__
-  //          << "oligomers to add:" << oligomersToAdd;
-
-  int addedOligomerCount = 0;
+  std::size_t initial_size = mp_oligomers->getOligomersCstRef().size();
 
   // We have to let know the model that we are going to modify the
   // list of oligomers and thus the number of rows in the table
   // view. We will append the oligomers to the preexisting ones,
   // which means we are going to have our first appended oligomer at
-  // index = oligomerCount.
+  // index = oligomer_count.
 
-  beginInsertRows(
-    QModelIndex(), oligomerCount, (oligomerCount + oligomersToAdd - 1));
+  beginInsertRows(QModelIndex(),
+                  initial_size,
+                  (initial_size + oligomers.getOligomersCstRef().size() - 1));
 
+  for(const libXpertMassCore::OligomerSPtr &oligomer_sp :
+      oligomers.getOligomersCstRef())
+    mp_oligomers->getOligomersRef().push_back(oligomer_sp);
+
+#if 0
+//  old version
   for(int iter = 0; iter < oligomersToAdd; ++iter)
     {
-      CleaveOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<CleaveOligomer>(oligomerList.at(iter));
-
-      mp_oligomerList->append(std::make_shared<CleaveOligomer>(*oligomer_sp));
-
-      ++addedOligomerCount;
+      libXpertMassCore::OligomerSPtr oligomer_sp =oligomers.at(iter);
+      mp_oligomers->append(std::make_shared<CleaveOligomer>(*oligomer_sp));
+      ++added_oligomer_count;
     }
+#endif
 
   endInsertRows();
 
-  return addedOligomerCount;
+  return mp_oligomers->size() - initial_size;
 }
 
 
 int
-CleaveOligomerTableViewModel::removeOligomers(int firstIndex, int lastIndex)
+CleaveOligomerTableViewModel::removeOligomers(std::size_t start_index,
+                                              std::size_t stop_index)
 {
-  // We are asked to remove the oligomers [firstIndex--lastIndex].
-
-  // firstIndex has to be >= 0 and < mp_oligomerList->size(). If
-  // lastIndex is -1, then remove all oligomers in range
-  // [firstIndex--last oligomer].
+  // We are asked to remove the oligomers [start_index--stop_index].
 
   // We are asked to remove the oligomers from the list of oligomers
   // that we actually do not own, since they are owned by the
@@ -326,62 +331,43 @@ CleaveOligomerTableViewModel::removeOligomers(int firstIndex, int lastIndex)
   // that the tableView takes them into account, we have to do that
   // removal work here.
 
-  int oligomerCount = mp_oligomerList->size();
+  std::size_t initial_size = mp_oligomers->size();
 
-  if(!oligomerCount)
+  if(!initial_size)
     return 0;
 
-  if(firstIndex < 0 || firstIndex >= oligomerCount)
-    {
-      // qDebug() << __FILE__ << __LINE__
-      //          << "firstIndex:" << firstIndex
-      //          << "lastIndex:" << lastIndex;
+  if(start_index >= initial_size)
+    qFatal() << "Programming error.";
+  if(stop_index >= initial_size)
+    qFatal() << "Programming error.";
 
-      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-    }
-
-  int firstIdx = firstIndex;
-
-  // lastIndex can be < 0 (that is -1) if the oligomer to remove
-  // must go up to the last oligomer in the list.
-  if(lastIndex < -1 || lastIndex >= oligomerCount)
-    {
-      // qDebug() << __FILE__ << __LINE__
-      //          << "firstIndex:" << firstIndex
-      //          << "lastIndex:" << lastIndex;
-
-      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-    }
-
-  int lastIdx = 0;
-
-  if(lastIndex == -1)
-    lastIdx = (oligomerCount - 1);
-  else
-    lastIdx = lastIndex;
-
-  beginRemoveRows(QModelIndex(), firstIdx, lastIdx);
-
-  int removedOligomerCount = 0;
+  beginRemoveRows(QModelIndex(), start_index, stop_index);
 
+#if 0
+// Old version
   int iter = lastIdx;
-
   while(iter >= firstIdx)
     {
       // The shared pointer will help with the life of the object.
-      mp_oligomerList->takeAt(iter);
-
+      mp_oligomers->takeAt(iter);
       ++removedOligomerCount;
-
       --iter;
-    }
+}
+#endif
+
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_begin_iterator =
+    mp_oligomers->getOligomersRef().begin() + start_index;
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_end_iterator =
+    mp_oligomers->getOligomersRef().begin() + stop_index + 1;
+
+  mp_oligomers->getOligomersRef().erase(the_begin_iterator, the_end_iterator);
 
   endRemoveRows();
 
-  return removedOligomerCount;
+  return initial_size - mp_oligomers->size();
 }
 
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
