@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +31,9 @@
  */
 
 
-#ifndef CLEAVE_OLIGOMER_TABLE_VIEW_MODEL_HPP
-#define CLEAVE_OLIGOMER_TABLE_VIEW_MODEL_HPP
+#pragma once
+
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
@@ -40,16 +41,25 @@
 #include <QAbstractTableModel>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/OligomerCollection.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include "../nongui/OligomerList.hpp"
 #include "CleavageDlg.hpp"
+#include "CleaveOligomerTableView.hpp"
+
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 enum
@@ -71,10 +81,10 @@ class CleaveOligomerTableViewModel : public QAbstractTableModel
   Q_OBJECT
 
   public:
-  CleaveOligomerTableViewModel(OligomerList *, QObject *);
+  CleaveOligomerTableViewModel(libXpertMassCore::OligomerCollection *, QObject *);
   ~CleaveOligomerTableViewModel();
 
-  const OligomerList *oligomerList();
+  libXpertMassCore::OligomerCollection *getOligomers();
   const CleavageDlg *parentDlg() const;
 
   void setTableView(CleaveOligomerTableView *);
@@ -88,19 +98,15 @@ class CleaveOligomerTableViewModel : public QAbstractTableModel
   QVariant data(const QModelIndex &parent = QModelIndex(),
                 int role                  = Qt::DisplayRole) const;
 
-  int addOligomers(const OligomerList &);
-  int removeOligomers(int, int);
+  int addOligomers(const libXpertMassCore::OligomerCollection &oligomers);
+  int removeOligomers(std::size_t start_index, std::size_t stop_index);
 
   private:
-  OligomerList *mp_oligomerList;
+  libXpertMassCore::OligomerCollection *mp_oligomers;
   CleavageDlg *mp_parentDlg;
   CleaveOligomerTableView *mp_tableView;
-
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // CLEAVE_OLIGOMER_TABLE_VIEW_MODEL_HPP
