@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,53 +39,53 @@
 /////////////////////// Local includes
 #include "CompositionTreeViewSortProxyModel.hpp"
 #include "CompositionTreeView.hpp"
-#include <libXpertMass/Monomer.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
 #include "CompositionsDlg.hpp"
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-  class CompositionTreeViewModel;
-  class CompositionTreeViewItem;
+class CompositionTreeViewModel;
+class CompositionTreeViewItem;
 
 
-  CompositionTreeView::CompositionTreeView(QWidget *parent) : QTreeView(parent)
-  {
+CompositionTreeView::CompositionTreeView(QWidget *parent) : QTreeView(parent)
+{
 
-    setAlternatingRowColors(true);
-    setAllColumnsShowFocus(true);
-    setSortingEnabled(true);
+  setAlternatingRowColors(true);
+  setAllColumnsShowFocus(true);
+  setSortingEnabled(true);
 
-    QHeaderView *headerView = header();
+  QHeaderView *headerView = header();
 
-    headerView->setSectionsClickable(true);
-    headerView->setSectionsMovable(true);
-  }
+  headerView->setSectionsClickable(true);
+  headerView->setSectionsMovable(true);
+}
 
 
-  CompositionTreeView::~CompositionTreeView()
-  {
-  }
+CompositionTreeView::~CompositionTreeView()
+{
+}
 
 
-  CompositionsDlg *
-  CompositionTreeView::getParentDlg()
-  {
-    return mp_parentDlg;
-  }
+CompositionsDlg *
+CompositionTreeView::getParentDlg()
+{
+  return mp_parentDlg;
+}
 
-  void
-  CompositionTreeView::setParentDlg(CompositionsDlg *dlg)
-  {
-    Q_ASSERT(dlg);
-    mp_parentDlg = dlg;
-  }
+void
+CompositionTreeView::setParentDlg(CompositionsDlg *dlg)
+{
+  Q_ASSERT(dlg);
+  mp_parentDlg = dlg;
+}
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
