@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,15 +43,14 @@
 
 
 /////////////////////// Local includes
-#include <libXpertMass/Monomer.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
 
 
 namespace MsXpS
 {
 
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 enum
@@ -70,7 +69,7 @@ class CompositionTreeViewItem
   QList<CompositionTreeViewItem *> m_childItemsList;
   QList<QVariant> m_itemData;
   CompositionTreeViewItem *mp_parentItem;
-  libXpertMass::Monomer *mp_monomer;
+  libXpertMassCore::Monomer *mp_monomer;
 
   public:
   CompositionTreeViewItem(const QList<QVariant> &data,
@@ -95,11 +94,11 @@ class CompositionTreeViewItem
 
   int row() const;
 
-  void setMonomer(libXpertMass::Monomer *);
-  libXpertMass::Monomer *monomer();
+  void setMonomer(libXpertMassCore::Monomer *);
+  libXpertMassCore::Monomer *monomer();
 };
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
 
