@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,27 +30,34 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef COMPOSITION_TREEVIEW_MODEL_HPP
-#define COMPOSITION_TREEVIEW_MODEL_HPP
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
 #include <QTreeView>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include <libXpertMass/Monomer.hpp>
 #include "CompositionTreeViewItem.hpp"
 #include "CompositionsDlg.hpp"
 
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class CompositionTreeViewItem;
@@ -62,13 +69,13 @@ class CompositionTreeViewModel : public QAbstractItemModel
   Q_OBJECT
 
   private:
-  QList<libXpertMass::Monomer *> *mp_list;
+  QList<libXpertMassCore::Monomer *> *mp_list;
   CompositionTreeViewItem *mpa_rootItem;
   QTreeView *mp_treeView;
   CompositionsDlg *mp_parentDlg;
 
   public:
-  CompositionTreeViewModel(QList<libXpertMass::Monomer *> *, QObject *);
+  CompositionTreeViewModel(QList<libXpertMassCore::Monomer *> *, QObject *);
 
   ~CompositionTreeViewModel();
 
@@ -92,15 +99,12 @@ class CompositionTreeViewModel : public QAbstractItemModel
 
   void removeAll();
 
-  void addMonomer(libXpertMass::Monomer *);
+  void addMonomer(libXpertMassCore::Monomer *);
   bool setupModelData(CompositionTreeViewItem *);
 
   public slots:
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // COMPOSITION_TREEVIEW_MODEL_HPP
