@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,10 +37,10 @@
 #include <QSettings>
 
 
-/////////////////////// libmass includes
-#include <libXpertMass/Coordinates.hpp>
-#include <libXpertMass/CrossLink.hpp>
-#include <libXpertMass/Monomer.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/CrossLink.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
+#include <MsXpS/libXpertMassCore/Ionizer.hpp>
 
 
 /////////////////////// Local includes
@@ -49,52 +49,46 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-CompositionsDlg::CompositionsDlg(SequenceEditorWnd *editorWnd,
-                                 libXpertMass::Polymer *polymer,
-                                 /* no libXpertMass::PolChemDef **/
-                                 const QString &configSettingsFilePath,
-                                 const QString &applicationName,
+CompositionsDlg::CompositionsDlg(SequenceEditorWnd *editor_wnd_p,
+                                 libXpertMassCore::PolymerQSPtr polymer_sp,
+                                 /* no libXpertMassCore::PolChemDef **/
+                                 const QString &config_settings_file_path,
+                                 const QString &application_name,
                                  const QString &description,
-                                 libXpertMass::CalcOptions *calcOptions,
-                                 libXpertMass::IonizeRule *ionizeRule)
-  : AbstractSeqEdWndDependentDlg(editorWnd,
-                                 polymer,
-                                 0 /*polChemDef*/,
-                                 configSettingsFilePath,
+                                 const libXpertMassCore::CalcOptions &calc_options,
+                                 const libXpertMassCore::Ionizer &ionizer)
+  : AbstractSeqEdWndDependentDlg(editor_wnd_p,
+                                 polymer_sp,
+                                 nullptr /*polChemDef*/,
+                                 config_settings_file_path,
                                  "CompositionsDlg",
-                                 applicationName,
+                                 application_name,
                                  description),
-    mp_calcOptions{calcOptions},
-    mp_ionizeRule{ionizeRule}
+    mref_calcOptions(calc_options),
+    m_ionizer(ionizer)
 {
-  if(!mp_polymer)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-
-  if(!mp_calcOptions || !mp_ionizeRule)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+  if(polymer_sp == nullptr)
+    qFatal() << "Programming error. Pointer cannot be nullptr.";
 
   if(!initialize())
-    qFatal(
-      "Fatal error at %s@%d. Failed to initialize the %s window. Program "
-      "aborted.",
-      __FILE__,
-      __LINE__,
-      m_wndTypeName.toLatin1().data());
+    qFatal()
+      << "Programming error. Failed to initialize the dialog window"
+      << m_wndTypeName.toLatin1().data();
 
-  setWindowTitle(QString("%1 - %2").arg(applicationName).arg(description));
+  setWindowTitle(QString("%1 - %2").arg(application_name).arg(description));
 }
 
 
 CompositionsDlg::~CompositionsDlg()
 {
+  writeSettings();
+
   delete mpa_compositionTreeViewModel;
-
   delete mpa_compositionProxyModel;
-
   delete mpa_resultsString;
 
   freeMonomerList();
@@ -130,6 +124,8 @@ CompositionsDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Update the window title because the window title element in m_ui might be
   // either erroneous or empty.
   setWindowTitle(
@@ -154,8 +150,6 @@ CompositionsDlg::initialize()
 
   mpa_resultsString = new QString();
 
-  //////////////////////////////////// The results-exporting menus.
-
   readSettings();
 
   connect(m_ui.updateSelectionDataPushButton,
@@ -182,10 +176,11 @@ CompositionsDlg::updateSelectionData()
 {
   // The selection might exist as a list of region selections.
 
-  if(mp_editorWnd->mpa_editorGraphicsView->selectionIndices(&m_coordinateList))
+  if(mp_editorWnd->mpa_editorGraphicsView->selectionIndices(
+       m_indexRangeCollection))
     {
       m_ui.selectionCoordinatesLineEdit->setText(
-        m_coordinateList.positionsAsText());
+        m_indexRangeCollection.positionsAsText());
 
       m_ui.selectedSequenceRadioButton->setChecked(true);
     }
@@ -203,39 +198,42 @@ CompositionsDlg::updateSelectionData()
 bool
 CompositionsDlg::fetchValidateInputData()
 {
+  qsizetype polymer_size = (qsizetype)msp_polymer->size();
+
   if(!m_ui.selectedSequenceRadioButton->isChecked())
     {
-      m_coordinateList.setCoordinates(
-        libXpertMass::Coordinates(0, mp_polymer->size() - 1));
+      m_indexRangeCollection.setIndexRange(
+        libXpertMassCore::IndexRange(0, polymer_size - 1));
     }
 
   // Make sure the sequence still has a number of residues
-  // compatible with the current m_coordinateList. this because the
+  // compatible with the current m_indexRangeCollection. this because the
   // sequence might be edited between opening this dialog window and
   // actually using it.
 
-  int polymerSize = mp_polymer->size();
-
-  for(int iter = 0; iter < m_coordinateList.size(); ++iter)
+  foreach(const libXpertMassCore::IndexRange *item,
+          m_indexRangeCollection.getRangesCstRef())
     {
-      // New coordinates instance we are iterating into.
-      libXpertMass::Coordinates *coordinates = m_coordinateList.at(iter);
-
-      if(coordinates->start() >= polymerSize ||
-         coordinates->end() >= polymerSize)
+      if(item->m_start >= polymer_size || item->m_stop >= polymer_size)
         {
           QMessageBox::warning(0,
-                               tr("massXpert - Compositions"),
-                               tr("Selection data are no more valid.\n"
+                               tr("MassXpert3 - Compositions"),
+                               tr("Sequence selection data are no more valid.\n"
                                   "Please update these data."),
                                QMessageBox::Ok);
           return false;
         }
     }
 
+  // Update the ionization status directly from the sequence editor window,
+  // which, in fact, will ask the Polymer its ionizer.
+
+  libXpertMassCore::Ionizer ionizer = msp_polymer->getIonizerCstRef();
+
   // We also want to know if the cross-links should be taken into account.
 
-  if(mp_calcOptions->monomerEntities() & libXpertMass::MONOMER_CHEMENT_CROSS_LINK)
+  if(static_cast<bool>(mref_calcOptions.getMonomerEntities() &
+                       libXpertMassCore::Enums::ChemicalEntity::CROSS_LINKER))
     {
       // If the whole sequence is dealt with, then by definition all
       // the cross-links are encompassed by the sequence.
@@ -247,26 +245,26 @@ CompositionsDlg::fetchValidateInputData()
         }
       else
         {
-          // We have to count the incomplete cross-links.
-
-          const libXpertMass::CrossLinkList &crossLinkList =
-            mp_polymer->crossLinkList();
 
           int crossLinkPartial = 0;
+          std::size_t in_count;
+          std::size_t out_count;
 
-          for(int iter = 0; iter < crossLinkList.size(); ++iter)
+          for(const libXpertMassCore::CrossLinkSPtr &cross_link_sp :
+              msp_polymer->getCrossLinksCstRef())
             {
-              libXpertMass::CrossLink *crossLink = crossLinkList.at(iter);
+              libXpertMassCore::Enums::CrossLinkEncompassed ret =
+                cross_link_sp->isEncompassedByIndexRangeCollection(
+                  m_indexRangeCollection, in_count, out_count);
 
-              int ret = crossLink->encompassedBy(m_coordinateList);
-
-              if(ret == libXpertMass::CROSS_LINK_ENCOMPASSED_FULL)
+              if(ret == libXpertMassCore::Enums::CrossLinkEncompassed::FULLY)
                 {
                   // 		qDebug() << __FILE__ << __LINE__
                   // 			  << "CrossLink at iter:" << iter
                   // 			  << "is fully encompassed";
                 }
-              else if(ret == libXpertMass::CROSS_LINK_ENCOMPASSED_PARTIAL)
+              else if(ret ==
+                      libXpertMassCore::Enums::CrossLinkEncompassed::PARTIALLY)
                 {
                   // 		qDebug() << __FILE__ << __LINE__
                   // 			  << "CrossLink at iter:" << iter
@@ -302,13 +300,13 @@ CompositionsDlg::monomericComposition()
   if(!fetchValidateInputData())
     {
       QMessageBox::warning(0,
-                           tr("massXpert - Compositions"),
+                           tr("MassXpert3 - Compositions"),
                            tr("Failed validating input data."),
                            QMessageBox::Ok);
       return;
     }
 
-  libXpertMass::Monomer *fakeMonomer = 0;
+  libXpertMassCore::Monomer *fakeMonomer = 0;
 
   // Empty the treeview. This will also remove all the monomer items
   // in the m_monomerList.
@@ -331,18 +329,20 @@ CompositionsDlg::monomericComposition()
       mpa_compositionProxyModel = 0;
     }
 
-  for(int iter = 0; iter < m_coordinateList.size(); ++iter)
-    {
-      // New coordinates instance we are iterating into.
-      libXpertMass::Coordinates *coordinates = m_coordinateList.at(iter);
+  qDebug() << "Determination of the monomeric composition for range:"
+           << m_indexRangeCollection.indicesAsText();
 
-      for(int jter = coordinates->start(); jter < coordinates->end() + 1;
-          ++jter)
+
+  foreach(const libXpertMassCore::IndexRange *item,
+          m_indexRangeCollection.getRangesCstRef())
+    {
+      for(qsizetype iter = item->m_start; iter < item->m_stop + 1; ++iter)
         {
           bool processed = false;
 
-          const libXpertMass::Monomer *iterMonomer = mp_polymer->at(jter);
-          Q_ASSERT(iterMonomer);
+          const libXpertMassCore::MonomerCstSPtr monomer_csp =
+            msp_polymer->getSequenceCstRef().getMonomerCstSPtrAt(iter);
+          Q_ASSERT(monomer_csp);
 
           // Check if monomer by same name is not already in our list of
           // fake monomers.
@@ -352,9 +352,9 @@ CompositionsDlg::monomericComposition()
               fakeMonomer = m_monomerList.at(kter);
               Q_ASSERT(fakeMonomer);
 
-              if(iterMonomer->name() == fakeMonomer->name())
+              if(monomer_csp->getName() == fakeMonomer->getName())
                 {
-                  libXpertMass::Prop *prop = fakeMonomer->prop("MONOMER_COUNT");
+                  libXpertMassCore::Prop *prop = fakeMonomer->prop("MONOMER_COUNT");
 
                   // The fake monomer MUST have a count prop !
                   Q_ASSERT(prop);
@@ -364,11 +364,12 @@ CompositionsDlg::monomericComposition()
                   ++(*count);
                   prop->setData(count);
 
-                  if(iterMonomer->isModified())
+                  if(monomer_csp->isModified())
                     {
                       // The monomer in the sequence is modified. Let the
                       // fake monomer know it.
-                      libXpertMass::Prop *prop = fakeMonomer->prop("MODIF_COUNT");
+                      libXpertMassCore::Prop *prop =
+                        fakeMonomer->prop("MODIF_COUNT");
 
                       // The fake monomer might not have a modif prop.
                       if(prop)
@@ -380,8 +381,8 @@ CompositionsDlg::monomericComposition()
                         }
                       else
                         {
-                          libXpertMass::IntProp *prop =
-                            new libXpertMass::IntProp("MODIF_COUNT", 1);
+                          libXpertMassCore::IntProp *prop =
+                            new libXpertMassCore::IntProp("MODIF_COUNT", 1);
                           fakeMonomer->appendProp(prop);
                         }
                     }
@@ -400,18 +401,19 @@ CompositionsDlg::monomericComposition()
 
           if(!processed)
             {
-              fakeMonomer = new libXpertMass::Monomer(*iterMonomer);
+              fakeMonomer = new libXpertMassCore::Monomer(*monomer_csp);
 
-              libXpertMass::IntProp *prop = new libXpertMass::IntProp("MONOMER_COUNT", 1);
+              libXpertMassCore::IntProp *prop =
+                new libXpertMassCore::IntProp("MONOMER_COUNT", 1);
               fakeMonomer->appendProp(prop);
 
-              if(iterMonomer->isModified())
+              if(monomer_csp->isModified())
                 {
                   // The monomer in the sequence is modified. Let the
                   // fake monomer know it.
 
-                  libXpertMass::IntProp *prop =
-                    new libXpertMass::IntProp("MODIF_COUNT", 1);
+                  libXpertMassCore::IntProp *prop =
+                    new libXpertMassCore::IntProp("MODIF_COUNT", 1);
                   fakeMonomer->appendProp(prop);
                 }
 
@@ -423,7 +425,7 @@ CompositionsDlg::monomericComposition()
 
       setupTreeView();
 
-      prepareResultsTxtString(TARGET_MONOMERIC);
+      prepareResultsTxtString(CompositionsTarget::MONOMERIC);
 
       setCursor(Qt::ArrowCursor);
     }
@@ -438,17 +440,18 @@ CompositionsDlg::elementalComposition()
   if(!fetchValidateInputData())
     {
       QMessageBox::warning(0,
-                           tr("massXpert - Compositions"),
+                           tr("MassXpert3 - Compositions"),
                            tr("Failed validating input data."),
                            QMessageBox::Ok);
       return;
     }
-
-  QString composition = mp_polymer->elementalComposition(
-    *mp_ionizeRule, m_coordinateList, *mp_calcOptions);
+  // We let the polymer use its own Ionizer below by not passing
+  // that argument.
+  QString composition = msp_polymer->elementalComposition(
+    m_indexRangeCollection, mref_calcOptions /*, polymer's ionizer*/);
   m_ui.elementalCompositionLineEdit->setText(composition);
 
-  prepareResultsTxtString(TARGET_ELEMENTAL);
+  prepareResultsTxtString(CompositionsTarget::ELEMENTAL);
 
   setCursor(Qt::ArrowCursor);
 }
@@ -507,7 +510,7 @@ CompositionsDlg::exportResults(int index)
 
 
 void
-CompositionsDlg::prepareResultsTxtString(int target)
+CompositionsDlg::prepareResultsTxtString(CompositionsTarget target)
 {
   mpa_resultsString->clear();
 
@@ -517,49 +520,46 @@ CompositionsDlg::prepareResultsTxtString(int target)
     "# Compositions: \n"
     "# -------------\n");
 
-  if(target == TARGET_ELEMENTAL)
+  if(target == CompositionsTarget::ELEMENTAL)
     {
       *mpa_resultsString += QObject::tr("\nIonization rule:\n");
-
       *mpa_resultsString +=
-        QObject::tr("Formula: %1 - ").arg(mp_ionizeRule->formula());
-
+        QObject::tr("Formula: %1 - ")
+          .arg(m_ionizer.getFormulaRef().getActionFormula());
       *mpa_resultsString +=
-        QObject::tr("Charge: %1 - ").arg(mp_ionizeRule->charge());
-
+        QObject::tr("Charge: %1 - ").arg(m_ionizer.getNominalCharge());
       *mpa_resultsString +=
-        QObject::tr("Level: %1\n").arg(mp_ionizeRule->level());
-
-
+        QObject::tr("Level: %1\n").arg(m_ionizer.getLevel());
       *mpa_resultsString += QObject::tr("\nCalculation options:\n");
 
-
-      bool withEntities =
-        (mp_calcOptions->monomerEntities() & libXpertMass::MONOMER_CHEMENT_MODIF);
+      bool with_modif_entities =
+        (static_cast<int>(mref_calcOptions.getMonomerEntities()) &
+         static_cast<int>(libXpertMassCore::Enums::ChemicalEntity::MODIF));
 
       // We want a delimited sequence with indication of the
       // different sequence regions for which the composition was
       // determined, thus the true below.
 
-      QString *sequence =
-        mp_polymer->monomerText(m_coordinateList, withEntities, true);
+      QString sequence = msp_polymer->getSequenceCstRef().getSequence(
+        m_indexRangeCollection, with_modif_entities, true);
 
-      *mpa_resultsString += *sequence;
+      *mpa_resultsString += sequence;
 
-      delete sequence;
-
-      if(withEntities)
+      if(with_modif_entities)
         *mpa_resultsString += QObject::tr("Account monomer modifs: yes\n");
       else
         *mpa_resultsString += QObject::tr("Account monomer modifs: no\n");
 
       // Left end and right end modifs
-      withEntities = (mp_calcOptions->polymerEntities() &
-                        libXpertMass::POLYMER_CHEMENT_LEFT_END_MODIF ||
-                      mp_calcOptions->polymerEntities() &
-                        libXpertMass::POLYMER_CHEMENT_RIGHT_END_MODIF);
+      with_modif_entities =
+        (static_cast<int>(mref_calcOptions.getPolymerEntities()) &
+           static_cast<int>(
+             libXpertMassCore::Enums::ChemicalEntity::LEFT_END_MODIF) ||
+         static_cast<int>(mref_calcOptions.getPolymerEntities()) &
+           static_cast<int>(
+             libXpertMassCore::Enums::ChemicalEntity::RIGHT_END_MODIF));
 
-      if(!withEntities)
+      if(!with_modif_entities)
         {
           *mpa_resultsString += QObject::tr("Account ends' modifs: no\n");
         }
@@ -568,28 +568,34 @@ CompositionsDlg::prepareResultsTxtString(int target)
           *mpa_resultsString += QObject::tr("Account ends' modifs: yes - ");
 
           // Left end modif
-          withEntities = (mp_calcOptions->polymerEntities() &
-                          libXpertMass::POLYMER_CHEMENT_LEFT_END_MODIF);
-          if(withEntities)
+          with_modif_entities =
+            (static_cast<int>(mref_calcOptions.getPolymerEntities()) &
+             static_cast<int>(
+               libXpertMassCore::Enums::ChemicalEntity::LEFT_END_MODIF));
+          if(with_modif_entities)
             {
-              *mpa_resultsString += QObject::tr("Left end modif: %1 - ")
-                                      .arg(mp_polymer->leftEndModif().name());
+              *mpa_resultsString +=
+                QObject::tr("Left end modif: %1 - ")
+                  .arg(msp_polymer->getLeftEndModifCstRef().getName());
             }
 
           // Right end modif
-          withEntities = (mp_calcOptions->polymerEntities() &
-                          libXpertMass::POLYMER_CHEMENT_RIGHT_END_MODIF);
-          if(withEntities)
+          with_modif_entities =
+            (static_cast<int>(mref_calcOptions.getPolymerEntities()) &
+             static_cast<int>(
+               libXpertMassCore::Enums::ChemicalEntity::RIGHT_END_MODIF));
+          if(with_modif_entities)
             {
-              *mpa_resultsString += QObject::tr("Right end modif: %1")
-                                      .arg(mp_polymer->leftEndModif().name());
+              *mpa_resultsString +=
+                QObject::tr("Right end modif: %1")
+                  .arg(msp_polymer->getLeftEndModifCstRef().getName());
             }
         }
 
       *mpa_resultsString += QObject::tr("\n\nElemental composition: %1")
                               .arg(m_ui.elementalCompositionLineEdit->text());
     }
-  else if(target == TARGET_MONOMERIC)
+  else if(target == CompositionsTarget::MONOMERIC)
     {
       CompositionTreeViewModel *model = static_cast<CompositionTreeViewModel *>(
         m_ui.compositionTreeView->model());
@@ -669,7 +675,7 @@ CompositionsDlg::exportResultsFile()
   if(!file.open(QIODevice::WriteOnly | QIODevice::Append))
     {
       QMessageBox::information(0,
-                               tr("massXpert - Export Data"),
+                               tr("MassXpert3 - Export Data"),
                                tr("Failed to open file in append mode."),
                                QMessageBox::Ok);
       return false;
@@ -704,6 +710,6 @@ CompositionsDlg::selectResultsFile()
 //////////////////////////////////// The results-exporting functions.
 //////////////////////////////////// The results-exporting functions.
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
