@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,9 @@
 #include <QSettings>
 
 
-/////////////////////// libmass includes
-#include <libXpertMass/PolChemDef.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/CrossLinker.hpp>
 
 
 /////////////////////// Local includes
@@ -48,31 +49,26 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-CrossLinkerDefDlg::CrossLinkerDefDlg(libXpertMass::PolChemDefSPtr polChemDefSPtr,
-                                     PolChemDefWnd *polChemDefWnd,
-                                     const QString &settingsFilePath,
-                                     const QString &applicationName,
-                                     const QString &description)
-  : AbstractPolChemDefDependentDlg(polChemDefSPtr,
-                                   polChemDefWnd,
-                                   settingsFilePath,
+CrossLinkerDefDlg::CrossLinkerDefDlg(
+  libXpertMassCore::PolChemDefSPtr pol_chem_def_sp,
+  PolChemDefWnd *pol_chem_def_wnd_p,
+  const QString &settings_file_path,
+  const QString &application_name,
+  const QString &description)
+  : AbstractPolChemDefDependentDlg(pol_chem_def_sp,
+                                   pol_chem_def_wnd_p,
+                                   settings_file_path,
                                    "CrossLinkerDefDlg",
-                                   applicationName,
-                                   description)
+                                   application_name,
+                                   description),
+    mref_crossLinkers(pol_chem_def_sp->getCrossLinkersRef())
 {
-  mp_list = polChemDefSPtr->crossLinkerListPtr();
-
   if(!initialize())
-    qFatal(
-      "Fatal error at %s@%d. Failed to initialize the %s window. Program "
-      "aborted.",
-      __FILE__,
-      __LINE__,
-      m_wndTypeName.toLatin1().data());
+    qFatal() << "Failed to initialize the dialog window.";
 }
 
 
@@ -87,7 +83,7 @@ CrossLinkerDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
   // close==destruction. Thus we only hide the dialog remembering its
   // position and size.
 
-  mp_polChemDefWnd->m_ui.crossLinkerPushButton->setChecked(false);
+  mp_pol_chem_def_wnd_p->m_ui.crossLinkerPushButton->setChecked(false);
 
   writeSettings();
 }
@@ -96,7 +92,7 @@ CrossLinkerDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
 void
 CrossLinkerDefDlg::readSettings()
 {
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -108,7 +104,7 @@ CrossLinkerDefDlg::readSettings()
 void
 CrossLinkerDefDlg::writeSettings()
 {
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   settings.setValue("geometry", saveGeometry());
@@ -122,18 +118,16 @@ CrossLinkerDefDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Now we need to actually show the window title (that element is empty in
   // m_ui)
   displayWindowTitle();
 
   // Set all the crossLinkers to the list widget.
 
-  for(int iter = 0; iter < mp_list->size(); ++iter)
-    {
-      libXpertMass::CrossLinker *crossLinker = mp_list->at(iter);
-
-      m_ui.crossLinkerListWidget->addItem(crossLinker->name());
-    }
+  for(libXpertMassCore::CrossLinkerSPtr cross_linker_sp : mref_crossLinkers)
+    m_ui.crossLinkerListWidget->addItem(cross_linker_sp->getName());
 
   readSettings();
 
@@ -217,14 +211,8 @@ CrossLinkerDefDlg::populateAvailableModifList()
   // is present in the cross-linker.
 
   m_ui.modifNameComboBox->addItem("");
-
-  for(int iter = 0; iter < msp_polChemDef->modifList().size(); ++iter)
-    {
-      libXpertMass::Modif *modif = msp_polChemDef->modifList().at(iter);
-      Q_ASSERT(modif);
-
-      m_ui.modifNameComboBox->addItem(modif->name());
-    }
+  for(libXpertMassCore::ModifSPtr modif_sp : msp_polChemDef->getModifsCstRef())
+    m_ui.modifNameComboBox->addItem(modif_sp->getName());
 
   return true;
 }
@@ -239,11 +227,12 @@ CrossLinkerDefDlg::addCrossLinkerPushButtonClicked()
   // Returns -1 if the list is empty.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
-  libXpertMass::CrossLinker *newCrossLinker = new libXpertMass::CrossLinker(
-    msp_polChemDef, tr("Type Name"), tr("Type Formula"));
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp =
+    std::make_shared<libXpertMassCore::CrossLinker>(
+      msp_polChemDef, "Type Name", "Type Formula", 0.0, 0.0);
 
-  mp_list->insert(index, newCrossLinker);
-  m_ui.crossLinkerListWidget->insertItem(index, newCrossLinker->name());
+  mref_crossLinkers.insert(mref_crossLinkers.begin() + index, cross_linker_sp);
+  m_ui.crossLinkerListWidget->insertItem(index, cross_linker_sp->getName());
 
   setModified();
 
@@ -254,7 +243,7 @@ CrossLinkerDefDlg::addCrossLinkerPushButtonClicked()
 
   m_ui.crossLinkerListWidget->setCurrentRow(index);
 
-  // Erase cleaveRule data that might be left over by precedent current
+  // Erase cleavageRule data that might be left over by precedent current
   // crossLinker.
   updateModifDetails(0);
 
@@ -279,9 +268,7 @@ CrossLinkerDefDlg::removeCrossLinkerPushButtonClicked()
   QListWidgetItem *item = m_ui.crossLinkerListWidget->takeItem(index);
   delete item;
 
-  libXpertMass::CrossLinker *crossLinker = mp_list->takeAt(index);
-  Q_ASSERT(crossLinker);
-  delete crossLinker;
+  mref_crossLinkers.erase(mref_crossLinkers.begin() + index);
 
   setModified();
 
@@ -322,11 +309,11 @@ CrossLinkerDefDlg::moveUpCrossLinkerPushButtonClicked()
   // Get the index of the crossLinker and the crossLinker itself.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
+  auto it = mref_crossLinkers.begin() + index;
   // If the item is already at top of list, do nothing.
-  if(!index)
+  if(it == mref_crossLinkers.begin())
     return;
-
-  mp_list->move(index, index - 1);
+  std::iter_swap(it, it - 1); // Swap with the previous element
 
   QListWidgetItem *item = m_ui.crossLinkerListWidget->takeItem(index);
 
@@ -354,11 +341,11 @@ CrossLinkerDefDlg::moveDownCrossLinkerPushButtonClicked()
   // Get the index of the crossLinker and the crossLinker itself.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
+  auto it = mref_crossLinkers.begin() + index;
   // If the item is already at bottom of list, do nothing.
-  if(index == m_ui.crossLinkerListWidget->count() - 1)
+  if(it == mref_crossLinkers.end())
     return;
-
-  mp_list->move(index, index + 1);
+  std::iter_swap(it, it + 1); // Swap with the next element
 
   QListWidgetItem *item = m_ui.crossLinkerListWidget->takeItem(index);
   m_ui.crossLinkerListWidget->insertItem(index + 1, item);
@@ -383,7 +370,7 @@ CrossLinkerDefDlg::addModifPushButtonClicked()
     {
       QMessageBox::information(
         this,
-        tr("massXpert - libXpertMass::CrossLinker definition"),
+        tr("MassXpert3 - libXpertMassCore::CrossLinker definition"),
         tr("Please, select a crossLinker first."),
         QMessageBox::Ok);
       return;
@@ -394,25 +381,26 @@ CrossLinkerDefDlg::addModifPushButtonClicked()
   int index = m_ui.crossLinkerListWidget->currentRow();
 
   // What's the actual crossLinker?
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
-  Q_ASSERT(crossLinker);
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Allocate the new modif.
-  libXpertMass::Modif *newModif =
-    new libXpertMass::Modif(msp_polChemDef, tr("Select modification"));
+  libXpertMassCore::ModifSPtr modif_sp = std::make_shared<libXpertMassCore::Modif>(
+    msp_polChemDef, "Select modification");
 
   // Get the row index of the current modif item. Returns -1 if the
   // list is empty.
   index = m_ui.modifListWidget->currentRow();
 
-  m_ui.modifListWidget->insertItem(index, newModif->name());
+  m_ui.modifListWidget->insertItem(index, modif_sp->getName());
 
-  // Needed so that the setCurrentRow() call below actually set the
+  // Needed so that the setCurrentRow() call below actually sets the
   // current row!
   if(index <= 0)
     index = 0;
 
-  crossLinker->modifList().insert(index, newModif);
+  std::vector<libXpertMassCore::ModifCstSPtr> modifs =
+    cross_linker_sp->getModifsRef();
+  modifs.insert(modifs.begin() + index, modif_sp);
 
   m_ui.modifListWidget->setCurrentRow(index);
 
@@ -431,13 +419,11 @@ CrossLinkerDefDlg::removeModifPushButtonClicked()
   if(selectedList.size() != 1)
     return;
 
-
   // Get the index of the current crossLinker so that we know from
   // which crossLinker we'll remove the modif.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
-  Q_ASSERT(crossLinker);
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Get the index of the current modif.
   index = m_ui.modifListWidget->currentRow();
@@ -450,13 +436,9 @@ CrossLinkerDefDlg::removeModifPushButtonClicked()
   delete item;
 
   // Remove the modif from the crossLinker proper.
-
-  QList<libXpertMass::Modif *> modifList = crossLinker->modifList();
-
-  libXpertMass::Modif *modif = modifList.at(index);
-
-  crossLinker->modifList().removeAt(index);
-  delete modif;
+  std::vector<MsXpS::libXpertMassCore::ModifCstSPtr> modifs =
+    cross_linker_sp->getModifsRef();
+  modifs.erase(modifs.begin() + index);
 
   // If there are remaining items, we want to set the next item the
   // currentItem. If not, then, the currentItem should be the one
@@ -496,21 +478,17 @@ CrossLinkerDefDlg::moveUpModifPushButtonClicked()
     return;
 
   // Get the crossLinker to which the modif belongs.
-  int index                         = m_ui.crossLinkerListWidget->currentRow();
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
+  int index = m_ui.crossLinkerListWidget->currentRow();
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Get the index of the current modif item.
   index = m_ui.modifListWidget->currentRow();
 
+  auto it = cross_linker_sp->getModifsRef().begin() + index;
   // If the item is already at top of list, do nothing.
-  if(!index)
+  if(it == cross_linker_sp->getModifsRef().begin())
     return;
-
-  // Get the modif itself from the crossLinker.
-  libXpertMass::Modif *modif = crossLinker->modifList().at(index);
-
-  crossLinker->modifList().removeAt(index);
-  crossLinker->modifList().insert(index - 1, modif);
+  std::iter_swap(it, it - 1);
 
   QListWidgetItem *item = m_ui.modifListWidget->takeItem(index);
   m_ui.modifListWidget->insertItem(index - 1, item);
@@ -534,21 +512,17 @@ CrossLinkerDefDlg::moveDownModifPushButtonClicked()
     return;
 
   // Get the crossLinker to which the modif belongs.
-  int index                         = m_ui.crossLinkerListWidget->currentRow();
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
+  int index = m_ui.crossLinkerListWidget->currentRow();
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Get the index of the current modif item.
   index = m_ui.modifListWidget->currentRow();
 
-  // If the item is already at top of list, do nothing.
-  if(index == m_ui.modifListWidget->count() - 1)
+  auto it = cross_linker_sp->getModifsRef().begin() + index;
+  // If the item is already at bottom of list, do nothing.
+  if(it == cross_linker_sp->getModifsRef().end())
     return;
-
-  // Get the modif itself from the crossLinker.
-  libXpertMass::Modif *modif = crossLinker->modifList().at(index);
-
-  crossLinker->modifList().removeAt(index);
-  crossLinker->modifList().insert(index + 1, modif);
+  std::iter_swap(it, it + 1);
 
   QListWidgetItem *item = m_ui.modifListWidget->takeItem(index);
   m_ui.modifListWidget->insertItem(index + 1, item);
@@ -575,14 +549,12 @@ CrossLinkerDefDlg::applyCrossLinkerPushButtonClicked()
   // Get the index of the current crossLinker item.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
-  // We do not want more than one crossLinker by the same name or the same
-  // symbol.
+  // We do not want more than one crossLinker by the same name.
 
   bool wasEditFormulaEmpty = false;
 
-
   QString editName    = m_ui.nameLineEdit->text();
   QString editFormula = m_ui.formulaLineEdit->text();
   if(editFormula.isEmpty())
@@ -596,12 +568,20 @@ CrossLinkerDefDlg::applyCrossLinkerPushButtonClicked()
   // crossLinker is not the one that is current in the crossLinker list,
   // then we are making a double entry, which is not allowed.
 
-  int nameRes = libXpertMass::CrossLinker::isNameInList(editName, *mp_list);
-  if(nameRes != -1 && nameRes != index)
+  std::vector<libXpertMassCore::CrossLinkerSPtr>::iterator the_iterator =
+    std::find_if(
+      mref_crossLinkers.begin(),
+      mref_crossLinkers.end(),
+      [editName](const libXpertMassCore::CrossLinkerSPtr iter_cross_linker_sp) {
+        return iter_cross_linker_sp->getName() == editName;
+      });
+
+  if(the_iterator != mref_crossLinkers.end() &&
+     std::distance(mref_crossLinkers.begin(), the_iterator) != index)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
-                           tr("A crossLinker with same name "
+                           tr("MassXpert3 - CrossLinker definition"),
+                           tr("A cross-linker with same name "
                               "exists already."),
                            QMessageBox::Ok);
       return;
@@ -611,28 +591,29 @@ CrossLinkerDefDlg::applyCrossLinkerPushButtonClicked()
   // modification. This is because the name of the crossLinker cannot
   // clash with the name of a modif when setting the graphical
   // vignette of the crossLinker.
-  if(msp_polChemDef->referenceModifByName(editName))
+  if(msp_polChemDef->getModifIndexByName(editName) != -1)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
+                           tr("MassXpert3 - CrossLinker definition"),
                            tr("The name of the cross-linker is already "
                               "used by a modification."),
                            QMessageBox::Ok);
       return;
     }
 
-  libXpertMass::Formula formula(editFormula);
+  libXpertMassCore::Formula formula(editFormula);
 
-  libXpertMass::IsotopicDataSPtr isotopic_data_sp =
+  libXpertMassCore::IsotopicDataSPtr isotopic_data_sp =
     msp_polChemDef->getIsotopicDataSPtr();
 
-  if(!formula.validate(isotopic_data_sp))
+  libXpertMassCore::ErrorList error_list;
+  if(!formula.validate(isotopic_data_sp, &error_list))
     {
       if(wasEditFormulaEmpty)
         {
           QMessageBox::warning(
             this,
-            tr("massXpert - libXpertMass::CrossLinker definition"),
+            tr("MassXpert3 - libXpertMassCore::CrossLinker definition"),
             tr("The formula is empty, please enter a formula "
                "like '-H+H' if no reaction is required."),
             QMessageBox::Ok);
@@ -641,8 +622,9 @@ CrossLinkerDefDlg::applyCrossLinkerPushButtonClicked()
         {
           QMessageBox::warning(
             this,
-            tr("massXpert - libXpertMass::CrossLinker definition"),
-            tr("The formula failed to validate."),
+            tr("MassXpert3 - libXpertMassCore::CrossLinker definition"),
+            QString("The formula failed to validate with errors:\n%1\n.")
+              .arg(libXpertMassCore::Utils::joinErrorList(error_list)),
             QMessageBox::Ok);
         }
 
@@ -651,13 +633,13 @@ CrossLinkerDefDlg::applyCrossLinkerPushButtonClicked()
 
   // Finally we can update the crossLinker's data:
 
-  crossLinker->setName(editName);
-  crossLinker->setFormula(editFormula);
+  cross_linker_sp->setName(editName);
+  cross_linker_sp->setFormula(editFormula);
 
   // Update the list widget item.
 
   QListWidgetItem *item = m_ui.crossLinkerListWidget->currentItem();
-  item->setData(Qt::DisplayRole, crossLinker->name());
+  item->setData(Qt::DisplayRole, cross_linker_sp->getName());
 
   setModified();
 }
@@ -676,29 +658,31 @@ CrossLinkerDefDlg::applyModifPushButtonClicked()
     return;
 
   // Get the crossLinker to which the modif belongs.
-  int index                         = m_ui.crossLinkerListWidget->currentRow();
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
+  int index = m_ui.crossLinkerListWidget->currentRow();
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Get the index of the current modif item.
   index = m_ui.modifListWidget->currentRow();
 
   // Get the modif itself from the crossLinker.
-  libXpertMass::Modif *crosslinker_modif = crossLinker->modifList().at(index);
+  libXpertMassCore::ModifCstSPtr modif_csp =
+    cross_linker_sp->getModifsCstRef().at(index);
 
-  QString modifName = m_ui.modifNameComboBox->currentText();
+  QString modif_name = m_ui.modifNameComboBox->currentText();
 
   // Make sure the modification is known to the polymer chemistry
   // definition.
 
-  libXpertMass::Modif reference_modif =
-    libXpertMass::Modif(msp_polChemDef, "NOT_SET", "NOT_SET");
+  libXpertMassCore::Modif reference_modif =
+    libXpertMassCore::Modif(msp_polChemDef, "NOT_SET", "NOT_SET");
 
-  // Populate reference_modif with the data in the modification by the same name
-  // in the reference list belonging to the polymer chemistry definition.
-  if(!msp_polChemDef->referenceModifByName(modifName, &reference_modif))
+  libXpertMassCore::ModifCstSPtr reference_modif_csp =
+    msp_polChemDef->getModifCstSPtrByName(modif_name);
+
+  if(reference_modif_csp == nullptr)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
+                           tr("MassXpert3 - CrossLinker definition"),
                            tr("The modification is not known to the "
                               "polymer chemistry definition."),
                            QMessageBox::Ok);
@@ -707,12 +691,12 @@ CrossLinkerDefDlg::applyModifPushButtonClicked()
 
   // Copy the reference modification to the modification that sits inside of the
   // CrossLinker being defined.
-  *crosslinker_modif = reference_modif;
+  modif_csp = reference_modif_csp;
 
   // Update the list widget item.
 
   QListWidgetItem *item = m_ui.modifListWidget->currentItem();
-  item->setData(Qt::DisplayRole, crosslinker_modif->name());
+  item->setData(Qt::DisplayRole, modif_csp->getName());
 
   setModified();
 }
@@ -728,58 +712,65 @@ CrossLinkerDefDlg::validatePushButtonClicked()
   // everything is fine and that colinearity is perfect between the
   // crossLinker list and the listwidget.
 
-  int itemCount = m_ui.crossLinkerListWidget->count();
+  std::size_t item_count = m_ui.crossLinkerListWidget->count();
 
-  if(itemCount != mp_list->size())
+  if(item_count != mref_crossLinkers.size())
     {
       errorList << QString(
-        tr("\nThe number of crossLinkers in the "
+        tr("\nThe number of cross-linkers in the "
            "list widget \n"
-           "and in the list of crossLinkers "
+           "and in the list of cross-linkers "
            "is not identical.\n"));
 
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
+                           tr("MassXpert3 - CrossLinker definition"),
                            errorList.join("\n"),
                            QMessageBox::Ok);
       return false;
     }
 
-  for(int iter = 0; iter < mp_list->size(); ++iter)
+  for(std::size_t iter = 0; iter < mref_crossLinkers.size(); ++iter)
     {
       QListWidgetItem *item = m_ui.crossLinkerListWidget->item(iter);
 
-      libXpertMass::CrossLinker *crossLinker = mp_list->at(iter);
+      libXpertMassCore::CrossLinkerSPtr cross_linker_sp =
+        mref_crossLinkers.at(iter);
 
-      if(item->text() != crossLinker->name())
+      if(item->text() != cross_linker_sp->getName())
         errorList << QString(tr("\nCrossLinker at index %1 has not the same\n"
                                 "name as the list widget item at the\n"
                                 "same index.\n")
                                .arg(iter));
 
-      if(!crossLinker->validate())
-        errorList << QString(
-          tr("\nCrossLinker at index %1 failed "
-             "to validate.\n"
-             "Please, make sure that there are either no"
-             " or at least two modifications for the cross-link.\n")
-            .arg(iter));
+      MsXpS::libXpertMassCore::ErrorList error_list;
+
+      if(!cross_linker_sp->validate(&error_list))
+        errorList
+          << QString(
+               "\nCrossLinker at index %1 failed "
+               "to validate.\n"
+               "Errors: %2\n\nPlease, make sure that there are either no"
+               " or at least two modifications for the cross-link.\n")
+               .arg(iter)
+               .arg(libXpertMassCore::Utils::joinErrorList(error_list));
     }
 
   if(errorList.size())
     {
-      QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
-                           errorList.join("\n"),
-                           QMessageBox::Ok);
+      QMessageBox::warning(
+        this,
+        tr("MassXpert3 - libXpertMassCore::CrossLinker definition"),
+        errorList.join("\n"),
+        QMessageBox::Ok);
       return false;
     }
   else
     {
-      QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::CrossLinker definition"),
-                           ("Validation: success\n"),
-                           QMessageBox::Ok);
+      QMessageBox::warning(
+        this,
+        tr("MassXpert3 - libXpertMassCore::CrossLinker definition"),
+        ("Validation: success\n"),
+        QMessageBox::Ok);
     }
 
   return true;
@@ -803,21 +794,16 @@ CrossLinkerDefDlg::crossLinkerListWidgetItemSelectionChanged()
   // Get the index of the current crossLinker.
   int index = m_ui.crossLinkerListWidget->currentRow();
 
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
-  Q_ASSERT(crossLinker);
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Set the data of the crossLinker to their respective widgets.
-  updateCrossLinkerIdentityDetails(crossLinker);
+  updateCrossLinkerIdentityDetails(cross_linker_sp);
 
   // The list of modifs
   m_ui.modifListWidget->clear();
 
-  for(int iter = 0; iter < crossLinker->modifList().size(); ++iter)
-    {
-      libXpertMass::Modif *modif = crossLinker->modifList().at(iter);
-
-      m_ui.modifListWidget->addItem(modif->name());
-    }
+  for(libXpertMassCore::ModifCstSPtr modif_csp : cross_linker_sp->getModifsCstRef())
+    m_ui.modifListWidget->addItem(modif_csp->getName());
 
   if(!m_ui.modifListWidget->count())
     updateModifDetails(0);
@@ -845,30 +831,29 @@ CrossLinkerDefDlg::modifListWidgetItemSelectionChanged()
   int index = m_ui.crossLinkerListWidget->currentRow();
 
   // Find the modif object in the list of modifs.
-  libXpertMass::CrossLinker *crossLinker = mp_list->at(index);
-  Q_ASSERT(crossLinker);
+  libXpertMassCore::CrossLinkerSPtr cross_linker_sp = mref_crossLinkers.at(index);
 
   // Get the index of the current modif.
   index = m_ui.modifListWidget->currentRow();
 
   // Get the modif that is currently selected from the crossLinker's list
   // of modifs.
-  libXpertMass::Modif *modif = crossLinker->modifList().at(index);
-  Q_ASSERT(modif);
+  libXpertMassCore::ModifCstSPtr modif_csp =
+    cross_linker_sp->getModifsCstRef().at(index);
 
   // Set the data of the modif to their respective widgets.
-  updateModifDetails(modif);
+  updateModifDetails(modif_csp);
 }
 
 
 void
 CrossLinkerDefDlg::updateCrossLinkerIdentityDetails(
-  libXpertMass::CrossLinker *crossLinker)
+  libXpertMassCore::CrossLinkerSPtr cross_link_sp)
 {
-  if(crossLinker)
+  if(cross_link_sp != nullptr)
     {
-      m_ui.nameLineEdit->setText(crossLinker->name());
-      m_ui.formulaLineEdit->setText(crossLinker->formula());
+      m_ui.nameLineEdit->setText(cross_link_sp->getName());
+      m_ui.formulaLineEdit->setText(cross_link_sp->getFormula());
     }
   else
     {
@@ -879,11 +864,11 @@ CrossLinkerDefDlg::updateCrossLinkerIdentityDetails(
 
 
 void
-CrossLinkerDefDlg::updateModifDetails(libXpertMass::Modif *modif)
+CrossLinkerDefDlg::updateModifDetails(libXpertMassCore::ModifCstSPtr modif_csp)
 {
-  if(modif)
+  if(modif_csp != nullptr)
     {
-      int index = m_ui.modifNameComboBox->findText(modif->name());
+      int index = m_ui.modifNameComboBox->findText(modif_csp->getName());
 
       // The name might be tr("Select modif") if the modif was just
       // created(see the function to add a modif to a
@@ -921,6 +906,6 @@ CrossLinkerDefDlg::validate()
   return validatePushButtonClicked();
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
