@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,20 +31,31 @@
  */
 
 
+/////////////////////// stdlib includes
+
+
 /////////////////////// Qt includes
 #include <QDebug>
 #include <QHeaderView>
 #include <QMouseEvent>
 #include <QMessageBox>
 #include <QDrag>
+#include <QMenu>
 
-#include <pappsomspp/utils.h>
 
-#include <libXpertMass/Oligomer.hpp>
+/////////////////////// pappsomspp includes
+#include <pappsomspp/core/utils.h>
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Oligomer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
 
 /////////////////////// Local includes
 #include "../nongui/globals.hpp"
-#include "../nongui/FragmentOligomer.hpp"
 #include "FragmentOligomerTableView.hpp"
 #include "FragmentOligomerTableViewSortProxyModel.hpp"
 #include "FragmentationDlg.hpp"
@@ -54,7 +65,7 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -105,16 +116,17 @@ FragmentOligomerTableView::~FragmentOligomerTableView()
 
 
 void
-FragmentOligomerTableView::setOligomerList(OligomerList *oligomerList)
+FragmentOligomerTableView::setOligomerCollection(
+  libXpertMassCore::OligomerCollection *oligomers)
 {
-  mp_oligomerList = oligomerList;
+  mp_oligomers = oligomers;
 }
 
 
-const OligomerList *
-FragmentOligomerTableView::oligomerList()
+const libXpertMassCore::OligomerCollection *
+FragmentOligomerTableView::getOligomers()
 {
-  return mp_oligomerList;
+  return mp_oligomers;
 }
 
 
@@ -131,28 +143,28 @@ FragmentOligomerTableView::setParentDlg(FragmentationDlg *dlg)
   mp_parentDlg = dlg;
 }
 
-
+// The oligomers OligomerCollection might not be empty and
+// index indicates at what index to insert the currently
+// selected oligomers. If index = -1, then the oligomers
+// need to be simply appended.
 int
-FragmentOligomerTableView::selectedOligomers(OligomerList *oligomerList,
-                                             int index) const
+FragmentOligomerTableView::selectedOligomers(
+  libXpertMassCore::OligomerCollection &oligomers, int index) const
 {
-  if(!oligomerList)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-
   int count = 0;
 
-  int localIndex = 0;
+  int oligomers_insertion_index = 0;
 
   // How many oligomers are there in the list passed as argument?
-  int oligomerCount = oligomerList->size();
+  int oligomerCount = oligomers.size();
 
   if(index > oligomerCount)
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
 
   // If index is -1 , then we are asked to append the oligomers to
-  // the list.
+  // the list (that is, to insert past the last item).
   if(index == -1)
-    localIndex = oligomerList->size();
+    oligomers_insertion_index = oligomers.size();
 
   // For each selected oligomer, duplicate it and append to the list
   // passed as argument.
@@ -227,57 +239,60 @@ FragmentOligomerTableView::selectedOligomers(OligomerList *oligomerList,
       row_text += local_index.data().toString();
       row_text += " ";
 
-      // We absolutely need to allocate a FragmentOligomer because only that
-      // class has the ability to provide the correct elemental composition.
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersRef().at(row);
 
-      FragmentOligomerSPtr new_oligomer_sp = std::make_shared<FragmentOligomer>(
-        *(std::dynamic_pointer_cast<FragmentOligomer>(
-          mp_oligomerList->at(row))));
+      libXpertMassCore::OligomerSPtr new_oligomer_sp =
+        std::make_shared<libXpertMassCore::Oligomer>(*oligomer_sp);
 
-      row_text += new_oligomer_sp->elementalComposition();
+      row_text += new_oligomer_sp->getFormulaCstRef().getActionFormula();
       row_text += "\n";
 
       // Create a NoDeletePointerProp, which might be used later by
       // the user of the list of oligomers to highlight regions in
       // the sequence editor.
 
-      libXpertMass::NoDeletePointerProp *prop = new libXpertMass::NoDeletePointerProp(
-        "SEQUENCE_EDITOR_WND", static_cast<void *>(mp_parentDlg->editorWnd()));
+      libXpertMassCore::NoDeletePointerProp *prop =
+        new libXpertMassCore::NoDeletePointerProp(
+          "SEQUENCE_EDITOR_WND",
+          static_cast<void *>(mp_parentDlg->editorWnd()));
 
       new_oligomer_sp->appendProp(prop);
 
-      oligomerList->insert(localIndex, new_oligomer_sp);
+      oligomers.getOligomersRef().insert(oligomers.getOligomersRef().begin() +
+                                           oligomers_insertion_index,
+                                         new_oligomer_sp);
 
-      ++localIndex;
+      ++oligomers_insertion_index;
       ++count;
     }
 
   // For debugging.
   // qDebug().noquote() << row_text;
-  //pappso::Utils::writeToFile(row_text,
-                             //QString("/tmp/mass/rows-%1.txt")
-                               //.arg(QDateTime::currentDateTime().toString()));
+  // pappso::Utils::writeToFile(row_text,
+  // QString("/tmp/mass/rows-%1.txt")
+  //.arg(QDateTime::currentDateTime().toString()));
 
   return count;
 }
 
 
 QString *
-FragmentOligomerTableView::selectedOligomersAsPlainText(
+FragmentOligomerTableView::describeOligomerAsPlainText(
   QString delimiter,
   bool withSequence,
   bool forXpertMiner,
-  libXpertMass::MassType massType) const
+  libXpertMassCore::Enums::MassType massType) const
 {
   // Let's get all the currently selected oligomers in one list.
-  OligomerList oligomerList;
+  libXpertMassCore::OligomerCollection oligomers;
 
   // Append the selected oligomers to the empty list.
-  int appendedOligomerCount = selectedOligomers(&oligomerList, -1);
+  std::size_t appended_oligomer_count = selectedOligomers(oligomers, -1);
 
   // Sanity check
-  if(appendedOligomerCount != oligomerList.size())
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+  if(appended_oligomer_count != oligomers.size())
+    qFatal() << "Programming error.";
 
   // If delimiter is empty, then set to "$".
   if(delimiter.isEmpty())
@@ -286,71 +301,92 @@ FragmentOligomerTableView::selectedOligomersAsPlainText(
   // Allocate a string in which we describe all the selected items.
 
   // For export to XpertMiner, we only want the masses asked for:
-  // MassType::MASS_MONO or MASS_AVG. Also, we want the format to be :
+  // MassType::MONO or MASS_AVG. Also, we want the format to be :
 
   // mass <delim> charge <delim> name <delim> coords
 
   QString *text = new QString();
 
-  while(!oligomerList.isEmpty())
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_iterator =
+    oligomers.getOligomersRef().begin();
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_end_iterator =
+    oligomers.getOligomersRef().end();
+
+  while(the_iterator != the_end_iterator)
     {
-      libXpertMass::OligomerSPtr oligomer_sp = oligomerList.takeFirst();
-
       if(!forXpertMiner)
-        *text +=
-          QString("\n%1%2").arg(oligomer_sp->description()).arg(delimiter);
+        *text += QString("\n%1%2")
+                   .arg((*the_iterator)->getDescription())
+                   .arg(delimiter);
 
-      if(forXpertMiner && massType == libXpertMass::MassType::MASS_AVG)
+      if(forXpertMiner && massType == libXpertMassCore::Enums::MassType::AVG)
         {
         }
       else
         {
-          *text += QString("%1%2")
-                     .arg(oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES))
-                     .arg(delimiter);
+          *text +=
+            QString("%1%2")
+              .arg(
+                (*the_iterator)->getMass(libXpertMassCore::Enums::MassType::MONO),
+                0,
+                'f',
+                libXpertMassCore::OLIGOMER_DEC_PLACES)
+              .arg(delimiter);
         }
 
-      if(forXpertMiner && massType == libXpertMass::MassType::MASS_MONO)
+      if(forXpertMiner && massType == libXpertMassCore::Enums::MassType::MONO)
         {
         }
       else
         {
-          *text += QString("%1%2")
-                     .arg(oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES))
-                     .arg(delimiter);
+          *text +=
+            QString("%1%2")
+              .arg((*the_iterator)->getMass(libXpertMassCore::Enums::MassType::AVG),
+                   0,
+                   'f',
+                   libXpertMassCore::OLIGOMER_DEC_PLACES)
+              .arg(delimiter);
         }
 
-      *text += QString("%1%2").arg(oligomer_sp->charge()).arg(delimiter);
-
-      *text += QString("%1%2").arg(oligomer_sp->name()).arg(delimiter);
-
       *text += QString("%1%2")
-                 .arg(static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())
-                        ->positionsAsText())
+                 .arg((*the_iterator)->getIonizerCstRef().charge())
+                 .arg(delimiter);
+
+      *text += QString("%1%2").arg((*the_iterator)->getName()).arg(delimiter);
+
+      *text += QString("%1%2")
+                 .arg((*the_iterator)
+                        ->getCalcOptionsCstRef()
+                        .getIndexRangeCollectionCstRef()
+                        .positionsAsText())
                  .arg(delimiter);
 
       if(!forXpertMiner)
-        *text += QString("%1%2").arg(oligomer_sp->isModified()).arg(delimiter);
+        *text +=
+          QString("%1%2").arg((*the_iterator)->isModified()).arg(delimiter);
 
       // We cannot export the sequence if data are for XpertMiner
       if(!forXpertMiner && withSequence)
         {
-          QString *sequence = oligomer_sp->monomerText();
+          QString sequence = (*the_iterator)
+                               ->getPolymerCstSPtr()
+                               ->getSequenceCstRef()
+                               .getSequence((*the_iterator)
+                                              ->getCalcOptionsCstRef()
+                                              .getIndexRangeCollectionCstRef());
 
-          *text += QString("\n%1").arg(*sequence);
-
-          delete sequence;
+          *text += QString("\n%1").arg(sequence);
         }
 
       // Terminate the stanza
       *text += QString("\n");
 
-
-      // We can now delete the allocated oligomer, since we do not
-      // need it anymore.
-      oligomer_sp.reset();
+      ++the_iterator;
     }
 
+  // At this point, just clear the oligomers.
+  oligomers.clear();
+
   // Terminate the string with a new line.
   *text += QString("\n");
 
@@ -425,18 +461,25 @@ FragmentOligomerTableView::currentChanged(const QModelIndex &current,
   // tableView (that list actually belongs to the FragmentationDlg
   // instance.
 
-  libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersCstRef().at(row);
 
   // If the oligomers obtained with the fragmentation are old and
   // the sequence has been changed since the fragmentation, then the
   // oligomers might point to a sequence element that is no more. We
   // want to avoid such kind of errors.
 
-  if(oligomer_sp->startIndex() >= oligomer_sp->polymer()->size() ||
-     oligomer_sp->endIndex() >= oligomer_sp->polymer()->size())
+  if(oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .leftMostIndexRangeStart() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+     oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .rightMostIndexRangeStop() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - Fragmentation"),
+                           tr("MassXpert3 - Fragmentation"),
                            tr("%1@%2\n"
                               "The monomer indices do not correspond "
                               "to a valid polymer sequence range.\n"
@@ -449,24 +492,23 @@ FragmentOligomerTableView::currentChanged(const QModelIndex &current,
       return;
     }
 
-  QString *text = oligomer_sp->polymer()->monomerText(
-    oligomer_sp->startIndex(), oligomer_sp->endIndex(), true);
+  QString text =
+    oligomer_sp->getPolymerCstSPtr()->getSequenceCstRef().getSequence(
+      oligomer_sp->getCalcOptionsCstRef().getIndexRangeCollectionCstRef());
 
   // We are getting text for an oligomer; it cannot be empty,
   // because that would mean the oligomer has no monomers. In that
   // case it is not conceivable that the oligomer be in the
   // fragmentation product list.
 
-  Q_ASSERT(!text->isEmpty());
+  Q_ASSERT(!text.isEmpty());
 
-  mp_parentDlg->updateOligomerSequence(text);
-
-  delete text;
+  mp_parentDlg->updateOligomerSequence(&text);
 
   // Get the mass calculation engine's options out of the oligomer,
   // so that we can display them correctly.
 
-  libXpertMass::CalcOptions calcOptions = oligomer_sp->calcOptions();
+  libXpertMassCore::CalcOptions calcOptions(oligomer_sp->getCalcOptionsCstRef());
 
   mp_parentDlg->updateFragmentationDetails(calcOptions);
 
@@ -491,7 +533,8 @@ FragmentOligomerTableView::itemActivated(const QModelIndex &index)
   // tableView (that list actually belongs to the FragmentationDlg
   // instance.
 
-  libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersRef().at(row);
 
   SequenceEditorWnd *editorWnd = mp_parentDlg->editorWnd();
 
@@ -503,11 +546,17 @@ FragmentOligomerTableView::itemActivated(const QModelIndex &index)
   // qDebug() << __FILE__ << __LINE__
   //          << "Elemental composition: " << formula;
 
-  if(oligomer_sp->startIndex() >= oligomer_sp->polymer()->size() ||
-     oligomer_sp->endIndex() >= oligomer_sp->polymer()->size())
+  if(oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .leftMostIndexRangeStart() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+     oligomer_sp->getCalcOptionsCstRef()
+         .getIndexRangeCollectionCstRef()
+         .rightMostIndexRangeStop() >=
+       (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - Fragmentation"),
+                           tr("MassXpert3 - Fragmentation"),
                            tr("%1@%2\n"
                               "The monomer indices do not correspond "
                               "to a valid polymer sequence range.\n"
@@ -521,7 +570,14 @@ FragmentOligomerTableView::itemActivated(const QModelIndex &index)
     }
 
   editorWnd->mpa_editorGraphicsView->setSelection(
-    oligomer_sp->startIndex(), oligomer_sp->endIndex(), false, false);
+    oligomer_sp->getCalcOptionsCstRef()
+      .getIndexRangeCollectionCstRef()
+      .leftMostIndexRangeStart(),
+    oligomer_sp->getCalcOptionsCstRef()
+      .getIndexRangeCollectionCstRef()
+      .rightMostIndexRangeStop(),
+    false,
+    false);
 }
 
 
@@ -530,21 +586,21 @@ FragmentOligomerTableView::itemActivated(const QModelIndex &index)
 void
 FragmentOligomerTableView::copyMono()
 {
-  return copyMassList(libXpertMass::MassType::MASS_MONO);
+  return copyMassList(libXpertMassCore::Enums::MassType::MONO);
 }
 
 
 void
 FragmentOligomerTableView::copyAvg()
 {
-  return copyMassList(libXpertMass::MassType::MASS_AVG);
+  return copyMassList(libXpertMassCore::Enums::MassType::AVG);
 }
 
 
 void
-FragmentOligomerTableView::copyMassList(int monoOrAvg)
+FragmentOligomerTableView::copyMassList(libXpertMassCore::Enums::MassType mass_type)
 {
-  QString massList;
+  QString text;
 
   // We want to prepare a textual list of masses (either MONO or
   // AVG) of all the oligomers in the tableview, exactly as they are
@@ -568,28 +624,36 @@ FragmentOligomerTableView::copyMassList(int monoOrAvg)
 
       // qDebug() << __FILE__ << __LINE__
       //          << "sourceRow:" << sourceRow;
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersRef().at(sourceRow);
 
-      libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(sourceRow);
-
-      if(monoOrAvg == libXpertMass::MassType::MASS_MONO)
-        massList += oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
-      else if(monoOrAvg == libXpertMass::MassType::MASS_AVG)
-        massList += oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
+      if(mass_type == libXpertMassCore::Enums::MassType::MONO)
+        text += QString("%1").arg(
+          oligomer_sp->getMass(libXpertMassCore::Enums::MassType::MONO),
+          0,
+          'f',
+          libXpertMassCore::OLIGOMER_DEC_PLACES);
+      else if(mass_type == libXpertMassCore::Enums::MassType::AVG)
+        text += QString("%1").arg(
+          oligomer_sp->getMass(libXpertMassCore::Enums::MassType::AVG),
+          0,
+          'f',
+          libXpertMassCore::OLIGOMER_DEC_PLACES);
       else
-        qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+        qFatal() << "Programming error.";
 
       // End the mass item with a new line.
-      massList += "\n";
+      text += "\n";
     }
 
-  if(massList.isEmpty())
+  if(text.isEmpty())
     return;
 
   QClipboard *clipboard = QApplication::clipboard();
 
-  clipboard->setText(massList, QClipboard::Clipboard);
+  clipboard->setText(text, QClipboard::Clipboard);
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
