@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,25 +36,24 @@
 
 /////////////////////// Local includes
 #include "FragmentOligomerTableViewModel.hpp"
-#include <libXpertMass/Oligomer.hpp>
-#include "../nongui/FragmentOligomer.hpp"
+#include <MsXpS/libXpertMassCore/Oligomer.hpp>
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
 FragmentOligomerTableViewModel::FragmentOligomerTableViewModel(
-  OligomerList *oligomerList, QObject *parent)
+  libXpertMassCore::OligomerCollection *oligomers_p, QObject *parent)
   : QAbstractTableModel(parent)
 {
-  if(!oligomerList)
+  if(!oligomers_p)
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
 
-  mp_oligomerList = oligomerList;
+  mp_oligomers = oligomers_p;
 
   if(!parent)
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
@@ -71,10 +70,10 @@ FragmentOligomerTableViewModel::~FragmentOligomerTableViewModel()
 }
 
 
-const OligomerList *
+const libXpertMassCore::OligomerCollection *
 FragmentOligomerTableViewModel::oligomerList()
 {
-  return mp_oligomerList;
+  return mp_oligomers;
 }
 
 
@@ -101,7 +100,7 @@ FragmentOligomerTableViewModel::rowCount(const QModelIndex &parent) const
 {
   Q_UNUSED(parent);
 
-  return mp_oligomerList->size();
+  return mp_oligomers->size();
 }
 
 
@@ -177,10 +176,10 @@ FragmentOligomerTableViewModel::data(const QModelIndex &index, int role) const
       // row. Let's find the row first, so that we get to the proper
       // oligomer.
 
-      // libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+      // libXpertMassCore::OligomerSPtr oligomer_sp = mp_oligomers->at(row);
 
-      FragmentOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<FragmentOligomer>(mp_oligomerList->at(row));
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersCstRef().at(row);
 
       // Now see what's the column that is asked for. Prepare a
       // string that we'll feed with the right data before returning
@@ -190,45 +189,49 @@ FragmentOligomerTableViewModel::data(const QModelIndex &index, int role) const
 
       if(column == FRAGMENT_OLIGO_PATTERN_COLUMN)
         {
-          valueString = oligomer_sp->description();
+          valueString = oligomer_sp->getDescription();
         }
       else if(column == FRAGMENT_OLIGO_NAME_COLUMN)
         {
-          valueString = oligomer_sp->name();
+          valueString = oligomer_sp->getName();
         }
       else if(column == FRAGMENT_OLIGO_COORDINATES_COLUMN)
         {
-          int oligomerCount =
-            static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())->size();
-
-          for(int jter = 0; jter < oligomerCount; ++jter)
-            {
-              libXpertMass::Coordinates *coordinates =
-                static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())
-                  ->at(jter);
-
-              valueString += QString("[%1-%2]")
-                               .arg(coordinates->start() + 1)
-                               .arg(coordinates->end() + 1);
-            }
+          foreach(const libXpertMassCore::IndexRange *item,
+                  oligomer_sp->getCalcOptionsCstRef()
+                    .getIndexRangeCollectionCstRef()
+                    .getRangesCstRef())
+            valueString +=
+              QString("[%1-%2]").arg(item->m_start + 1).arg(item->m_stop + 1);
         }
       else if(column == FRAGMENT_OLIGO_MONO_COLUMN)
         {
-          valueString = oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
+          valueString = QString("%1\n").arg(
+            oligomer_sp->getMass(libXpertMassCore::Enums::MassType::MONO),
+            0,
+            'f',
+            libXpertMassCore::OLIGOMER_DEC_PLACES);
         }
       else if(column == FRAGMENT_OLIGO_AVG_COLUMN)
         {
-          valueString = oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
+          valueString = QString("%1\n").arg(
+            oligomer_sp->getMass(libXpertMassCore::Enums::MassType::AVG),
+            0,
+            'f',
+            libXpertMassCore::OLIGOMER_DEC_PLACES);
         }
       else if(column == FRAGMENT_OLIGO_CHARGE_COLUMN)
         {
-          int charge = oligomer_sp->charge();
-
-          valueString.setNum(charge);
+          valueString.setNum(oligomer_sp->getIonizerCstRef().charge());
         }
       else if(column == FRAGMENT_OLIGO_FORMULA_COLUMN)
         {
-          valueString = oligomer_sp->elementalComposition();
+          // Do not use the oligomer_sp->elementalComposition() function
+          // because it cannot account for all the fragmentation specifics.
+          // This is why Oligomer::m_formula is for: while fragmenting,
+          // store in that Formula all the fragmentation specifics that
+          // translate into a Formula.
+          valueString = oligomer_sp->getFormulaCstRef().getActionFormula();
         }
       else if(column == FRAGMENT_OLIGO_MODIF_COLUMN)
         {
@@ -255,7 +258,8 @@ FragmentOligomerTableViewModel::data(const QModelIndex &index, int role) const
 }
 
 int
-FragmentOligomerTableViewModel::addOligomers(const OligomerList &oligomerList)
+FragmentOligomerTableViewModel::addOligomers(
+  const libXpertMassCore::OligomerCollection &oligomers)
 {
   // We receive a list of oligomers which we are asked to add to the
   // list of oligomers that we actually do not own, since they are
@@ -266,29 +270,29 @@ FragmentOligomerTableViewModel::addOligomers(const OligomerList &oligomerList)
   // addition work here.
 
   // Note that we are acutally *transferring* all the oligomers from
-  // the oligomerList to this model's mp_oligomerList.
+  // the oligomerList to this model's mp_oligomers.
 
-  int oligomerCount = mp_oligomerList->size();
+  std::size_t initial_size = mp_oligomers->getOligomersCstRef().size();
 
   // qDebug() << __FILE__ << __LINE__
   //          << "model oligomers:" << oligomerCount;
 
-  int oligomersToAdd = oligomerList.size();
-
-  // qDebug() << __FILE__ << __LINE__
-  //          << "oligomers to add:" << oligomersToAdd;
-
-  int addedOligomerCount = 0;
-
   // We have to let know the model that we are going to modify the
   // list of oligomers and thus the number of rows in the table
   // view. We will append the oligomers to the preexisting ones,
   // which means we are going to have our first appended oligomer at
   // index = oligomerCount.
 
-  beginInsertRows(
-    QModelIndex(), oligomerCount, (oligomerCount + oligomersToAdd - 1));
+  beginInsertRows(QModelIndex(),
+                  initial_size,
+                  (initial_size + oligomers.getOligomersCstRef().size() - 1));
 
+  for(const libXpertMassCore::OligomerSPtr &oligomer_sp :
+      oligomers.getOligomersCstRef())
+    mp_oligomers->getOligomersRef().push_back(oligomer_sp);
+
+#if 0
+//  old version
   for(int iter = 0; iter < oligomersToAdd; ++iter)
     {
       // We need to insert the oligomers as FragmentOligomer's because only this
@@ -297,35 +301,27 @@ FragmentOligomerTableViewModel::addOligomers(const OligomerList &oligomerList)
       FragmentOligomerSPtr oligomer_sp =
         std::dynamic_pointer_cast<FragmentOligomer>(oligomerList.at(iter));
 
-      //qDebug() << "Adding to the table view model the oligomer:"
-               //<< oligomer_sp->elementalComposition()
-               //<< "with charge:" << oligomer_sp->charge();
+      // qDebug() << "Adding to the table view model the oligomer:"
+      //<< oligomer_sp->elementalComposition()
+      //<< "with charge:" << oligomer_sp->charge();
 
-      mp_oligomerList->append(oligomer_sp);
+      mp_oligomers->append(oligomer_sp);
 
       ++addedOligomerCount;
     }
+#endif
 
   endInsertRows();
 
-  //FragmentOligomerSPtr oligomer_sp =
-    //std::dynamic_pointer_cast<FragmentOligomer>(mp_oligomerList->at(0));
-  //qDebug() << "Getting back from the table view model the oligomer:"
-           //<< oligomer_sp->elementalComposition()
-           //<< "with charge:" << oligomer_sp->charge();
-
-  return addedOligomerCount;
+  return mp_oligomers->size() - initial_size;
 }
 
 
 int
-FragmentOligomerTableViewModel::removeOligomers(int firstIndex, int lastIndex)
+FragmentOligomerTableViewModel::removeOligomers(std::size_t start_index,
+                                                std::size_t stop_index)
 {
-  // We are asked to remove the oligomers [firstIndex--lastIndex].
-
-  // firstIndex has to be >= 0 and < mp_oligomerList->size(). If
-  // lastIndex is -1, then remove all oligomers in range
-  // [firstIndex--last oligomer].
+  // We are asked to remove the oligomers [start_index--stop_index].
 
   // We are asked to remove the oligomers from the list of oligomers
   // that we actually do not own, since they are owned by the
@@ -335,63 +331,44 @@ FragmentOligomerTableViewModel::removeOligomers(int firstIndex, int lastIndex)
   // that the tableView takes them into account, we have to do that
   // removal work here.
 
-  int oligomerCount = mp_oligomerList->size();
+  std::size_t initial_size = mp_oligomers->getOligomersCstRef().size();
 
-  if(!oligomerCount)
+  // qDebug() << "The oligomers container has size:" << initial_size;
+
+  if(!initial_size)
     return 0;
 
-  if(firstIndex < 0 || firstIndex >= oligomerCount)
-    {
-      // qDebug() << __FILE__ << __LINE__
-      //          << "firstIndex:" << firstIndex
-      //          << "lastIndex:" << lastIndex;
+  if(start_index >= initial_size)
+    qFatal() << "Programming error. Out-of-bounds error.";
+  if(stop_index >= initial_size)
+    qFatal() << "Programming error. Out-of-bounds error.";
 
-      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-    }
-
-  int firstIdx = firstIndex;
-
-  // lastIndex can be < 0 (that is -1) if the oligomer to remove
-  // must go up to the last oligomer in the list.
-  if(lastIndex < -1 || lastIndex >= oligomerCount)
-    {
-      // qDebug() << __FILE__ << __LINE__
-      //          << "firstIndex:" << firstIndex
-      //          << "lastIndex:" << lastIndex;
-
-      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-    }
-
-  int lastIdx = 0;
-
-  if(lastIndex == -1)
-    lastIdx = (oligomerCount - 1);
-  else
-    lastIdx = lastIndex;
-
-  beginRemoveRows(QModelIndex(), firstIdx, lastIdx);
-
-  int removedOligomerCount = 0;
+  beginRemoveRows(QModelIndex(), start_index, stop_index);
 
+#if 0
+// Old version
   int iter = lastIdx;
-
   while(iter >= firstIdx)
     {
-      libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->takeAt(iter);
-
+      libXpertMassCore::OligomerSPtr oligomer_sp = mp_oligomers->takeAt(iter);
       oligomer_sp.reset();
-
       ++removedOligomerCount;
-
       --iter;
     }
+#endif
 
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_begin_iterator =
+    mp_oligomers->getOligomersRef().begin() + start_index;
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_end_iterator =
+    mp_oligomers->getOligomersRef().begin() + stop_index + 1;
+
+  mp_oligomers->getOligomersRef().erase(the_begin_iterator, the_end_iterator);
   endRemoveRows();
 
-  return removedOligomerCount;
+  return initial_size - mp_oligomers->size();
 }
 
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
