@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,9 +30,9 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef FRAGMENT_OLIGOMER_TABLE_VIEW_MODEL_HPP
-#define FRAGMENT_OLIGOMER_TABLE_VIEW_MODEL_HPP
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
@@ -40,15 +40,24 @@
 #include <QAbstractTableModel>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/OligomerCollection.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include "../nongui/OligomerList.hpp"
 #include "FragmentationDlg.hpp"
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -70,10 +79,10 @@ class FragmentOligomerTableViewModel : public QAbstractTableModel
 {
   Q_OBJECT
   public:
-  FragmentOligomerTableViewModel(OligomerList *, QObject *);
+  FragmentOligomerTableViewModel(libXpertMassCore::OligomerCollection *, QObject *);
   ~FragmentOligomerTableViewModel();
 
-  const OligomerList *oligomerList();
+  const libXpertMassCore::OligomerCollection *oligomerList();
   FragmentationDlg *parentDlg();
 
   void setTableView(FragmentOligomerTableView *);
@@ -87,18 +96,15 @@ class FragmentOligomerTableViewModel : public QAbstractTableModel
   QVariant data(const QModelIndex &parent = QModelIndex(),
                 int role                  = Qt::DisplayRole) const;
 
-  int addOligomers(const OligomerList &);
-  int removeOligomers(int, int);
+  int addOligomers(const libXpertMassCore::OligomerCollection &oligomers);
+  int removeOligomers(std::size_t start_index, std::size_t stop_index);
 
   private:
-  OligomerList *mp_oligomerList;
+  libXpertMassCore::OligomerCollection *mp_oligomers;
   FragmentationDlg *mp_parentDlg;
   FragmentOligomerTableView *mp_tableView;
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // FRAGMENT_OLIGOMER_TABLE_VIEW_MODEL_HPP
