@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,73 +36,69 @@
 #include <QFileDialog>
 #include <QSettings>
 
-#include <globals.hpp>
-#include <pappsomspp/utils.h>
+#include <pappsomspp/core/utils.h>
 
-/////////////////////// libmass includes
-#include <libXpertMass/PolChemDefEntity.hpp>
-#include <libXpertMass/IsotopicDataUserConfigHandler.hpp>
-#include <libXpertMass/IsotopicClusterGenerator.hpp>
-#include <libXpertMass/IsotopicClusterShaper.hpp>
-#include <libXpertMass/IsotopicData.hpp>
-#include <libXpertMass/MassPeakShaper.hpp>
-#include <libXpertMass/MassPeakShaperConfig.hpp>
-#include <libXpertMassGui/ColorSelector.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/IsotopicDataUserConfigHandler.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicClusterGenerator.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicClusterShaper.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicData.hpp>
+#include <MsXpS/libXpertMassCore/MassPeakShaper.hpp>
+#include <MsXpS/libXpertMassCore/MassPeakShaperConfig.hpp>
+#include <MsXpS/libXpertMassGui/ColorSelector.hpp>
 
 
 /////////////////////// Local includes
 #include "../nongui/globals.hpp"
-#include "../nongui/FragmentOligomer.hpp"
 #include "FragmentationDlg.hpp"
+#include "PolChemDefRendering.hpp"
+#include "PolChemDefRendering.hpp"
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
 FragmentationDlg::FragmentationDlg(
-  SequenceEditorWnd *editorWnd,
-  libXpertMass::Polymer *polymer,
-  const libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr,
-  const QString &configSettingsFilePath,
-  const QString &applicationName,
+  SequenceEditorWnd *editor_wnd_p,
+  libXpertMassCore::PolymerQSPtr polymer_sp,
+  const libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp,
+  const QString &config_settings_file_path,
+  const QString &application_name,
   const QString &description,
-  const libXpertMass::CalcOptions &calcOptions,
-  const libXpertMass::IonizeRule *ionizeRule)
-  : AbstractSeqEdWndDependentDlg(editorWnd,
-                                 polymer,
-                                 polChemDefCstSPtr,
-                                 configSettingsFilePath,
+  const libXpertMassCore::CalcOptions &calc_options,
+  const libXpertMassCore::Ionizer &ionizer)
+  : AbstractSeqEdWndDependentDlg(editor_wnd_p,
+                                 polymer_sp,
+                                 pol_chem_def_csp,
+                                 config_settings_file_path,
                                  "CompositionsDlg",
-                                 applicationName,
+                                 application_name,
                                  description),
-    m_calcOptions{calcOptions},
-    mp_ionizeRule{ionizeRule}
+    m_calcOptions(calc_options),
+    m_ionizer(ionizer)
 {
-  if(!mp_polymer || !mcsp_polChemDef)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-
-  if(!mp_ionizeRule)
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+  if(polymer_sp == nullptr || pol_chem_def_csp == nullptr)
+    qFatal() << "Programming error. Pointer cannot be nullptr.";
 
   if(!initialize())
-    qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+    qFatal() << "Programming error. Failed to initialize dialog window.";
 }
 
 
 FragmentationDlg::~FragmentationDlg()
 {
-  m_oligomerList.clear();
+  m_oligomers.clear();
 
   delete mpa_resultsString;
 
   delete mpa_oligomerTableViewModel;
   delete mpa_proxyModel;
 
-  delete mpa_fragmenter;
+  writeSettings();
 }
 
 
@@ -145,6 +141,8 @@ FragmentationDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Update the window title because the window title element in m_ui might be
   // either erroneous or empty.
   setWindowTitle(
@@ -166,7 +164,6 @@ FragmentationDlg::initialize()
   // Set pointers to 0 so that after the setupTableView call below
   // they'll get their proper value. We'll then be able to free all
   // that stuff in the destructor.
-  mpa_fragmenter             = 0;
   mpa_proxyModel             = 0;
   mpa_oligomerTableViewModel = 0;
 
@@ -291,38 +288,44 @@ FragmentationDlg::initialize()
 bool
 FragmentationDlg::populateSelectedOligomerData()
 {
-  libXpertMass::CoordinateList coordList;
+  libXpertMassCore::IndexRangeCollection index_range_collection;
 
-  bool res = mp_editorWnd->mpa_editorGraphicsView->selectionIndices(&coordList);
-
-  if(!res)
+  if(!mp_editorWnd->mpa_editorGraphicsView->selectionIndices(
+       index_range_collection))
     {
       QMessageBox::information(this,
-                               tr("massxpert - Fragmentation"),
+                               tr("MassXpert3 - Fragmentation"),
                                tr("No oligomer is selected. "
                                   "Select an oligomer first"),
                                QMessageBox::Ok);
       return false;
     }
 
-  if(coordList.size() > 1)
+  qDebug() << "Got selection indices from sequence editor window:"
+           << index_range_collection.indicesAsText();
+
+  if(index_range_collection.size() > 1)
     {
-      QMessageBox::information(this,
-                               tr("massxpert - Fragmentation"),
-                               tr("Fragmentation simulations with\n"
-                                  "multi-region selection are not "
-                                  "supported."),
-                               QMessageBox::Ok);
-      return false;
+      QMessageBox::information(
+        this,
+        tr("MassXpert3 - Fragmentation"),
+        tr("Fragmentation simulations with\n"
+           "multi-region selection are not "
+           "supported. Using the most inclusive sequence index range."),
+        QMessageBox::Ok);
     }
 
-  m_calcOptions.setCoordinateList(coordList);
+  libXpertMassCore::IndexRange *index_range_p =
+    index_range_collection.mostInclusiveLeftRightIndexRange();
+  m_fragmentIndexRange.initialize(*index_range_p);
+  delete index_range_p;
 
-  libXpertMass::Coordinates *coordinates = coordList.last();
+  m_calcOptions.setIndexRange(m_fragmentIndexRange);
 
-  m_ui.oligomerStartLabel->setText(QString().setNum(coordinates->start() + 1));
-
-  m_ui.oligomerEndLabel->setText(QString().setNum(coordinates->end() + 1));
+  m_ui.oligomerStartLabel->setText(
+    QString().setNum(m_fragmentIndexRange.m_start + 1));
+  m_ui.oligomerEndLabel->setText(
+    QString().setNum(m_fragmentIndexRange.m_stop + 1));
 
   return true;
 }
@@ -331,13 +334,9 @@ FragmentationDlg::populateSelectedOligomerData()
 void
 FragmentationDlg::populateFragSpecList()
 {
-  for(int iter = 0; iter < mcsp_polChemDef->fragSpecList().size(); ++iter)
-    {
-      libXpertMass::FragSpec *fragSpec = mcsp_polChemDef->fragSpecList().at(iter);
-      Q_ASSERT(fragSpec);
-
-      m_ui.fragmentationPatternListWidget->addItem(fragSpec->name());
-    }
+  for(libXpertMassCore::FragmentationPathwaySPtr frag_pathway_sp :
+      mcsp_polChemDef->getFragmentationPathwaysCstRef())
+    m_ui.fragmentationPatternListWidget->addItem(frag_pathway_sp->getName());
 
   return;
 }
@@ -348,7 +347,7 @@ FragmentationDlg::setupTableView()
 {
   // Model stuff all thought for sorting.
   mpa_oligomerTableViewModel =
-    new FragmentOligomerTableViewModel(&m_oligomerList, this);
+    new FragmentOligomerTableViewModel(&m_oligomers, this);
 
   mpa_proxyModel = new FragmentOligomerTableViewSortProxyModel(this);
   mpa_proxyModel->setSourceModel(mpa_oligomerTableViewModel);
@@ -356,7 +355,7 @@ FragmentationDlg::setupTableView()
 
   m_ui.oligomerTableView->setModel(mpa_proxyModel);
   m_ui.oligomerTableView->setParentDlg(this);
-  m_ui.oligomerTableView->setOligomerList(&m_oligomerList);
+  m_ui.oligomerTableView->setOligomerCollection(&m_oligomers);
   mpa_oligomerTableViewModel->setTableView(m_ui.oligomerTableView);
 }
 
@@ -364,43 +363,40 @@ FragmentationDlg::setupTableView()
 void
 FragmentationDlg::fragment()
 {
-  // Update the options from the parent window.
-  m_calcOptions = mp_editorWnd->calcOptions();
-
-  // And now override the selection data by using the
-  // selection-specific function below.
-  if(!populateSelectedOligomerData())
-    return;
-
   // For each fragmentation specification selected in the list of
   // available fragmentation specifications we have to construct a
-  // FragOptions instance and add it to the list of such frag
+  // FragmentationConfig instance and add it to the list of such frag
   // options. If no item is currently selected, just return.
 
   // What are the currently selected specifications ?
-  QList<QListWidgetItem *> selectionList =
+  QList<QListWidgetItem *> selected_frag_path_list_widget_items =
     m_ui.fragmentationPatternListWidget->selectedItems();
 
-  if(selectionList.isEmpty())
-    return;
+  if(selected_frag_path_list_widget_items.isEmpty())
+    {
+      QMessageBox::information(
+        this,
+        tr("MassXpert3 - Polymer fragmentation"),
+        tr("Please select at least one fragmentation pathway."),
+        QMessageBox::Ok);
+
+      return;
+    }
 
   // We'll need to use the specifications in the polymer chemistry
   // definition... for each frag options object.
-  QList<libXpertMass::FragSpec *> refList = mcsp_polChemDef->fragSpecList();
+  std::vector<MsXpS::libXpertMassCore::FragmentationPathwaySPtr> ref_frag_pathways =
+    mcsp_polChemDef->getFragmentationPathwaysCstRef();
 
   // We'll have to store all the fragOptions in a list:
-  QList<FragOptions *> fragOptionList;
-
-  // We'll need a fragmentation specification each time we iterate in
-  // a new specification item below.
-  libXpertMass::FragSpec fragSpec(mcsp_polChemDef, "NOT_SET");
+  std::vector<MsXpS::libXpertMassCore::FragmentationConfig> fragmentation_configs;
 
   // Finally parse all the different fragmentation specification items
   // currently selected in the list widget.
 
-  for(int iter = 0; iter < selectionList.size(); ++iter)
+  for(int iter = 0; iter < selected_frag_path_list_widget_items.size(); ++iter)
     {
-      QListWidgetItem *item = selectionList.at(iter);
+      QListWidgetItem *item = selected_frag_path_list_widget_items.at(iter);
 
       if(item->isSelected())
         {
@@ -413,88 +409,101 @@ FragmentationDlg::fragment()
 
           QString name = item->text();
 
-          int res = libXpertMass::FragSpec::isNameInList(name, refList, &fragSpec);
+          qDebug() << "Name of the fragmentation pathway:" << name;
 
-          if(res == -1)
-            {
-              return;
-            }
+          libXpertMassCore::FragmentationPathwayCstSPtr frag_path_sp =
+            mcsp_polChemDef->getFragmentationPathwayCstSPtrByName(name);
 
-          FragOptions *fragOptions = new FragOptions(fragSpec);
+          if(frag_path_sp == nullptr)
+            return;
+
+          qDebug() << "The fragmentation pathway as known by the polymer "
+                      "chemistry definition:"
+                   << frag_path_sp->toString();
+
+          libXpertMassCore::FragmentationConfig fragmentation_config(*frag_path_sp);
 
           // The user might ask that some formulas be applied while
           // making the fragmentation with the selection
-          // fragmentation option. Lets feed the FragOptions
+          // fragmentation option. Lets feed the FragmentationConfig
           // instance with the related formulas.
 
           if(m_ui.decompositionsMinusWaterCheckBox->checkState() == Qt::Checked)
             {
-              fragOptions->addFormula(QString("-H2O"));
+              qDebug() << "Accounting for extra fragmentation formula:"
+                       << "-H2O";
+              fragmentation_config.addFormula(QString("-H2O"));
             }
 
           if(m_ui.decompositionsMinusAmmoniaCheckBox->checkState() ==
              Qt::Checked)
             {
-              fragOptions->addFormula(QString("-NH3"));
+              qDebug() << "Accounting for extra fragmentation formula:"
+                       << "-NH3";
+              fragmentation_config.addFormula(QString("-NH3"));
             }
 
           if(!m_ui.decompositionsFormulaLineEdit->text().isEmpty())
             {
-              libXpertMass::Formula formula(
+              qDebug() << "Accounting for extra fragmentation formula:"
+                       << m_ui.decompositionsFormulaLineEdit->text();
+              libXpertMassCore::Formula formula(
                 m_ui.decompositionsFormulaLineEdit->text());
 
               // At this point, we should check if the formula is valid.
 
-              libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+              libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
                 mcsp_polChemDef->getIsotopicDataCstSPtr();
 
-                qDebug() << "Going to validate decomposition formula" << formula.toString();
+              qDebug() << "Going to validate decomposition formula"
+                       << formula.getActionFormula();
 
-              if(!formula.validate(isotopic_data_csp))
+              libXpertMassCore::ErrorList error_list;
+
+              if(!formula.validate(isotopic_data_csp, &error_list))
                 {
                   QMessageBox::information(
                     0,
-                    tr("massXpert - Fragmentation"),
+                    tr("MassXpert3 - Fragmentation"),
                     tr("Failed to validate the decomposition formula."),
                     QMessageBox::Ok);
 
                   return;
                 }
 
-              fragOptions->addFormula(formula);
+              fragmentation_config.addFormula(formula);
             }
 
           // We got the coordinates of the oligomer to fragment in the
           // mp_calcOptions from the sequence editor window.
 
-          libXpertMass::Coordinates *coordinates =
-            m_calcOptions.coordinateList().last();
+          // Update the options from the parent window.
+          m_calcOptions.initialize(*mp_editorWnd->getCalcOptions());
 
-          fragOptions->setStartIndex(coordinates->start());
-          fragOptions->setEndIndex(coordinates->end());
+          // And now override the selection data by using the
+          // selection-specific function below.
+          if(!populateSelectedOligomerData())
+            return;
 
+          fragmentation_config.setStartIndex(m_fragmentIndexRange.m_start);
+          fragmentation_config.setStopIndex(m_fragmentIndexRange.m_stop);
 
           // Set the ionization levels.
-          int valueStart = m_ui.ionizeLevelStartSpinBox->value();
-          int valueEnd   = m_ui.ionizeLevelEndSpinBox->value();
+          int ionization_level_start = m_ui.ionizeLevelStartSpinBox->value();
+          int ionization_level_stop  = m_ui.ionizeLevelEndSpinBox->value();
 
-          if(valueStart > valueEnd)
-            {
-              fragOptions->setStartIonizeLevel(valueEnd);
-              fragOptions->setEndIonizeLevel(valueStart);
-            }
-          else
-            {
-              fragOptions->setStartIonizeLevel(valueStart);
-              fragOptions->setEndIonizeLevel(valueEnd);
-            }
+          if(ionization_level_start > ionization_level_stop)
+            std::swap(ionization_level_start, ionization_level_stop);
 
-
-          //   qDebug() << "Start:" << fragOptions.startIonizeLevel();
-          //   qDebug() << "End:" << fragOptions.endIonizeLevel();
+          fragmentation_config.setStartIonizeLevel(ionization_level_start);
+          fragmentation_config.setStopIonizeLevel(ionization_level_stop);
 
           // Add the frag options to the list.
-          fragOptionList.append(fragOptions);
+
+          qDebug() << "Pushing back fragmentation config:"
+                   << fragmentation_config.toString();
+
+          fragmentation_configs.push_back(fragmentation_config);
         }
       else
         continue;
@@ -504,48 +513,60 @@ FragmentationDlg::fragment()
   // number of items as the number of items selected in the list
   // widget.
 
-  Q_ASSERT(fragOptionList.size() == selectionList.size());
+  Q_ASSERT(static_cast<int>(fragmentation_configs.size()) ==
+           selected_frag_path_list_widget_items.size());
 
-  // The list in which we'll store all the allocated oligomers.
-  OligomerList oligomerList;
+  // m_ionizer is the copy of the Ionizer in the SequenceEditorWnd. It might
+  // contain an ionization status (that is, charge() or currentCharge() return
+  // non-0) but we do not want this, we want to transmit an ionizer with the
+  // ionization formula and the nominal charge set. The other data must be 0.
 
-  // Now that we have all the fragmentation options in the list, we
-  // can allocate the fragmenter! Note that this function might be
-  // called many times, which means that we first have to free the
-  // mpa_fragmenter allocated member if non-nullptr.
+  libXpertMassCore::Ionizer ionizer(m_ionizer);
+  ionizer.setLevel(0);
+  ionizer.forceCurrentStateLevel(0);
 
-  delete mpa_fragmenter;
+  libXpertMassCore::Fragmenter fragmenter(msp_polymer,
+                                      mcsp_polChemDef,
+                                      fragmentation_configs,
+                                      m_calcOptions,
+                                      ionizer);
 
-  // Update the mass calculation engine's configuration data
-  // directly from the sequence editor window.
-  m_calcOptions = mp_editorWnd->calcOptions();
+  qDebug()
+    << "Now asking that the fragmenter fragments with calculation options:"
+    << m_calcOptions.toString();
 
-  mpa_fragmenter = new Fragmenter(
-    mp_polymer, mcsp_polChemDef, fragOptionList, m_calcOptions, *mp_ionizeRule);
-
-  mpa_fragmenter->setOligomerList(&oligomerList);
-
-  if(!mpa_fragmenter->fragment())
+  if(!fragmenter.fragment())
     {
-      delete mpa_fragmenter;
-
       QMessageBox::critical(this,
-                            tr("massXpert - libXpertMass::Polymer Cleavage"),
+                            tr("MassXpert3 - Polymer fragmentation"),
                             tr("Failed to perform fragmentation."),
                             QMessageBox::Ok);
 
       return;
     }
 
-  FragmentOligomerSPtr oligomer_sp =
-    std::dynamic_pointer_cast<FragmentOligomer>(oligomerList.first());
+  // The list in which all the allocated oligomers have been stored by the
+  // fragmented inside of it..
+  const MsXpS::libXpertMassCore::OligomerCollection &fragment_oligomers =
+    fragmenter.getOligomerCollectionCstRef();
 
-  // qDebug()
-  //<< "First fragmentation oligomer has formula (elemental composition):"
-  //<< oligomer_sp->elementalComposition();
+  if(!fragment_oligomers.size())
+    {
+      QMessageBox::information(
+        this,
+        tr("MassXpert3 - Polymer fragmentation"),
+        tr("The fragmentation did not yield any fragment."),
+        QMessageBox::Ok);
+
+      return;
+    }
+
+  qDebug().noquote()
+    << "After fragmentation, all the obtained fragment oligomers:\n"
+    << fragment_oligomers.toString();
 
   // At this point we have a brand new set of oligomers in the local
-  // list of oligomers (oligomerList). We either stack these on top
+  // list of oligomers (fragment_oligomers). We either stack these on top
   // of the previous ones, or we replace the previous ones with the
   // new ones.  Are we stacking new oligomers on top of the old
   // ones?
@@ -553,32 +574,26 @@ FragmentationDlg::fragment()
   if(!m_ui.stackOligomersCheckBox->isChecked())
     {
       // We are going to remove *all* the previous oligomers.
-      mpa_oligomerTableViewModel->removeOligomers(0, -1);
+      mpa_oligomerTableViewModel->removeOligomers(0, m_oligomers.size() - 1);
     }
 
   // At this point we can set up the data to the treeview model.
 
-  int initialOligomers = oligomerList.size();
+  int existing_oligomer_count = fragment_oligomers.size();
 
-  // qDebug() << __FILE__ << __LINE__
-  //          << "initialOligomers:" << initialOligomers;
-
-  // We are *transferring* the oligomers from oligomerList to the
+  // We are *transferring* the oligomers from fragment_oligomers to the
   // list of oligomers that is in the model : we are not duplicating
-  // the oligomers. When the transfer has been done, oligomerList
-  // will be empty.
-  int addedOligomers = mpa_oligomerTableViewModel->addOligomers(oligomerList);
+  // the oligomers. When the transfer has been done, the fragment_oligomers will
+  // be cleared,  as their owner,  fragmenter,  will be destroyed going out of
+  // scope.
+  int newly_added_oligomer_count =
+    mpa_oligomerTableViewModel->addOligomers(fragment_oligomers);
 
   // qDebug() << __FILE__ << __LINE__
   //          << "addedOligomers:" << addedOligomers;
 
-  if(initialOligomers != addedOligomers)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-
-  // Now that we have *transferred* (not copied) all the oligomers
-  // in the model, we can clear the oligomerList without freeing the
-  // instances it currently contain...
-  oligomerList.clear();
+  if(existing_oligomer_count != newly_added_oligomer_count)
+    qFatal() << "Programming error.";
 
   // Set focus to the treeView.
   m_ui.oligomerTableView->setFocus();
@@ -604,24 +619,28 @@ FragmentationDlg::fragment()
 
 void
 FragmentationDlg::updateFragmentationDetails(
-  const libXpertMass::CalcOptions &calcOptions)
+  const libXpertMassCore::CalcOptions &calcOptions)
 {
-  if(calcOptions.monomerEntities() & libXpertMass::MONOMER_CHEMENT_MODIF)
+  if(static_cast<int>(calcOptions.getMonomerEntities()) &
+     static_cast<int>(libXpertMassCore::Enums::ChemicalEntity::MODIF))
     m_ui.modificationsCheckBox->setCheckState(Qt::Checked);
   else
     m_ui.modificationsCheckBox->setCheckState(Qt::Unchecked);
 
-  if(calcOptions.polymerEntities() & libXpertMass::POLYMER_CHEMENT_LEFT_END_MODIF)
+  if(static_cast<int>(calcOptions.getPolymerEntities()) &
+     static_cast<int>(libXpertMassCore::Enums::ChemicalEntity::LEFT_END_MODIF))
     m_ui.leftModifCheckBox->setCheckState(Qt::Checked);
   else
     m_ui.leftModifCheckBox->setCheckState(Qt::Unchecked);
 
-  if(calcOptions.polymerEntities() & libXpertMass::POLYMER_CHEMENT_RIGHT_END_MODIF)
+  if(static_cast<int>(calcOptions.getPolymerEntities()) &
+     static_cast<int>(libXpertMassCore::Enums::ChemicalEntity::RIGHT_END_MODIF))
     m_ui.rightModifCheckBox->setCheckState(Qt::Checked);
   else
     m_ui.rightModifCheckBox->setCheckState(Qt::Unchecked);
 
-  if(calcOptions.monomerEntities() & libXpertMass::MONOMER_CHEMENT_CROSS_LINK)
+  if(static_cast<int>(calcOptions.getMonomerEntities()) &
+     static_cast<int>(libXpertMassCore::Enums::ChemicalEntity::CROSS_LINKER))
     m_ui.crossLinksCheckBox->setCheckState(Qt::Checked);
   else
     m_ui.crossLinksCheckBox->setCheckState(Qt::Unchecked);
@@ -908,7 +927,7 @@ FragmentationDlg::massSpectrumSynthesisMenuActivated(int index)
       if(!file.open(QIODevice::WriteOnly))
         {
           QMessageBox::information(0,
-                                   tr("massXpert2 - Export mass spectrum"),
+                                   tr("MassXpert3 - Export mass spectrum"),
                                    tr("Failed to open file in write mode."),
                                    QMessageBox::Ok);
           return;
@@ -941,15 +960,15 @@ FragmentationDlg::configureMassPeakShaper()
     }
 
   // The signal below is only emitted when checking parameters worked ok.
-  connect(
-    mp_massPeakShaperConfigDlg,
-    &libXpertMassGui::MassPeakShaperConfigDlg::updatedMassPeakShaperConfigSignal,
-    [this](const libXpertMass::MassPeakShaperConfig &config) {
-      m_massPeakShaperConfig = config;
+  connect(mp_massPeakShaperConfigDlg,
+          &libXpertMassGui::MassPeakShaperConfigDlg::
+            updatedMassPeakShaperConfigSignal,
+          [this](const libXpertMassCore::MassPeakShaperConfig &config) {
+            m_massPeakShaperConfig.initialize(config);
 
-      // qDebug().noquote() << "Our local copy of the config:"
-      //<< m_massPeakShaperConfig.toString();
-    });
+            // qDebug().noquote() << "Our local copy of the config:"
+            //<< m_massPeakShaperConfig.toString();
+          });
 
   mp_massPeakShaperConfigDlg->activateWindow();
   mp_massPeakShaperConfigDlg->raise();
@@ -974,7 +993,7 @@ FragmentationDlg::loadIsotopicDataFromFile()
       return false;
     }
 
-  libXpertMass::IsotopicDataUserConfigHandler handler;
+  libXpertMassCore::IsotopicDataUserConfigHandler handler;
   handler.loadData(file_name);
 
   msp_isotopicData = handler.getIsotopicData();
@@ -984,7 +1003,7 @@ FragmentationDlg::loadIsotopicDataFromFile()
       QMessageBox::warning(
         this,
         QString("%1 - %2").arg(m_applicationName).arg(m_windowDescription),
-       "Failed to load the isotopic data.",
+        "Failed to load the isotopic data.",
         QMessageBox::Ok);
 
       return false;
@@ -1011,24 +1030,35 @@ FragmentationDlg::synthesizeMassSpectra()
 
   setCursor(Qt::WaitCursor);
 
-  if(!m_massPeakShaperConfig.resolve())
-    {
-      QMessageBox msgBox;
-      QString msg("Please, configure the mass peak shaper.");
-      msgBox.setText(msg);
-      msgBox.exec();
+  libXpertMassCore::ErrorList error_list;
 
-      configureMassPeakShaper();
-    }
+  if(!m_massPeakShaperConfig.resolve(error_list))
+  {
+    QMessageBox msgBox;
+    QString msg =
+    QString("Mass peak shaper configuration errors:\n%1")
+    .arg(libXpertMassCore::Utils::joinErrorList(error_list, "\n"));
+    msgBox.setText(msg);
+    msgBox.exec();
+
+    configureMassPeakShaper();
+  }
 
   // All the selected oligomers need to be processed such that using both
-  // their elemental composition and their charge, we create isotopic
+  // their formula and their charge, we create isotopic
   // clusters that we can then merge into a single mass spectrum.
 
-  OligomerList selected_oligomers;
+  // Remember, Oligomer instances created during fragmentation do not calculate
+  // the formula properly using Oligomer::elementalComposition() because
+  // the Oligomer itself is not aware of the fragmentation chemistry. This
+  // is the reason why Oligomer::m_formula exists: to store, during the
+  // fragmentation, the state of the formula that describes the final
+  // product ion.
+
+  libXpertMassCore::OligomerCollection selected_oligomers;
 
   std::size_t count =
-    m_ui.oligomerTableView->selectedOligomers(&selected_oligomers, -1);
+    m_ui.oligomerTableView->selectedOligomers(selected_oligomers, -1);
 
   if(!count)
     {
@@ -1047,34 +1077,32 @@ FragmentationDlg::synthesizeMassSpectra()
   // want a non-const shared pointer!
 
   if(msp_isotopicData == nullptr)
-    msp_isotopicData = std::make_shared<libXpertMass::IsotopicData>(
-      *mp_editorWnd->polChemDefRendering()->getPolChemDef()->getIsotopicDataCstSPtr());
+    msp_isotopicData = std::make_shared<libXpertMassCore::IsotopicData>(
+      *mp_editorWnd->getPolChemDefRenderingCstRPtr()
+         ->getPolChemDefCstSPtr()
+         ->getIsotopicDataCstSPtr());
 
   if(msp_isotopicData == nullptr)
-    qFatal("Programming error. The isotopic data cannot be nullptr.");
+    qFatal() << "Programming error. The isotopic data cannot be nullptr.";
 
   // Great, we now we have isotopes to do the calculations!
 
   // And now we can instantiate a cluster generator
 
-  libXpertMass::IsotopicClusterGenerator isotopic_cluster_generator(
+  libXpertMassCore::IsotopicClusterGenerator isotopic_cluster_generator(
     msp_isotopicData);
 
   // Construct the formula/charge pairs needed for the isotopic cluster
   // calculations.
-  std::vector<libXpertMass::FormulaChargePair> formula_charge_pairs;
+  std::vector<libXpertMassCore::FormulaChargePair> formula_charge_pairs;
 
   // For debugging purposes.
   // std::size_t oligomer_count = 0;
 
-  for(int iter = 0; iter < selected_oligomers.size(); ++iter)
+  for(libXpertMassCore::OligomerSPtr oligomer_sp :
+      selected_oligomers.getOligomersCstRef())
     {
-      FragmentOligomerSPtr oligomer_sp =
-        std::dynamic_pointer_cast<FragmentOligomer>(
-          selected_oligomers.at(iter));
-
-      int charge           = oligomer_sp->charge();
-      QString formula_text = oligomer_sp->elementalComposition();
+      int charge = oligomer_sp->getIonizerCstRef().charge();
 
       // qDebug() << formula_text << charge << oligomer_sp->mono();
 
@@ -1085,13 +1113,19 @@ FragmentationDlg::synthesizeMassSpectra()
       // could end with something like (specifically with immonium ions that are
       // very small) CxHyNz-Ow
 
-      libXpertMass::Formula formula(formula_text);
+      // Do not use Oligomer::elementalComposition() !! See above, for
+      // explanations.
+      libXpertMassCore::Formula formula(oligomer_sp->getFormulaCstRef());
 
       // We want to check the formula "fresh", store the symbol/count pairs and
       // reset (although not really needed since we have created the formula at
       // the code line above).
-      if(!formula.validate(msp_isotopicData, true, true))
-        qFatal("Programming error. The formula failed to validate.");
+      libXpertMassCore::ErrorList error_list;
+      if(!formula.validate(msp_isotopicData, true, true, &error_list))
+        qFatal() << "Programming error."
+                       << QString("The formula failed to validate with errors:")
+                            .arg(
+                              libXpertMassCore::Utils::joinErrorList(error_list));
 
       if(formula.hasNetMinusPart())
         {
@@ -1104,7 +1138,7 @@ FragmentationDlg::synthesizeMassSpectra()
       // generator.
 
       formula_charge_pairs.push_back(
-        libXpertMass::FormulaChargePair(formula_text, charge));
+        libXpertMassCore::FormulaChargePair(formula.getActionFormula(), charge));
 
       //++oligomer_count;
     }
@@ -1117,7 +1151,7 @@ FragmentationDlg::synthesizeMassSpectra()
   isotopic_cluster_generator.setFormulaChargePairs(formula_charge_pairs);
 
   isotopic_cluster_generator.setIsotopicDataType(
-    libXpertMass::IsotopicDataType::LIBRARY_CONFIG);
+    libXpertMassCore::IsotopicDataType::LIBRARY_CONFIG);
 
   double cumulated_probabilities = m_ui.cumulatedProbsDoubleSpinBox->value();
   isotopic_cluster_generator.setMaxSummedProbability(cumulated_probabilities);
@@ -1126,14 +1160,14 @@ FragmentationDlg::synthesizeMassSpectra()
 
   // Not needed here because it will be taken into account at peak shaping
   // time
-  //isotopic_cluster_generator.setNormalizationIntensity(normalization_intensity);
-  //qDebug() << "Set the generator's normalization intensity:"
-           //<< normalization_intensity;
+  // isotopic_cluster_generator.setNormalizationIntensity(normalization_intensity);
+  // qDebug() << "Set the generator's normalization intensity:"
+  //<< normalization_intensity;
 
   // And now perform the work. For each formula/charge pair, the generator will
   // create an isotopic cluster shaped trace associated to the corresponding
-  // charge in a libXpertMass::IsotopicClusterChargePair. All these pairs are stored
-  // in a vector.
+  // charge in a libXpertMassCore::IsotopicClusterChargePair. All these pairs are
+  // stored in a vector.
 
   count = isotopic_cluster_generator.run();
 
@@ -1153,7 +1187,7 @@ FragmentationDlg::synthesizeMassSpectra()
 
   // qDebug() << "The number of clusters generated:" << count;
 
-  std::vector<libXpertMass::IsotopicClusterChargePair>
+  std::vector<libXpertMassCore::IsotopicClusterChargePair>
     isotopic_cluster_charge_pairs =
       isotopic_cluster_generator.getIsotopicClusterChargePairs();
 
@@ -1166,7 +1200,8 @@ FragmentationDlg::synthesizeMassSpectra()
 
   // Note how we ask a reference to the pair that is iterated into, otherwise we
   // would get a copy and we would lose the local charge modification.
-  for(libXpertMass::IsotopicClusterChargePair &pair : isotopic_cluster_charge_pairs)
+  for(libXpertMassCore::IsotopicClusterChargePair &pair :
+      isotopic_cluster_charge_pairs)
     pair.second = 1;
 
   qDebug();
@@ -1174,7 +1209,7 @@ FragmentationDlg::synthesizeMassSpectra()
   // Now instantiate the isotopic cluster shaper and set the clusters' data in
   // it.
 
-  libXpertMass::IsotopicClusterShaper isotopic_cluster_shaper(
+  libXpertMassCore::IsotopicClusterShaper isotopic_cluster_shaper(
     isotopic_cluster_charge_pairs, m_massPeakShaperConfig);
 
   isotopic_cluster_shaper.setNormalizeIntensity(normalization_intensity);
@@ -1205,7 +1240,7 @@ FragmentationDlg::synthesizeMassSpectra()
     trace_title = mp_editorWnd->getSequenceName();
 
   if(trace_title.isEmpty())
-    trace_title = mp_editorWnd->polymer()->filePath();
+    trace_title = mp_editorWnd->getPolymerSPtr()->getFilePath();
 
   emit displayMassSpectrumSignal(
     trace_title,
@@ -1261,8 +1296,8 @@ FragmentationDlg::prepareResultsTxtString()
   // mass <delim> charge <delim> name <delim> coords
   //
   // Note that name and coords are optional.
-  bool forXpertMiner         = false;
-  libXpertMass::MassType massType = libXpertMass::MassType::MASS_NONE;
+  bool forXpertMiner = false;
+  libXpertMassCore::Enums::MassType mass_type;
 
   forXpertMiner = m_ui.forXpertMinerCheckBox->isChecked();
 
@@ -1274,21 +1309,16 @@ FragmentationDlg::prepareResultsTxtString()
         "# Fragmentation: \n"
         "# ---------------------------\n");
     }
-  else
-    {
-      // Then, we should ask whether the masses to be exported are
-      // the mono or avg masses.
 
-      if(m_ui.monoRadioButton->isChecked())
-        massType = libXpertMass::MassType::MASS_MONO;
-      else
-        massType = libXpertMass::MassType::MASS_AVG;
-    }
+  if(m_ui.monoRadioButton->isChecked())
+    mass_type = libXpertMassCore::Enums::MassType::MONO;
+  else
+    mass_type = libXpertMassCore::Enums::MassType::AVG;
 
   bool withSequence = m_ui.withSequenceCheckBox->isChecked();
 
-  QString *text = m_ui.oligomerTableView->selectedOligomersAsPlainText(
-    delimiter, withSequence, forXpertMiner, massType);
+  QString *text = m_ui.oligomerTableView->describeOligomerAsPlainText(
+    delimiter, withSequence, forXpertMiner, mass_type);
 
   *mpa_resultsString += *text;
 
@@ -1323,7 +1353,7 @@ FragmentationDlg::exportResultsFile()
   if(!file.open(QIODevice::WriteOnly | QIODevice::Append))
     {
       QMessageBox::information(0,
-                               tr("massXpert - Export Data"),
+                               tr("MassXpert3 - Export Data"),
                                tr("Failed to open file in append mode."),
                                QMessageBox::Ok);
       return false;
@@ -1360,6 +1390,6 @@ FragmentationDlg::selectResultsFile()
 //////////////////////////////////// The results-exporting functions.
 //////////////////////////////////// The results-exporting functions.
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
