@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,35 +30,37 @@
  * END software license
  */
 
-
-#ifndef FRAGMENTATION_DLG_HPP
-#define FRAGMENTATION_DLG_HPP
+#pragma once
 
 
 /////////////////////// Qt includes
 #include <QSortFilterProxyModel>
-#include <libXpertMass/PolChemDefEntity.hpp>
 
 
-/////////////////////// libmassgui includes
-#include <libXpertMassGui/MassPeakShaperConfigDlg.hpp>
-#include <pappsomspp/trace/trace.h>
+/////////////////////// libpappsomspp includes
+#include <pappsomspp/core/trace/trace.h>
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Fragmenter.hpp>
+#include <MsXpS/libXpertMassCore/FragmentationConfig.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+#include <MsXpS/libXpertMassGui/MassPeakShaperConfigDlg.hpp>
 
 
 /////////////////////// Local includes
 #include "ui_FragmentationDlg.h"
 #include "AbstractSeqEdWndDependentDlg.hpp"
-#include "../nongui/Fragmenter.hpp"
 #include "SequenceEditorWnd.hpp"
 #include "FragmentOligomerTableViewModel.hpp"
 #include "FragmentOligomerTableViewSortProxyModel.hpp"
-#include "../nongui/FragOptions.hpp"
 
 
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -71,14 +73,14 @@ class FragmentationDlg : public AbstractSeqEdWndDependentDlg
 
 
   public:
-  FragmentationDlg(SequenceEditorWnd *editorWnd,
-                   libXpertMass::Polymer *polymer,
-                   const libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr,
-                   const QString &configSettingsFilePath,
-                   const QString &applicationName,
+  FragmentationDlg(SequenceEditorWnd *editor_wnd_p,
+                   libXpertMassCore::PolymerQSPtr polymer_sp,
+                   const libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp,
+                   const QString &config_settings_file_path,
+                   const QString &application_name,
                    const QString &description,
-                   const libXpertMass::CalcOptions &calcOptions,
-                   const libXpertMass::IonizeRule *ionizeRule);
+                   const libXpertMassCore::CalcOptions &calc_options,
+                   const libXpertMassCore::Ionizer &ionizer);
 
   ~FragmentationDlg();
 
@@ -87,7 +89,7 @@ class FragmentationDlg : public AbstractSeqEdWndDependentDlg
   bool populateSelectedOligomerData();
   void populateFragSpecList();
 
-  void updateFragmentationDetails(const libXpertMass::CalcOptions &);
+  void updateFragmentationDetails(const libXpertMassCore::CalcOptions &);
   void updateOligomerSequence(QString *);
 
   bool calculateTolerance(double);
@@ -127,17 +129,16 @@ class FragmentationDlg : public AbstractSeqEdWndDependentDlg
   QString m_resultsFilePath;
   //////////////////////////////////// The results-exporting strings.
 
-  OligomerList m_oligomerList;
+  libXpertMassCore::OligomerCollection m_oligomers;
 
-  libXpertMass::IsotopicDataSPtr msp_isotopicData = nullptr;
-  libXpertMass::MassPeakShaperConfig m_massPeakShaperConfig;
+  libXpertMassCore::IsotopicDataSPtr msp_isotopicData = nullptr;
+  libXpertMassCore::MassPeakShaperConfig m_massPeakShaperConfig;
 
   pappso::Trace m_syntheticMassSpectrum;
 
-  libXpertMass::CalcOptions m_calcOptions;
-  const libXpertMass::IonizeRule *mp_ionizeRule = nullptr;
-
-  Fragmenter *mpa_fragmenter = nullptr;
+  libXpertMassCore::CalcOptions m_calcOptions;
+  libXpertMassCore::IndexRange m_fragmentIndexRange;
+  const libXpertMassCore::Ionizer m_ionizer;
 
   // For the mass spectra that are synthesized and served.
   QByteArray m_colorByteArray;
@@ -146,13 +147,14 @@ class FragmentationDlg : public AbstractSeqEdWndDependentDlg
   void readSettings();
 
   FragmentOligomerTableViewModel *mpa_oligomerTableViewModel = nullptr;
-  FragmentOligomerTableViewSortProxyModel *mpa_proxyModel = nullptr;
+  FragmentOligomerTableViewSortProxyModel *mpa_proxyModel    = nullptr;
 
-  libXpertMassGui::MassPeakShaperConfigDlg *mp_massPeakShaperConfigDlg = nullptr;
+  libXpertMassGui::MassPeakShaperConfigDlg *mp_massPeakShaperConfigDlg =
+    nullptr;
 
   // For the filtering of the data in the treeview.
-  QAction *filterAct = nullptr;
-  double m_tolerance;
+  QAction *filterAct      = nullptr;
+  double m_tolerance      = 0;
   QWidget *mp_focusWidget = nullptr;
 
   void setupTableView();
@@ -160,9 +162,6 @@ class FragmentationDlg : public AbstractSeqEdWndDependentDlg
   void synthesizeMassSpectra();
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // FRAGMENTATION_DLG_HPP
