@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,16 +40,16 @@
 #include <QTimer>
 
 
-/////////////////////// libmass includes
-#include <libXpertMass/PolChemDef.hpp>
-#include <libXpertMass/Isotope.hpp>
-#include <libXpertMass/IsotopicData.hpp>
-#include <libXpertMass/IsotopicDataLibraryHandler.hpp>
-#include <libXpertMass/IsotopicDataUserConfigHandler.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/Isotope.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicData.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicDataLibraryHandler.hpp>
+#include <MsXpS/libXpertMassCore/IsotopicDataUserConfigHandler.hpp>
 
 
-/////////////////////// libmassgui includes
-#include <libXpertMassGui/IsotopicDataTableViewModel.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassGui/IsotopicDataTableViewModel.hpp>
 
 /////////////////////// Local includes
 #include "IsotopeDefDlg.hpp"
@@ -59,20 +59,20 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-IsotopeDefDlg::IsotopeDefDlg(libXpertMass::PolChemDefSPtr polChemDefSPtr,
-                             PolChemDefWnd *polChemDefWnd,
-                             const QString &settingsFilePath,
-                             const QString &applicationName,
+IsotopeDefDlg::IsotopeDefDlg(libXpertMassCore::PolChemDefSPtr pol_chem_def_sp,
+                             PolChemDefWnd *pol_chem_def_wnd_p,
+                             const QString &settings_file_path,
+                             const QString &application_name,
                              const QString &description)
-  : AbstractPolChemDefDependentDlg(polChemDefSPtr,
-                                   polChemDefWnd,
-                                   settingsFilePath,
+  : AbstractPolChemDefDependentDlg(pol_chem_def_sp,
+                                   pol_chem_def_wnd_p,
+                                   settings_file_path,
                                    "IsotopeDefDlg",
-                                   applicationName,
+                                   application_name,
                                    description)
 {
 
@@ -94,7 +94,7 @@ IsotopeDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
   // close==destruction. Thus we only hide the dialog remembering its
   // position and size.
 
-  mp_polChemDefWnd->m_ui.isotopePushButton->setChecked(false);
+  mp_pol_chem_def_wnd_p->m_ui.isotopePushButton->setChecked(false);
 
   writeSettings();
 }
@@ -103,7 +103,7 @@ IsotopeDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
 void
 IsotopeDefDlg::readSettings()
 {
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -116,7 +116,7 @@ void
 IsotopeDefDlg::writeSettings()
 {
 
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -130,6 +130,8 @@ IsotopeDefDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Now we need to actually show the window title (that element is empty in
   // m_ui)
   displayWindowTitle();
@@ -194,18 +196,18 @@ IsotopeDefDlg::initialize()
 bool
 IsotopeDefDlg::validate()
 {
-  std::vector<QString> errors;
+  libXpertMassCore::ErrorList error_list;
 
-  msp_polChemDef->getIsotopicDataSPtr()->validate(errors);
+  msp_polChemDef->getIsotopicDataSPtr()->validate(&error_list);
 
-  if(errors.size())
+  if(error_list.size())
     {
 
       qDebug() << "Validation failed.";
 
       QString joined_errors;
 
-      for(auto error : errors)
+      for(auto error : error_list)
         {
           qDebug() << error;
 
@@ -259,7 +261,7 @@ IsotopeDefDlg::loadData()
   // case that has to be user config handler, we do not want the library
   // handler.
 
-  libXpertMass::IsotopicDataUserConfigHandler isotopic_data_handler(
+  libXpertMassCore::IsotopicDataUserConfigHandler isotopic_data_handler(
     msp_polChemDef->getIsotopicDataSPtr());
 
   if(!isotopic_data_handler.loadData(file_path))
@@ -300,7 +302,7 @@ IsotopeDefDlg::save()
   // We need to instantiate the proper isotopic data handler. In our present
   // case that can be user config handler, we do not need the library handler.
 
-  libXpertMass::IsotopicDataUserConfigHandler isotopic_data_handler(
+  libXpertMassCore::IsotopicDataUserConfigHandler isotopic_data_handler(
     msp_polChemDef->getIsotopicDataSPtr());
 
   bool res = false;
@@ -347,7 +349,7 @@ IsotopeDefDlg::saveAs()
   // case that has to be user config handler, we do not want the library
   // handler.
 
-  libXpertMass::IsotopicDataUserConfigHandler isotopic_data_handler(
+  libXpertMassCore::IsotopicDataUserConfigHandler isotopic_data_handler(
     msp_polChemDef->getIsotopicDataSPtr());
 
   // Provide the user with the opportunity to define a file name.
@@ -401,13 +403,13 @@ IsotopeDefDlg::setupIsotopicDataTableView()
     {
 
       msp_polChemDef->setIsotopicDataSPtr(
-        std::make_shared<libXpertMass::IsotopicData>());
+        std::make_shared<libXpertMassCore::IsotopicData>());
 
       // Silently load the IsoSpec's isotopic data tables.  For this, we can can
       // allocate the proper handler but first we need to allocate the isotopic
       // data.
 
-      libXpertMass::IsotopicDataLibraryHandler isotopic_data_handler(
+      libXpertMassCore::IsotopicDataLibraryHandler isotopic_data_handler(
         msp_polChemDef->getIsotopicDataSPtr());
 
       if(!isotopic_data_handler.loadData())
@@ -416,8 +418,9 @@ IsotopeDefDlg::setupIsotopicDataTableView()
 
   // Now we can link the data to the table view model.
 
-  mpa_isotopicDataTableViewModel = new libXpertMassGui::IsotopicDataTableViewModel(
-    this, msp_polChemDef->getIsotopicDataSPtr());
+  mpa_isotopicDataTableViewModel =
+    new libXpertMassGui::IsotopicDataTableViewModel(
+      this, msp_polChemDef->getIsotopicDataSPtr());
 
   m_ui.isotopicDataTableView->setModel(mpa_isotopicDataTableViewModel);
 
@@ -507,8 +510,8 @@ IsotopeDefDlg::updateMassAndProbDetails(std::set<int> rows_set)
 
   // First make sure the isotopes in the set of selected rows is always the
   // same.
-  std::size_t rows_count = rows_set.size();
-  qDebug() << "The number of selected rows:" << rows_count;
+  std::size_t selected_rows_count = rows_set.size();
+  qDebug() << "The number of selected rows:" << selected_rows_count;
 
   // Start fresh on the line edit widgets.
 
@@ -558,28 +561,35 @@ IsotopeDefDlg::updateMassAndProbDetails(std::set<int> rows_set)
   // isotopes.
 
   // Compute the iterator range for the selected rows.
+  const QList<libXpertMassCore::IsotopeQSPtr> &reference_isotopes =
+    msp_polChemDef->getIsotopicDataSPtr()->getIsotopes();
 
-  libXpertMass::IsotopeCstIterator iter_begin =
-    msp_polChemDef->getIsotopicDataSPtr()->getIsotopes().cbegin() +
-    *rows_set.cbegin();
+  //  The rows_set argument is a set of integers describing the rows being
+  //  selected as their index.
 
-  libXpertMass::IsotopeCstIterator iter_end = iter_begin + rows_set.size();
+  libXpertMassCore::IsotopeListCstIterator the_begin_iterator =
+    reference_isotopes.cbegin() + *(rows_set.cbegin());
 
-  qDebug() << "Distance: " << std::distance(iter_begin, iter_end);
+  libXpertMassCore::IsotopeListCstIterator the_end_iterator =
+    the_begin_iterator + rows_set.size();
 
-  libXpertMass::IsotopeCstIteratorPair iter_pair(iter_begin, iter_end);
+  qDebug() << "Distance: "
+           << std::distance(the_begin_iterator, the_end_iterator);
+
+  libXpertMassCore::IsotopeListCstIteratorPair iter_pair(the_begin_iterator,
+                                                     the_end_iterator);
 
   // Now actually ask the details.
-  std::vector<QString> errors;
+  libXpertMassCore::ErrorList error_list;
 
   double cumulated_probabilities =
-    msp_polChemDef->getIsotopicDataSPtr()->getCumulatedProbabilities(iter_pair,
-                                                                     errors);
-  if(errors.size())
+    msp_polChemDef->getIsotopicDataSPtr()->getCumulatedProbabilities(
+      iter_pair, &error_list);
+  if(error_list.size())
     {
       QString joined_errors;
 
-      for(auto error : errors)
+      for(auto error : error_list)
         joined_errors.append(error + "\n");
 
       QMessageBox::warning(
@@ -595,13 +605,15 @@ IsotopeDefDlg::updateMassAndProbDetails(std::set<int> rows_set)
     QString::number(cumulated_probabilities, 'f', 6));
 
   double mono_mass = msp_polChemDef->getIsotopicDataSPtr()->getMonoMass(
-    libXpertMass::IsotopeCstIteratorPair(iter_begin, iter_end), errors);
+    libXpertMassCore::IsotopeListCstIteratorPair(the_begin_iterator,
+                                             the_end_iterator),
+    &error_list);
 
-  if(errors.size())
+  if(error_list.size())
     {
       QString joined_errors;
 
-      for(auto error : errors)
+      for(auto error : error_list)
         joined_errors.append(error + "\n");
 
       QMessageBox::warning(this,
@@ -619,13 +631,15 @@ IsotopeDefDlg::updateMassAndProbDetails(std::set<int> rows_set)
   m_ui.monoMassLineEdit->setText(QString::number(mono_mass, 'f', 12));
 
   double avg_mass = msp_polChemDef->getIsotopicDataSPtr()->computeAvgMass(
-    libXpertMass::IsotopeCstIteratorPair(iter_begin, iter_end), errors);
+    libXpertMassCore::IsotopeListCstIteratorPair(the_begin_iterator,
+                                             the_end_iterator),
+    &error_list);
 
-  if(errors.size())
+  if(error_list.size())
     {
       QString joined_errors;
 
-      for(auto error : errors)
+      for(auto error : error_list)
         joined_errors.append(error + "\n");
 
       QMessageBox::warning(this,
@@ -643,6 +657,6 @@ IsotopeDefDlg::updateMassAndProbDetails(std::set<int> rows_set)
   m_ui.avgMassLineEdit->setText(QString::number(avg_mass, 'f', 12));
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
