@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include <QMessageBox>
 #include <QApplication>
 #include <QDrag>
+#include <QMenu>
 
 
 /////////////////////// Local includes
@@ -44,7 +45,7 @@
 #include "MassSearchOligomerTableView.hpp"
 #include "MassSearchOligomerTableViewSortProxyModel.hpp"
 
-#include <libXpertMass/Oligomer.hpp>
+#include <MsXpS/libXpertMassCore/Oligomer.hpp>
 
 #include "MassSearchDlg.hpp"
 
@@ -54,7 +55,7 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -105,16 +106,17 @@ MassSearchOligomerTableView::~MassSearchOligomerTableView()
 
 
 void
-MassSearchOligomerTableView::setOligomerList(OligomerList *oligomerList)
+MassSearchOligomerTableView::setOligomerCollection(
+  libXpertMassCore::OligomerCollection *oligomerList)
 {
-  mp_oligomerList = oligomerList;
+  mp_oligomers = oligomerList;
 }
 
 
-const OligomerList *
+const libXpertMassCore::OligomerCollection *
 MassSearchOligomerTableView::oligomerList()
 {
-  return mp_oligomerList;
+  return mp_oligomers;
 }
 
 
@@ -132,27 +134,24 @@ MassSearchOligomerTableView::parentDlg()
   return mp_parentDlg;
 }
 
+// The newly added oligomers are added at index in the OligomerCollection
+// passed as parameter.
 int
-MassSearchOligomerTableView::selectedOligomers(OligomerList *oligomerList,
-                                               int index) const
+MassSearchOligomerTableView::selectedOligomers(
+  libXpertMassCore::OligomerCollection *oligomers_p, std::size_t index) const
 {
-  if(!oligomerList)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+  if(oligomers_p == nullptr)
+    qFatal() << "Programming error.";
 
   int count = 0;
 
-  int localIndex = 0;
+  std::size_t local_index = 0;
 
   // How many oligomers are there in the list passed as argument?
-  int oligomerCount = oligomerList->size();
+  std::size_t oligomer_count = oligomers_p->size();
 
-  if(index > oligomerCount)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
-
-  // If index is -1 , then we are asked to append the oligomers to
-  // the list.
-  if(index == -1)
-    localIndex = oligomerList->size();
+  if(index > oligomer_count)
+    qFatal() << "Programming error.";
 
   // We first have to get the selection model for the proxy model.
 
@@ -200,23 +199,41 @@ MassSearchOligomerTableView::selectedOligomers(OligomerList *oligomerList,
       else
         processedRowList.append(row);
 
-      libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersCstRef().at(row);
 
-      libXpertMass::OligomerSPtr new_oligomer_sp =
-        std::make_shared<libXpertMass::Oligomer>(*oligomer_sp);
+      libXpertMassCore::OligomerSPtr new_oligomer_sp =
+        std::make_shared<libXpertMassCore::Oligomer>(*oligomer_sp);
+
+#ifdef QT_DEBUG
+      // Sanity check:
+      libXpertMassCore::Prop *the_prop = oligomer_sp->prop("SEARCHED_MZ");
+      Q_ASSERT(the_prop);
+      double mzSearchedDouble = *static_cast<const double *>(the_prop->data());
+      qDebug() << "The searched m/z:" << mzSearchedDouble;
+
+      // Sanity check:
+      libXpertMassCore::Prop *the_other_prop = new_oligomer_sp->prop("SEARCHED_MZ");
+      Q_ASSERT(the_other_prop);
+      mzSearchedDouble = *static_cast<const double *>(the_other_prop->data());
+      qDebug() << "The other searched m/z:" << mzSearchedDouble;
+#endif
 
       // Create a NoDeletePointerProp, which might be used later by
       // the user of the list of oligomers to highlight regions in
       // the sequence editor.
 
-      libXpertMass::NoDeletePointerProp *prop = new libXpertMass::NoDeletePointerProp(
-        "SEQUENCE_EDITOR_WND", static_cast<void *>(mp_parentDlg->editorWnd()));
+      libXpertMassCore::NoDeletePointerProp *prop =
+        new libXpertMassCore::NoDeletePointerProp(
+          "SEQUENCE_EDITOR_WND",
+          static_cast<void *>(mp_parentDlg->editorWnd()));
 
       new_oligomer_sp->appendProp(prop);
 
-      oligomerList->insert(localIndex, new_oligomer_sp);
+      oligomers_p->getOligomersRef().insert(
+        oligomers_p->getOligomersRef().begin() + local_index, new_oligomer_sp);
 
-      ++localIndex;
+      ++local_index;
       ++count;
     }
 
@@ -225,120 +242,158 @@ MassSearchOligomerTableView::selectedOligomers(OligomerList *oligomerList,
 
 
 QString *
-MassSearchOligomerTableView::selectedOligomersAsPlainText(
+MassSearchOligomerTableView::describeSelectedOligomersAsPlainText(
   QString delimiter,
   bool withSequence,
   bool forXpertMiner,
-  libXpertMass::MassType massType) const
+  libXpertMassCore::Enums::MassType massType) const
 {
   // Let's get all the currently selected oligomers in one list.
 
-  OligomerList oligomerList;
+  libXpertMassCore::OligomerCollection selected_oligomers;
 
-  // Append the selected oligomers to the empty list.
-  int appendedOligomerCount = selectedOligomers(&oligomerList, -1);
+  // The selected oligomers are *allocated* into the selected_oligomers
+  // OligomerCollection. These oligomers are inserted at index 0 below (which
+  // is like appending, since the collection is empty).
+
+  // As we use them below,  we'll free them by simply erasing them from the
+  // container.
+  std::size_t selected_oligomer_count =
+    selectedOligomers(&selected_oligomers, /* insertion index*/ 0);
 
   // Sanity check
-  if(appendedOligomerCount != oligomerList.size())
+  if(selected_oligomer_count != selected_oligomers.size())
     qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
 
   // Allocate a string in which we describe all the selected items.
 
   QString *text = new QString();
 
-
   // For export to XpertMiner, we only want the masses asked for:
-  // libXpertMass::MassType::MASS_MONO or MASS_AVG. Also, we want the format to be
-  // :
+  // libXpertMassCore::Enums::MassType::MONO or MASS_AVG. Also, we want the format
+  // to be :
 
   // mass <delim> charge <delim> name <delim> coords
 
+  // When exporting for clipboard or file:
+
+  // clang-format off
+  // Searched:  searched mass $ found mono $ found avg $ charge $ name $ index_range $ isModified $
+  // Searched: 827.34700 $ 827.34654 $ 828.05672 $ 1 $ 827.347-z1#1 $ [1-8] $ 0 $
+  // MAMISGMS
+  // clang-format on
+
   if(!forXpertMiner)
     {
       // The header of the column set output only if the data are
       // not for XpertMiner.
-      *text += QObject::tr(
-        "\nSearched mass \t : Name \t :"
-        "Coordinates \t : Error \t :"
-        "Mono \t : Avg \t : Modif\n\n");
+      *text += QString(
+                 "\nSearched mass%1Found Mono%1Found Avg%1Charge%1Name%1Index "
+                 "Range%1isModified?\n\n")
+                 .arg(delimiter);
     }
 
-  while(!oligomerList.isEmpty())
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_iterator =
+    selected_oligomers.getOligomersRef().begin();
+  std::vector<libXpertMassCore::OligomerSPtr>::iterator the_end_iterator =
+    selected_oligomers.getOligomersRef().end();
+
+  while(the_iterator != the_end_iterator)
     {
-      libXpertMass::OligomerSPtr oligomer_sp = oligomerList.takeFirst();
+      // Get the searched mz out of the libXpertMassCore::Prop object.
 
-      // Get the searched mz out of the libXpertMass::Prop object.
-
-      libXpertMass::Prop *prop = oligomer_sp->prop("SEARCHED_MZ");
+      libXpertMassCore::Prop *prop = (*the_iterator)->prop("SEARCHED_MZ");
       Q_ASSERT(prop);
 
       double mzSearchedDouble = *static_cast<const double *>(prop->data());
 
-      QString mzSearched;
-      mzSearched.setNum(mzSearchedDouble, 'f', libXpertMass::OLIGOMER_DEC_PLACES);
+      QString mzSearchedString;
+      mzSearchedString.setNum(
+        mzSearchedDouble, 'f', libXpertMassCore::OLIGOMER_DEC_PLACES);
 
-      prop = 0;
-      prop = oligomer_sp->prop("ERROR_MZ");
+      prop = nullptr;
+      prop = (*the_iterator)->prop("ERROR_MZ");
       Q_ASSERT(prop);
 
       double mzErrorDouble = *static_cast<const double *>(prop->data());
 
-      QString mzError;
-      mzError.setNum(mzErrorDouble, 'f', libXpertMass::OLIGOMER_DEC_PLACES);
+      QString mzErrorString;
+      mzErrorString.setNum(
+        mzErrorDouble, 'f', libXpertMassCore::OLIGOMER_DEC_PLACES);
+
+      // Searched: 827.34700 $ 827.34654 $ 828.05672 $ 1 $ 827.347-z1#1 $ [1-8]
+      // $ 0 $ MAMISGMS
 
       if(!forXpertMiner)
-        *text += QString("\nSearched: %1%2").arg(mzSearched).arg(delimiter);
+        *text += QString("\n%1%2").arg(mzSearchedString).arg(delimiter);
 
-      if(forXpertMiner && massType == libXpertMass::MassType::MASS_AVG)
+      if(forXpertMiner && massType == libXpertMassCore::Enums::MassType::AVG)
         {
         }
       else
         {
-          *text += QString("%1%2")
-                     .arg(oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES))
-                     .arg(delimiter);
+          *text +=
+            QString("%1%2")
+              .arg(
+                (*the_iterator)->getMass(libXpertMassCore::Enums::MassType::MONO),
+                0,
+                'f',
+                libXpertMassCore::OLIGOMER_DEC_PLACES)
+              .arg(delimiter);
         }
 
-      if(forXpertMiner && massType == libXpertMass::MassType::MASS_MONO)
+      if(forXpertMiner && massType == libXpertMassCore::Enums::MassType::MONO)
         {
         }
       else
         {
-          *text += QString("%1%2")
-                     .arg(oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES))
-                     .arg(delimiter);
+          *text +=
+            QString("%1%2")
+              .arg((*the_iterator)->getMass(libXpertMassCore::Enums::MassType::AVG),
+                   0,
+                   'f',
+                   libXpertMassCore::OLIGOMER_DEC_PLACES)
+              .arg(delimiter);
         }
 
-      *text += QString("%1%2").arg(oligomer_sp->charge()).arg(delimiter);
-
-      *text += QString("%1%2").arg(oligomer_sp->name()).arg(delimiter);
-
       *text += QString("%1%2")
-                 .arg(static_cast<libXpertMass::CoordinateList *>(oligomer_sp.get())
-                        ->positionsAsText())
+                 .arg((*the_iterator)->getIonizerCstRef().charge())
                  .arg(delimiter);
 
+      *text += QString("%1%2").arg((*the_iterator)->getName()).arg(delimiter);
+
+      *text +=
+        QString("%1%2")
+          .arg(
+            (*the_iterator)->getIndexRangeCollectionCstRef().positionsAsText())
+          .arg(delimiter);
+
       if(!forXpertMiner)
-        *text += QString("%1%2").arg(oligomer_sp->isModified()).arg(delimiter);
+        *text +=
+          QString("%1%2").arg((*the_iterator)->isModified()).arg(delimiter);
 
       // We cannot export the sequence if data are for XpertMiner
       if(!forXpertMiner && withSequence)
         {
-          QString *sequence = oligomer_sp->monomerText();
+          QString sequence =
+            (*the_iterator)
+              ->getPolymerCstSPtr()
+              ->getSequenceCstRef()
+              .getSequence((*the_iterator)->getIndexRangeCollectionCstRef());
 
-          *text += QString("\n%1").arg(*sequence);
-
-          delete sequence;
+          *text += QString("\n%1").arg(sequence);
         }
 
       // Terminate the stanza
       *text += QString("\n");
 
-      // We can now delete the allocated oligomer, since we do not
-      // need it anymore.
-      oligomer_sp.reset();
+      ++the_iterator;
     }
 
+  // We can now delete all the allocated oligomers, since we do not
+  // need them anymore.
+  selected_oligomers.getOligomersRef().clear();
+
   // Terminate the string with a new line.
   *text += QString("\n");
 
@@ -413,18 +468,29 @@ MassSearchOligomerTableView::currentChanged(const QModelIndex &current,
   // tableView (that list actually belongs to the MassSearchDlg
   // instance.
 
-  libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersCstRef().at(row);
 
   // If the oligomers obtained with the cleavage are old and the
   // sequence has been changed since the cleavage, then the
   // oligomers might point to a sequence element that is no more. We
   // want to avoid such kind of errors.
 
-  if(oligomer_sp->startIndex() >= oligomer_sp->polymer()->size() ||
-     oligomer_sp->endIndex() >= oligomer_sp->polymer()->size())
+  libXpertMassCore::IndexRange *index_range_p =
+    oligomer_sp->getCalcOptionsCstRef()
+      .getIndexRangeCollectionCstRef()
+      .mostInclusiveLeftRightIndexRange();
+
+  qsizetype start_index = index_range_p->m_start;
+  qsizetype stop_index  = index_range_p->m_stop;
+
+  delete index_range_p;
+
+  if(start_index >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+     stop_index >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - Mass Search"),
+                           tr("MassXpert3 - Mass Search"),
                            tr("%1@%2\n"
                               "The monomer indices do not correspond "
                               "to a valid polymer sequence range.\n"
@@ -437,6 +503,7 @@ MassSearchOligomerTableView::currentChanged(const QModelIndex &current,
       return;
     }
 
+
   // Old version, check if it needs to be so, or if the version
   // below is fine.
 
@@ -444,18 +511,17 @@ MassSearchOligomerTableView::currentChanged(const QModelIndex &current,
   //   oligomer->polymer()->monomerText(oligomer->startIndex(),
   //                                    oligomer->endIndex(), true);
 
-  QString *text = oligomer_sp->monomerText();
+  QString text =
+    oligomer_sp->getPolymerCstSPtr()->getSequenceCstRef().getSequence();
 
   // We are getting text for an oligomer; it cannot be empty,
   // because that would mean the oligomer has no monomers. In that
   // case it is not conceivable that the oligomer be in the mass
   // search product list.
 
-  Q_ASSERT(!text->isEmpty());
+  Q_ASSERT(!text.isEmpty());
 
-  mp_parentDlg->updateOligomerSequence(text);
-
-  delete text;
+  mp_parentDlg->updateOligomerSequence(&text);
 
   QTableView::currentChanged(sourceIndex, previous);
 }
@@ -478,13 +544,22 @@ MassSearchOligomerTableView::itemActivated(const QModelIndex &index)
   // tableView (that list actually belongs to the CleavageDlg
   // instance.
 
-  libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(row);
+  libXpertMassCore::OligomerSPtr oligomer_sp =
+    mp_oligomers->getOligomersCstRef().at(row);
 
-  if(oligomer_sp->startIndex() >= oligomer_sp->polymer()->size() ||
-     oligomer_sp->endIndex() >= oligomer_sp->polymer()->size())
+  libXpertMassCore::IndexRange *index_range_p =
+    oligomer_sp->getCalcOptionsCstRef()
+      .getIndexRangeCollectionCstRef()
+      .mostInclusiveLeftRightIndexRange();
+
+  qsizetype start_index = index_range_p->m_start;
+  qsizetype stop_index  = index_range_p->m_stop;
+
+  if(start_index >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size() ||
+     stop_index >= (qsizetype)oligomer_sp->getPolymerCstSPtr()->size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - Mass Search"),
+                           tr("MassXpert3 - Mass Search"),
                            tr("%1@%2\n"
                               "The monomer indices do not correspond "
                               "to a valid polymer sequence range.\n"
@@ -502,8 +577,9 @@ MassSearchOligomerTableView::itemActivated(const QModelIndex &index)
   // Remove the previous selection, so that we can start fresh.
   editorWnd->mpa_editorGraphicsView->resetSelection();
 
-  editorWnd->mpa_editorGraphicsView->setSelection(
-    oligomer_sp->startIndex(), oligomer_sp->endIndex(), false, false);
+  editorWnd->mpa_editorGraphicsView->setSelection(*index_range_p, false, false);
+
+  delete index_range_p;
 }
 
 
@@ -512,21 +588,25 @@ MassSearchOligomerTableView::itemActivated(const QModelIndex &index)
 void
 MassSearchOligomerTableView::copyMono()
 {
-  return copyMassList(libXpertMass::MassType::MASS_MONO);
+  return copyMassList(libXpertMassCore::Enums::MassType::MONO);
 }
 
 
 void
 MassSearchOligomerTableView::copyAvg()
 {
-  return copyMassList(libXpertMass::MassType::MASS_AVG);
+  return copyMassList(libXpertMassCore::Enums::MassType::AVG);
 }
 
 
 void
-MassSearchOligomerTableView::copyMassList(int monoOrAvg)
+MassSearchOligomerTableView::copyMassList(
+  libXpertMassCore::Enums::MassType mass_type)
 {
-  QString massList;
+  if(mass_type == libXpertMassCore::Enums::MassType::BOTH)
+    qFatal() << "Programming error.";
+
+  QString text;
 
 
   // We want to prepare a textual list of masses (either MONO or
@@ -552,27 +632,34 @@ MassSearchOligomerTableView::copyMassList(int monoOrAvg)
       // qDebug() << __FILE__ << __LINE__
       //          << "sourceRow:" << sourceRow;
 
-      libXpertMass::OligomerSPtr oligomer_sp = mp_oligomerList->at(sourceRow);
+      libXpertMassCore::OligomerSPtr oligomer_sp =
+        mp_oligomers->getOligomersCstRef().at(sourceRow);
 
-      if(monoOrAvg == libXpertMass::MassType::MASS_MONO)
-        massList += oligomer_sp->monoString(libXpertMass::OLIGOMER_DEC_PLACES);
-      else if(monoOrAvg == libXpertMass::MassType::MASS_AVG)
-        massList += oligomer_sp->avgString(libXpertMass::OLIGOMER_DEC_PLACES);
+      if(mass_type == libXpertMassCore::Enums::MassType::MONO)
+        text += QString::number(
+          oligomer_sp->getMass(libXpertMassCore::Enums::MassType::MONO),
+          'f',
+          libXpertMassCore::OLIGOMER_DEC_PLACES);
+      else if(mass_type == libXpertMassCore::Enums::MassType::AVG)
+        text += QString::number(
+          oligomer_sp->getMass(libXpertMassCore::Enums::MassType::AVG),
+          'f',
+          libXpertMassCore::OLIGOMER_DEC_PLACES);
       else
-        qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+        qFatal() << "Programming error.";
 
       // End the mass item with a new line.
-      massList += "\n";
+      text += "\n";
     }
 
-  if(massList.isEmpty())
+  if(text.isEmpty())
     return;
 
   QClipboard *clipboard = QApplication::clipboard();
 
-  clipboard->setText(massList, QClipboard::Clipboard);
+  clipboard->setText(text, QClipboard::Clipboard);
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
