@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,25 +30,32 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef MASS_SEARCH_OLIGOMER_TABLE_VIEW_HPP
-#define MASS_SEARCH_OLIGOMER_TABLE_VIEW_HPP
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
 #include <QTableView>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/OligomerCollection.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include "../nongui/OligomerList.hpp"
 
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class MassSearchDlg;
@@ -57,10 +64,36 @@ class MassSearchOligomerTableView : public QTableView
 {
   Q_OBJECT
 
-  private:
+  public:
+  MassSearchOligomerTableView(QWidget *parent = 0);
+  ~MassSearchOligomerTableView();
+
+  void setOligomerCollection(libXpertMassCore::OligomerCollection *);
+  const libXpertMassCore::OligomerCollection *oligomerList();
+
+  void setParentDlg(MassSearchDlg *);
+  MassSearchDlg *parentDlg();
+
+  int selectedOligomers(libXpertMassCore::OligomerCollection *,
+                        std::size_t index = 0) const;
+  QString *describeSelectedOligomersAsPlainText(
+    QString                       = "$",
+    bool                          = false,
+    bool                          = false,
+    libXpertMassCore::Enums::MassType = libXpertMassCore::Enums::MassType::MONO) const;
+
+  void mousePressEvent(QMouseEvent *);
+  void mouseMoveEvent(QMouseEvent *);
+
+  public slots:
+  void itemActivated(const QModelIndex &);
+  void copyMono();
+  void copyAvg();
+
+  protected:
   MassSearchDlg *mp_parentDlg;
 
-  OligomerList *mp_oligomerList;
+  libXpertMassCore::OligomerCollection *mp_oligomers;
 
   // For drag operations.
   QPoint m_dragStartPos;
@@ -76,39 +109,11 @@ class MassSearchOligomerTableView : public QTableView
   QAction *copyAvgAct;
   /////////
 
-  protected:
+
   void currentChanged(const QModelIndex &, const QModelIndex &);
-  void copyMassList(int);
-
-  public:
-  MassSearchOligomerTableView(QWidget *parent = 0);
-  ~MassSearchOligomerTableView();
-
-  void setOligomerList(OligomerList *);
-  const OligomerList *oligomerList();
-
-  void setParentDlg(MassSearchDlg *);
-  MassSearchDlg *parentDlg();
-
-  int selectedOligomers(OligomerList *, int = 0) const;
-  QString *selectedOligomersAsPlainText(
-    QString         = "$",
-    bool            = false,
-    bool            = false,
-    libXpertMass::MassType = libXpertMass::MassType::MASS_MONO) const;
-
-  void mousePressEvent(QMouseEvent *);
-  void mouseMoveEvent(QMouseEvent *);
-
-  public slots:
-  void itemActivated(const QModelIndex &);
-  void copyMono();
-  void copyAvg();
+  void copyMassList(libXpertMassCore::Enums::MassType mass_type);
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // MASS_SEARCH_OLIGOMER_TABLE_VIEW_HPP
