@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,26 +30,33 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef MASS_SEARCH_OLIGOMER_TABLE_VIEW_MODEL_HPP
-#define MASS_SEARCH_OLIGOMER_TABLE_VIEW_MODEL_HPP
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
 #include <QTableView>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Oligomer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include <libXpertMass/Oligomer.hpp>
 #include "MassSearchDlg.hpp"
 
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 enum
@@ -69,16 +76,12 @@ class MassSearchOligomerTableViewModel : public QAbstractTableModel
 {
   Q_OBJECT
 
-  private:
-  OligomerList *mp_oligomerList;
-  MassSearchDlg *mp_parentDlg;
-  MassSearchOligomerTableView *mp_tableView;
-
   public:
-  MassSearchOligomerTableViewModel(OligomerList *, QObject *);
+  MassSearchOligomerTableViewModel(libXpertMassCore::OligomerCollection *,
+                                   QObject *);
   ~MassSearchOligomerTableViewModel();
 
-  const OligomerList *oligomerList();
+  const libXpertMassCore::OligomerCollection *oligomerList();
   MassSearchDlg *parentDlg();
 
   void setTableView(MassSearchOligomerTableView *);
@@ -92,14 +95,16 @@ class MassSearchOligomerTableViewModel : public QAbstractTableModel
   QVariant data(const QModelIndex &parent = QModelIndex(),
                 int role                  = Qt::DisplayRole) const;
 
-  int addOligomer(libXpertMass::Oligomer *);
-  int addOligomers(const OligomerList &);
-  int removeOligomers(int, int);
+  int addOligomer(libXpertMassCore::Oligomer *oligomer_p);
+  int addOligomers(const libXpertMassCore::OligomerCollection &oligomer_collection);
+  int removeOligomers(std::size_t start_indes, std::size_t stop_index);
+
+  private:
+  libXpertMassCore::OligomerCollection *mp_oligomers;
+  MassSearchDlg *mp_parentDlg;
+  MassSearchOligomerTableView *mp_tableView;
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // MASS_SEARCH_OLIGOMER_TABLE_VIEW_MODEL_HPP
