@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,8 @@
 namespace MsXpS
 {
 
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class MassSearchOligomerTableViewSortProxyModel : public QSortFilterProxyModel
@@ -53,12 +52,12 @@ class MassSearchOligomerTableViewSortProxyModel : public QSortFilterProxyModel
   bool sortCoordinates(QString, QString) const;
   bool sortName(QString, QString) const;
 
-  double m_searchedFilter;
+  double m_searchedFilter = 0;
 
-  double m_errorFilter;
-  double m_monoFilter;
-  double m_avgFilter;
-  double m_tolerance;
+  double m_errorFilter = 0;
+  double m_monoFilter  = 0;
+  double m_avgFilter   = 0;
+  double m_tolerance   = 0;
 
   protected:
   bool lessThan(const QModelIndex &, const QModelIndex &) const;
@@ -78,7 +77,7 @@ class MassSearchOligomerTableViewSortProxyModel : public QSortFilterProxyModel
   void applyNewFilter();
 };
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
 
