@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,14 +30,23 @@
  * END software license
  */
 
+/////////////////////// stdlib includes
+
 
 /////////////////////// Qt includes
 #include <QMessageBox>
 #include <QSettings>
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/Modif.hpp>
+#include <MsXpS/libXpertMassCore/CrossLinker.hpp>
+
+/////////////////////// libXpertMassGui includes
 
-/////////////////////// libmass includes
-#include <libXpertMass/PolChemDef.hpp>
 
 /////////////////////// Local includes
 #include "ModifDefDlg.hpp"
@@ -47,32 +56,31 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-ModifDefDlg::ModifDefDlg(libXpertMass::PolChemDefSPtr polChemDefSPtr,
-                         PolChemDefWnd *polChemDefWnd,
-                         const QString &settingsFilePath,
-                         const QString &applicationName,
+ModifDefDlg::ModifDefDlg(libXpertMassCore::PolChemDefSPtr pol_chem_def_sp,
+                         PolChemDefWnd *pol_chem_def_wnd_p,
+                         const QString &settings_file_path,
+                         const QString &application_name,
                          const QString &description)
-  : AbstractPolChemDefDependentDlg(polChemDefSPtr,
-                                   polChemDefWnd,
-                                   settingsFilePath,
+  : AbstractPolChemDefDependentDlg(pol_chem_def_sp,
+                                   pol_chem_def_wnd_p,
+                                   settings_file_path,
                                    "ModifDefDlg",
-                                   applicationName,
-                                   description)
+                                   application_name,
+                                   description),
+    mref_modifs(pol_chem_def_sp->getModifsRef())
 {
-  Q_ASSERT(polChemDefSPtr);
-  msp_polChemDef = polChemDefSPtr;
-  mp_list        = polChemDefSPtr->modifListPtr();
-
-  Q_ASSERT(polChemDefWnd);
-  mp_polChemDefWnd = polChemDefWnd;
+  if(msp_polChemDef == nullptr)
+    qFatal() << "Programming error. Pointer cannot be nullptr.";
+  if(pol_chem_def_wnd_p == nullptr)
+    qFatal() << "Programming error. Pointer cannot be nullptr.";
 
   if(!initialize())
     {
-      qDebug() << "Failed to initialize the modif definition window";
+      qCritical() << "Failed to initialize the modif definition window";
     }
 }
 
@@ -89,7 +97,7 @@ ModifDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
   // close==destruction. Thus we only hide the dialog remembering its
   // position and size.
 
-  mp_polChemDefWnd->m_ui.modifPushButton->setChecked(false);
+  mp_pol_chem_def_wnd_p->m_ui.modifPushButton->setChecked(false);
 
   readSettings();
 }
@@ -98,7 +106,7 @@ ModifDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
 void
 ModifDefDlg::readSettings()
 {
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -111,7 +119,7 @@ void
 ModifDefDlg::writeSettings()
 {
 
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -125,18 +133,17 @@ ModifDefDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Now we need to actually show the window title (that element is empty in
   // m_ui)
   displayWindowTitle();
 
   // Set all the modifs to the list widget.
 
-  for(int iter = 0; iter < mp_list->size(); ++iter)
-    {
-      libXpertMass::Modif *modif = mp_list->at(iter);
+  for(libXpertMassCore::ModifSPtr modif_sp : mref_modifs)
+    m_ui.modifListWidget->addItem(modif_sp->getName());
 
-      m_ui.modifListWidget->addItem(modif->name());
-    }
 
   readSettings();
 
@@ -191,11 +198,11 @@ ModifDefDlg::addModifPushButtonClicked()
   // Returns -1 if the list is empty.
   int index = m_ui.modifListWidget->currentRow();
 
-  libXpertMass::Modif *newModif =
-    new libXpertMass::Modif(msp_polChemDef, tr("Type Name"), tr("Type Formula"));
+  libXpertMassCore::ModifSPtr modif_sp = std::make_shared<libXpertMassCore::Modif>(
+    msp_polChemDef, "Type Name", "Type Formula");
 
-  mp_list->insert(index, newModif);
-  m_ui.modifListWidget->insertItem(index, newModif->name());
+  mref_modifs.insert(mref_modifs.begin() + index, modif_sp);
+  m_ui.modifListWidget->insertItem(index, modif_sp->getName());
 
   setModified();
 
@@ -226,39 +233,37 @@ ModifDefDlg::removeModifPushButtonClicked()
   // Get a pointer to the modification, as we have to make one
   // check.
 
-  libXpertMass::Modif *modif = mp_list->at(index);
-  Q_ASSERT(modif);
+  libXpertMassCore::ModifSPtr modif_sp = mref_modifs.at(index);
+  Q_ASSERT(modif_sp);
 
   // It is essential that we check if the modification being removed
   // is already used or not in a cross-linker, as it would not be
   // possible to tolerate this.
 
-  const QList<libXpertMass::CrossLinker *> &list = msp_polChemDef->crossLinkerList();
+  const std::vector<libXpertMassCore::CrossLinkerSPtr>::const_iterator
+    the_iterator = std::find_if(
+      msp_polChemDef->getCrossLinkersCstRef().begin(),
+      msp_polChemDef->getCrossLinkersCstRef().end(),
+      [modif_sp](const libXpertMassCore::CrossLinkerSPtr cross_linker_sp) {
+        return cross_linker_sp->hasModif(modif_sp->getName());
+      });
 
-  for(int iter = 0; iter < list.size(); ++iter)
+  if(the_iterator != msp_polChemDef->getCrossLinkersCstRef().end())
     {
-      libXpertMass::CrossLinker *crossLinker = list.at(iter);
+      QMessageBox::warning(this,
+                           "MassXpert - Modif definition",
+                           QString(tr("A modif with same name is used"
+                                      " in cross-linker %1. Cannot remove it.")
+                                     .arg((*the_iterator)->getName())),
+                           QMessageBox::Ok);
 
-      if(crossLinker->hasModif(modif->name()) != -1)
-        {
-          QString message(tr("A modif with same name is used"
-                             " in cross-linker %1.")
-                            .arg(crossLinker->name()));
-
-          QMessageBox::warning(this,
-                               tr("massXpert - libXpertMass::Modif definition"),
-                               message,
-                               QMessageBox::Ok);
-
-          return;
-        }
+      return;
     }
 
   // At this point we know we can delete the modif, after we have
   // removed it from the modification list of the polymer chemistry
   // definition.
-  mp_list->removeAt(index);
-  delete modif;
+  mref_modifs.erase(mref_modifs.begin() + index);
 
   // And the item.
   QListWidgetItem *item = m_ui.modifListWidget->takeItem(index);
@@ -301,11 +306,11 @@ ModifDefDlg::moveUpModifPushButtonClicked()
   // Get the index of the modif and the modif itself.
   int index = m_ui.modifListWidget->currentRow();
 
+  auto it = mref_modifs.begin() + index;
   // If the item is already at top of list, do nothing.
-  if(!index)
+  if(it == mref_modifs.begin())
     return;
-
-  mp_list->move(index, index - 1);
+  std::iter_swap(it, it - 1); // Swap with the previous element
 
   QListWidgetItem *item = m_ui.modifListWidget->takeItem(index);
 
@@ -332,11 +337,11 @@ ModifDefDlg::moveDownModifPushButtonClicked()
   // Get the index of the modif and the modif itself.
   int index = m_ui.modifListWidget->currentRow();
 
+  auto it = mref_modifs.begin() + index;
   // If the item is already at bottom of list, do nothing.
-  if(index == m_ui.modifListWidget->count() - 1)
+  if(it == mref_modifs.end())
     return;
-
-  mp_list->move(index, index + 1);
+  std::iter_swap(it, it + 1); // Swap with the next element
 
   QListWidgetItem *item = m_ui.modifListWidget->takeItem(index);
   m_ui.modifListWidget->insertItem(index + 1, item);
@@ -362,7 +367,7 @@ ModifDefDlg::applyModifPushButtonClicked()
   // Get the index of the current modif item.
   int index = m_ui.modifListWidget->currentRow();
 
-  libXpertMass::Modif *modif = mp_list->at(index);
+  libXpertMassCore::ModifSPtr modif_sp = mref_modifs.at(index);
 
   // We do not want more than one modif by the same name or the same
   // symbol.
@@ -379,12 +384,20 @@ ModifDefDlg::applyModifPushButtonClicked()
   // modif is not the one that is current in the modif list, then we
   // are making a double entry, which is not allowed.
 
-  int nameRes = libXpertMass::Modif::isNameInList(editName, *mp_list);
-  if(nameRes != -1 && nameRes != index)
+  std::vector<libXpertMassCore::ModifSPtr>::iterator the_iterator = std::find_if(
+    mref_modifs.begin(),
+    mref_modifs.end(),
+    [editName](const libXpertMassCore::ModifSPtr iter_cross_linker_sp) {
+      return iter_cross_linker_sp->getName() == editName;
+    });
+
+  if(the_iterator != mref_modifs.end() &&
+     std::distance(mref_modifs.begin(), the_iterator) != index)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
-                           tr("A modif with same name exists already."),
+                           tr("MassXpert3 - Modif definition"),
+                           tr("A modification with same name "
+                              "exists already."),
                            QMessageBox::Ok);
       return;
     }
@@ -393,10 +406,10 @@ ModifDefDlg::applyModifPushButtonClicked()
   // because the name of the modif cannot clash with the name of a
   // crossLinker when setting the graphical vignette of the
   // modif.
-  if(msp_polChemDef->referenceCrossLinkerByName(editName))
+  if(msp_polChemDef->getCrossLinkerCstSPtrByName(editName))
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
+                           tr("MassXpert3 - Modif definition"),
                            tr("The name of the modification is already "
                               "used by a cross-linker."),
                            QMessageBox::Ok);
@@ -405,15 +418,16 @@ ModifDefDlg::applyModifPushButtonClicked()
 
   // At this point, validate the formula:
 
-  libXpertMass::Formula formula(editFormula);
+  libXpertMassCore::Formula formula(editFormula);
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
-  if(!formula.validate(isotopic_data_csp))
+  libXpertMassCore::ErrorList error_list;
+  if(!formula.validate(isotopic_data_csp, &error_list))
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
+                           tr("MassXpert3 - Modif definition"),
                            tr("The formula failed to validate."),
                            QMessageBox::Ok);
       return;
@@ -424,41 +438,30 @@ ModifDefDlg::applyModifPushButtonClicked()
   if(maxCount <= 0)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
+                           tr("MassXpert3 - Modif definition"),
                            tr("The max count value is invalid."),
                            QMessageBox::Ok);
       return;
     }
 
-  modif->setMaxCount(maxCount);
+  modif_sp->setMaxCount(maxCount);
+  modif_sp->setName(editName);
+  modif_sp->setFormula(editFormula);
+  modif_sp->setTargets(editTargets);
 
-  // Validate the target list.
-
-  libXpertMass::Modif tempModif(*modif);
-
-  tempModif.setTargets(editTargets);
-
-  if(!tempModif.validateTargets())
+  if(!modif_sp->isValid())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
+                           tr("MassXpert3 - Modif definition"),
                            tr("The targets string is invalid."),
                            QMessageBox::Ok);
       return;
     }
 
-  modif->setTargets(tempModif.targets());
-
-
-  // Finally we can update the modif's data:
-
-  modif->setName(editName);
-  modif->setFormula(editFormula);
-
   // Update the list widget item.
 
   QListWidgetItem *item = m_ui.modifListWidget->currentItem();
-  item->setData(Qt::DisplayRole, modif->name());
+  item->setData(Qt::DisplayRole, modif_sp->getName());
 
   setModified();
 }
@@ -467,58 +470,55 @@ ModifDefDlg::applyModifPushButtonClicked()
 bool
 ModifDefDlg::validatePushButtonClicked()
 {
-  QStringList errorList;
-
   // All we have to do is validate the modif definition. For that we'll
   // go in the listwidget items one after the other and make sure that
   // everything is fine and that colinearity is perfect between the
   // modif list and the listwidget.
 
-  int itemCount = m_ui.modifListWidget->count();
+  std::size_t modifs_count = m_ui.modifListWidget->count();
 
-  if(itemCount != mp_list->size())
+  if(modifs_count != mref_modifs.size())
     {
-      errorList << QString(
-        tr("\nThe number of modifs in the list widget \n"
-           "and in the list of modifs is not "
-           "identical.\n"));
-
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
-                           errorList.join("\n"),
+                           "MassXpert - Modif definition",
+                           "\nThe number of modifs in the list widget \n"
+                           "and in the list of modifs is not "
+                           "identical.\n",
                            QMessageBox::Ok);
       return false;
     }
 
-  for(int iter = 0; iter < mp_list->size(); ++iter)
+  libXpertMassCore::ErrorList error_list;
+
+  for(std::size_t iter = 0; iter < mref_modifs.size(); ++iter)
     {
       QListWidgetItem *item = m_ui.modifListWidget->item(iter);
 
-      libXpertMass::Modif *modif = mp_list->at(iter);
+      libXpertMassCore::ModifSPtr modif_sp = mref_modifs.at(iter);
 
-      if(item->text() != modif->name())
-        errorList << QString(tr("\nModif at index %1 has not the same\n"
-                                "name as the list widget item at the\n"
-                                "same index.\n")
+      if(item->text() != modif_sp->getName())
+        error_list.push_back(QString("\nModif at index %1 has not the same\n"
+                                     "name as the list widget item at the\n"
+                                     "same index.\n")
                                .arg(iter));
 
-      if(!modif->validate())
-        errorList << QString(
-          tr("\nModif at index %1  failed to validate.\n").arg(iter));
+      modif_sp->validate(&error_list);
     }
 
-  if(errorList.size())
+  if(error_list.size())
     {
-      QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
-                           errorList.join("\n"),
-                           QMessageBox::Ok);
+      QMessageBox::warning(
+        this,
+        "MassXpert - Modif definition",
+        QString("The modification failed to validate with errors: %1")
+          .arg(libXpertMassCore::Utils::joinErrorList(error_list, "\n")),
+        QMessageBox::Ok);
       return false;
     }
   else
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Modif definition"),
+                           tr("MassXpert3 - Modif definition"),
                            ("Validation: success\n"),
                            QMessageBox::Ok);
     }
@@ -542,23 +542,22 @@ ModifDefDlg::modifListWidgetItemSelectionChanged()
   // Get the index of the current modif.
   int index = m_ui.modifListWidget->currentRow();
 
-  libXpertMass::Modif *modif = mp_list->at(index);
-  Q_ASSERT(modif);
+  libXpertMassCore::ModifSPtr modif_sp = mref_modifs.at(index);
 
   // Set the data of the modif to their respective widgets.
-  updateModifDetails(modif);
+  updateModifDetails(modif_sp);
 }
 
 
 void
-ModifDefDlg::updateModifDetails(libXpertMass::Modif *modif)
+ModifDefDlg::updateModifDetails(libXpertMassCore::ModifSPtr modif_sp)
 {
-  if(modif)
+  if(modif_sp != nullptr)
     {
-      m_ui.nameLineEdit->setText(modif->name());
-      m_ui.formulaLineEdit->setText(modif->formula());
-      m_ui.targetLineEdit->setText(modif->targets());
-      m_ui.maxCountSpinBox->setValue(modif->maxCount());
+      m_ui.nameLineEdit->setText(modif_sp->getName());
+      m_ui.formulaLineEdit->setText(modif_sp->getFormula());
+      m_ui.targetLineEdit->setText(modif_sp->getTargets());
+      m_ui.maxCountSpinBox->setValue(modif_sp->getMaxCount());
     }
   else
     clearAllDetails();
@@ -582,6 +581,6 @@ ModifDefDlg::validate()
   return validatePushButtonClicked();
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
