@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,25 +33,25 @@
 
 /////////////////////// Local includes
 #include "MonomerCodeEvaluator.hpp"
-#include <libXpertMass/PolChemDef.hpp>
-#include <libXpertMass/Monomer.hpp>
-#include <libXpertMass/Polymer.hpp>
-#include <libXpertMass/PolChemDefEntity.hpp>
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
+#include <MsXpS/libXpertMassCore/Polymer.hpp>
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-MonomerCodeEvaluator::MonomerCodeEvaluator(libXpertMass::Polymer *polymer,
-                                           SequenceEditorWnd *wnd,
-                                           QLineEdit *elab,
-                                           QLineEdit *error)
-  : mp_polymer(polymer),
-    mp_editorWnd(wnd),
+MonomerCodeEvaluator::MonomerCodeEvaluator(
+  libXpertMassCore::PolymerQSPtr polymer_sp,
+  SequenceEditorWnd *seq_editor_wnd_p,
+  QLineEdit *elab,
+  QLineEdit *error)
+  : msp_polymer(polymer_sp),
+    mp_editorWnd(seq_editor_wnd_p),
     m_elabCodeLineEdit(elab),
     m_errorCodeLineEdit(error)
 {
@@ -85,22 +85,22 @@ MonomerCodeEvaluator::setEdits(QLineEdit *elab, QLineEdit *error)
 bool
 MonomerCodeEvaluator::evaluateCode(const QString &code)
 {
-  libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr =
-    mp_polymer->getPolChemDefCstSPtr();
+  libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp =
+    msp_polymer->getPolChemDefCstSPtr();
 
-  const QList<libXpertMass::Monomer *> refList = polChemDefCstSPtr->monomerList();
+  //  We just want to make sure that the code is known to the polymer chemistry
+  //  definition.
+  libXpertMassCore::MonomerCstSPtr monomer_csp =
+    pol_chem_def_csp->getMonomerCstSPtrByCode(code);
 
-  libXpertMass::Monomer *monomer =
-    new libXpertMass::Monomer(polChemDefCstSPtr, "NOT_SET");
-
-  int ret = -1;
-  ret     = libXpertMass::Monomer::isCodeInList(code, refList, monomer);
-  if(ret == -1)
-    qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+  if(monomer_csp == nullptr)
+    qFatal() << "Programming error. Monomer code is not known.";
 
   mp_editorWnd->mpa_editorGraphicsView->removeSelectedOligomer();
 
-  mp_editorWnd->mpa_editorGraphicsView->insertMonomerAtPoint(monomer);
+  //  The monomer will be allocated by libXpertMassCore::Sequence anew as a
+  //  MonomerSPtr on the basis of the reference to the Monomer.
+  mp_editorWnd->mpa_editorGraphicsView->insertMonomerAtPoint(*monomer_csp);
 
   mp_editorWnd->clearCompletionsListSelection();
 
@@ -137,19 +137,16 @@ MonomerCodeEvaluator::reportCompletions()
   if(!ret)
     return true;
 
-  for(int iter = 0; iter < m_completionsList.size(); ++iter)
-    {
-      int index = m_completionsList.at(iter);
+  for(int index : m_completionsList)
+    mp_editorWnd->completionsListSelectAt(index);
 
-      mp_editorWnd->completionsListSelectAt(index);
-    }
 
   return true;
 }
 
 
 bool
-MonomerCodeEvaluator::newKey(QString key)
+MonomerCodeEvaluator::newKey(QString &key)
 {
   QChar::Category category = key.at(0).category();
 
@@ -167,12 +164,15 @@ MonomerCodeEvaluator::newKey(QString key)
 bool
 MonomerCodeEvaluator::upperCaseChar(QChar qchar)
 {
-  libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr =
-    mp_polymer->getPolChemDefCstSPtr();
+  //  An uppercase character was entered. By definition,  that means that
+  //  a new monomer code is being started. If there is the elaborating code
+  //  is not empty,  then this is an error.
 
-  const QList<libXpertMass::Monomer *> refList = polChemDefCstSPtr->monomerList();
+  libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp =
+    msp_polymer->getPolChemDefCstSPtr();
 
-  // Start of a new code. There should be nothing in m_elabCode.
+  const std::vector<libXpertMassCore::MonomerSPtr> &reference_monomers =
+    pol_chem_def_csp->getMonomersCstRef();
 
   if(!m_elabCode.isEmpty())
     {
@@ -197,17 +197,17 @@ MonomerCodeEvaluator::upperCaseChar(QChar qchar)
     }
   else if(autoCompletions == 1)
     {
-      int index                 = m_completionsList.at(0);
-      libXpertMass::Monomer *monomer = refList.at(index);
-      QString code              = monomer->code();
+      int index                            = m_completionsList.at(0);
+      libXpertMassCore::MonomerSPtr monomer_sp = reference_monomers.at(index);
+      QString code                         = monomer_sp->getCode();
 
-      // The code was from completion monomers, so they come right
+      // The code was from completion monomer_sps, so they come right
       // from the polymer chemistry defintion: impossible that
       // this code does not evaluate.
       bool ret = false;
       ret      = evaluateCode(code);
       if(ret == 0)
-        qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+        qFatal() << "Programming error.";
 
       m_evalCode.clear();
       m_elabCode.clear();
@@ -217,7 +217,7 @@ MonomerCodeEvaluator::upperCaseChar(QChar qchar)
     }
   else if(autoCompletions > 1)
     {
-      if(m_elabCode.size() >= polChemDefCstSPtr->codeLength())
+      if(m_elabCode.size() >= pol_chem_def_csp->getCodeLength())
         {
           QString msg = QString(tr("Bad char: %1.")).arg(qchar);
           reportError(msg);
@@ -242,9 +242,11 @@ MonomerCodeEvaluator::upperCaseChar(QChar qchar)
 bool
 MonomerCodeEvaluator::lowerCaseChar(QChar qchar)
 {
-  libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr =
-    mp_polymer->getPolChemDefCstSPtr();
-  const QList<libXpertMass::Monomer *> refList = polChemDefCstSPtr->monomerList();
+  libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp =
+    msp_polymer->getPolChemDefCstSPtr();
+
+  const std::vector<libXpertMassCore::MonomerSPtr> &reference_monomers =
+    pol_chem_def_csp->getMonomersCstRef();
 
   // There should be something in m_elabCode.
 
@@ -271,9 +273,9 @@ MonomerCodeEvaluator::lowerCaseChar(QChar qchar)
     }
   else if(autoCompletions == 1)
     {
-      int index                 = m_completionsList.at(0);
-      libXpertMass::Monomer *monomer = refList.at(index);
-      QString code              = monomer->code();
+      int index                            = m_completionsList.at(0);
+      libXpertMassCore::MonomerSPtr monomer_sp = reference_monomers.at(index);
+      QString code                         = monomer_sp->getCode();
 
       // The code was from completion monomers, so they come right
       // from the polymer chemistry defintion: impossible that
@@ -292,7 +294,7 @@ MonomerCodeEvaluator::lowerCaseChar(QChar qchar)
     }
   else if(autoCompletions > 1)
     {
-      if(m_elabCode.size() >= polChemDefCstSPtr->codeLength())
+      if(m_elabCode.size() >= pol_chem_def_csp->getCodeLength())
         {
           QString msg = QString(tr("Bad char: %1.")).arg(qchar);
           reportError(msg);
@@ -314,18 +316,20 @@ MonomerCodeEvaluator::lowerCaseChar(QChar qchar)
 int
 MonomerCodeEvaluator::autoComplete(QString &code)
 {
-  libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr =
-    mp_polymer->getPolChemDefCstSPtr();
-  const QList<libXpertMass::Monomer *> refList = polChemDefCstSPtr->monomerList();
+  libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp =
+    msp_polymer->getPolChemDefCstSPtr();
+
+  const std::vector<libXpertMassCore::MonomerSPtr> &reference_monomers =
+    pol_chem_def_csp->getMonomersCstRef();
 
   m_completionsList.clear();
 
-  for(int iter = 0; iter < refList.size(); ++iter)
+  for(std::size_t iter = 0; iter < reference_monomers.size(); ++iter)
     {
-      libXpertMass::Monomer *monomer = refList.at(iter);
+      libXpertMassCore::MonomerSPtr monomer_sp = reference_monomers.at(iter);
 
-      if(monomer->code().startsWith(code, Qt::CaseSensitive))
-        m_completionsList.append(iter);
+      if(monomer_sp->getCode().startsWith(code, Qt::CaseSensitive))
+        m_completionsList.push_back(iter);
     }
 
   return m_completionsList.size();
@@ -342,19 +346,16 @@ MonomerCodeEvaluator::reportError(QString &msg)
 void
 MonomerCodeEvaluator::debugCompletionsPutStdErr()
 {
-  libXpertMass::PolChemDefCstSPtr polChemDefCstSPtr =
-    mp_polymer->getPolChemDefCstSPtr();
-  const QList<libXpertMass::Monomer *> refList = polChemDefCstSPtr->monomerList();
+  libXpertMassCore::PolChemDefCstSPtr pol_chem_def_csp =
+    msp_polymer->getPolChemDefCstSPtr();
 
-  for(int iter = 0; iter < m_completionsList.size(); ++iter)
-    {
-      int index = m_completionsList.at(iter);
+  const std::vector<libXpertMassCore::MonomerSPtr> &reference_monomers =
+    pol_chem_def_csp->getMonomersCstRef();
 
-      libXpertMass::Monomer *monomer = refList.at(index);
-      qDebug() << "Completion:" << monomer->code();
-    }
+  for(int index : m_completionsList)
+    libXpertMassCore::MonomerSPtr monomer_sp = reference_monomers.at(index);
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
