@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,24 +31,33 @@
  */
 
 
-#ifndef MONOMER_CODE_EVALUATOR_HPP
-#define MONOMER_CODE_EVALUATOR_HPP
+#pragma once
+
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
 #include <QLineEdit>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Polymer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
 #include "SequenceEditorWnd.hpp"
 
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class Polymer;
@@ -60,50 +69,46 @@ class MonomerCodeEvaluator : public QObject
   Q_OBJECT
 
   public:
-  MonomerCodeEvaluator(libXpertMass::Polymer *           = 0,
-                       SequenceEditorWnd * = 0,
-                       QLineEdit *         = 0,
-                       QLineEdit *         = 0);
+  MonomerCodeEvaluator(libXpertMassCore::PolymerQSPtr polymer_sp = nullptr,
+                       SequenceEditorWnd *seq_editor_wnd_p   = nullptr,
+                       QLineEdit *elab                       = nullptr,
+                       QLineEdit *error                      = nullptr);
 
   ~MonomerCodeEvaluator();
 
-  void setSequenceEditorWnd(SequenceEditorWnd *);
-  void setEdits(QLineEdit *, QLineEdit *);
+  void setSequenceEditorWnd(SequenceEditorWnd *wnd);
+  void setEdits(QLineEdit *elab, QLineEdit *error);
 
-  bool evaluateCode(const QString &);
+  bool evaluateCode(const QString &code);
 
   void escapeKey();
-  bool newKey(QString);
+  bool newKey(QString &key);
 
-  bool upperCaseChar(QChar);
-  bool lowerCaseChar(QChar);
+  bool upperCaseChar(QChar qchar);
+  bool lowerCaseChar(QChar qchar);
 
-  int autoComplete(QString &);
+  int autoComplete(QString &code);
   void emptyCompletionsList();
   bool reportCompletions();
 
-  void reportError(QString &);
+  void reportError(QString &msg);
 
   void debugCompletionsPutStdErr();
 
   private:
-  libXpertMass::Polymer *mp_polymer;
+  libXpertMassCore::PolymerQSPtr msp_polymer;
   SequenceEditorWnd *mp_editorWnd;
 
   QLineEdit *m_elabCodeLineEdit;
   QLineEdit *m_errorCodeLineEdit;
 
-  QList<int> m_completionsList;
+  std::vector<int> m_completionsList;
 
 
   QString m_evalCode;
   QString m_elabCode;
-
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // MONOMER_CODE_EVALUATOR_HPP
