@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,9 @@
 #include <QSettings>
 
 
-/////////////////////// libmass includes
-#include <libXpertMass/PolChemDef.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
 
 
 /////////////////////// Local includes
@@ -48,31 +49,25 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-MonomerDefDlg::MonomerDefDlg(libXpertMass::PolChemDefSPtr polChemDefSPtr,
-                             PolChemDefWnd *polChemDefWnd,
-                             const QString &settingsFilePath,
-                             const QString &applicationName,
+MonomerDefDlg::MonomerDefDlg(libXpertMassCore::PolChemDefSPtr pol_chem_def_sp,
+                             PolChemDefWnd *pol_chem_def_wnd_p,
+                             const QString &settings_file_path,
+                             const QString &application_name,
                              const QString &description)
-  : AbstractPolChemDefDependentDlg(polChemDefSPtr,
-                                   polChemDefWnd,
-                                   settingsFilePath,
+  : AbstractPolChemDefDependentDlg(pol_chem_def_sp,
+                                   pol_chem_def_wnd_p,
+                                   settings_file_path,
                                    "MonomerDefDlg",
-                                   applicationName,
-                                   description)
+                                   application_name,
+                                   description),
+    mref_monomers(pol_chem_def_sp->getMonomersRef())
 {
-  mp_list = polChemDefSPtr->monomerListPtr();
-
   if(!initialize())
-    qFatal(
-      "Fatal error at %s@%d. Failed to initialize the %s window. Program "
-      "aborted.",
-      __FILE__,
-      __LINE__,
-      m_wndTypeName.toLatin1().data());
+    qFatal() << "Failed to initialize the dialog window.";
 }
 
 
@@ -88,7 +83,7 @@ MonomerDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
   // close==destruction. Thus we only hide the dialog remembering its
   // position and size.
 
-  mp_polChemDefWnd->m_ui.monomerPushButton->setChecked(false);
+  mp_pol_chem_def_wnd_p->m_ui.monomerPushButton->setChecked(false);
 
   writeSettings();
 }
@@ -97,7 +92,7 @@ MonomerDefDlg::closeEvent([[maybe_unused]] QCloseEvent *event)
 void
 MonomerDefDlg::readSettings()
 {
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -110,7 +105,7 @@ void
 MonomerDefDlg::writeSettings()
 {
 
-  QSettings settings(m_settingsFilePath, QSettings::IniFormat);
+  QSettings settings(m_settings_file_path, QSettings::IniFormat);
 
   settings.beginGroup(m_wndTypeName);
   restoreGeometry(settings.value("geometry").toByteArray());
@@ -124,27 +119,24 @@ MonomerDefDlg::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   // Now we need to actually show the window title (that element is empty in
   // m_ui)
   displayWindowTitle();
 
   // Set all the monomers to the list widget.
-
-  for(int iter = 0; iter < mp_list->size(); ++iter)
-    {
-      libXpertMass::Monomer *monomer = mp_list->at(iter);
-
-      m_ui.monomerListWidget->addItem(monomer->name());
-    }
+  for(libXpertMassCore::MonomerSPtr monomer_sp : mref_monomers)
+    m_ui.monomerListWidget->addItem(monomer_sp->getName());
 
   readSettings();
 
   // Get the monomer code length and create a validator for use later.
 
-  m_ui.codeLengthSpinBox->setValue(msp_polChemDef->codeLength());
+  m_ui.codeLengthSpinBox->setValue(msp_polChemDef->getCodeLength());
 
   QString regExp =
-    QString("[A-Z][a-z]{0,%1}").arg(msp_polChemDef->codeLength() - 1);
+    QString("[A-Z][a-z]{0,%1}").arg(msp_polChemDef->getCodeLength() - 1);
 
   QRegularExpression codeRegExp(regExp);
   QValidator *validator = new QRegularExpressionValidator(codeRegExp, this);
@@ -217,11 +209,12 @@ MonomerDefDlg::addMonomerPushButtonClicked()
   // Returns -1 if the list is empty.
   int index = m_ui.monomerListWidget->currentRow();
 
-  libXpertMass::Monomer *newMonomer = new libXpertMass::Monomer(
-    msp_polChemDef, tr("Type Name"), tr("Type Code"), tr("Type Formula"));
+  libXpertMassCore::MonomerSPtr monomer_sp =
+    std::make_shared<libXpertMassCore::Monomer>(
+      msp_polChemDef, tr("Type Name"), tr("Type Code"), tr("Type Formula"));
 
-  mp_list->insert(index, newMonomer);
-  m_ui.monomerListWidget->insertItem(index, newMonomer->name());
+  mref_monomers.insert(mref_monomers.begin() + index, monomer_sp);
+  m_ui.monomerListWidget->insertItem(index, monomer_sp->getName());
 
   setModified();
 
@@ -253,9 +246,7 @@ MonomerDefDlg::removeMonomerPushButtonClicked()
   QListWidgetItem *item = m_ui.monomerListWidget->takeItem(index);
   delete item;
 
-  libXpertMass::Monomer *monomer = mp_list->takeAt(index);
-  Q_ASSERT(monomer);
-  delete monomer;
+  mref_monomers.erase(mref_monomers.begin() + index);
 
   setModified();
 
@@ -295,11 +286,11 @@ MonomerDefDlg::moveUpMonomerPushButtonClicked()
   // Get the index of the monomer and the monomer itself.
   int index = m_ui.monomerListWidget->currentRow();
 
+  auto it = mref_monomers.begin() + index;
   // If the item is already at top of list, do nothing.
-  if(!index)
+  if(it == mref_monomers.begin())
     return;
-
-  mp_list->move(index, index - 1);
+  std::iter_swap(it, it - 1); // Swap with the previous element
 
   QListWidgetItem *item = m_ui.monomerListWidget->takeItem(index);
 
@@ -327,11 +318,11 @@ MonomerDefDlg::moveDownMonomerPushButtonClicked()
   // Get the index of the monomer and the monomer itself.
   int index = m_ui.monomerListWidget->currentRow();
 
+  auto it = mref_monomers.begin() + index;
   // If the item is already at bottom of list, do nothing.
-  if(index == m_ui.monomerListWidget->count() - 1)
+  if(it == mref_monomers.end())
     return;
-
-  mp_list->move(index, index + 1);
+  std::iter_swap(it, it + 1); // Swap with the next element
 
   QListWidgetItem *item = m_ui.monomerListWidget->takeItem(index);
   m_ui.monomerListWidget->insertItem(index + 1, item);
@@ -358,7 +349,7 @@ MonomerDefDlg::applyMonomerPushButtonClicked()
   // Get the index of the current monomer item.
   int index = m_ui.monomerListWidget->currentRow();
 
-  libXpertMass::Monomer *monomer = mp_list->at(index);
+  libXpertMassCore::MonomerSPtr monomer_sp = mref_monomers.at(index);
 
   // We do not want more than one monomer by the same name or the same
   // symbol.
@@ -371,41 +362,58 @@ MonomerDefDlg::applyMonomerPushButtonClicked()
 
   QString editFormula = m_ui.formulaLineEdit->text();
 
-  // If a monomer is found in the list with any of these two strings,
-  // and that monomer is not the one that is current in the monomer list,
+  // If a monomer is found in the member ref to vector with any of these two
+  // strings, and that monomer is not the one that is current in that vector,
   // then we are making a double entry, which is not allowed.
 
-  int nameRes = libXpertMass::Monomer::isNameInList(editName, *mp_list);
-  if(nameRes != -1 && nameRes != index)
+  std::vector<libXpertMassCore::MonomerSPtr>::iterator the_iterator =
+    std::find_if(mref_monomers.begin(),
+                 mref_monomers.end(),
+                 [editName](const libXpertMassCore::MonomerSPtr iter_monomer_sp) {
+                   return iter_monomer_sp->getName() == editName;
+                 });
+
+  if(the_iterator != mref_monomers.end() &&
+     std::distance(mref_monomers.begin(), the_iterator) != index)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
-                           tr("A monomer with same name exists already."),
+                           tr("MassXpert3 - Monomer definition"),
+                           tr("A monomer with same name "
+                              "exists already."),
                            QMessageBox::Ok);
       return;
     }
 
-  int codeRes = libXpertMass::Monomer::isCodeInList(editCode, *mp_list);
-  if(codeRes != -1 && codeRes != index)
+  the_iterator =
+    std::find_if(mref_monomers.begin(),
+                 mref_monomers.end(),
+                 [editCode](const libXpertMassCore::MonomerSPtr iter_monomer_sp) {
+                   return iter_monomer_sp->getCode() == editCode;
+                 });
+
+  if(the_iterator != mref_monomers.end() &&
+     std::distance(mref_monomers.begin(), the_iterator) != index)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
-                           tr("A monomer with same code exists already."),
+                           tr("MassXpert3 - Monomer definition"),
+                           tr("A monomer with same code "
+                              "exists already."),
                            QMessageBox::Ok);
       return;
     }
 
   // At this point, validate the formula:
 
-  libXpertMass::Formula formula(editFormula);
+  libXpertMassCore::Formula formula(editFormula);
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
+  libXpertMassCore::ErrorList error_list;
 
-  if(!formula.validate(isotopic_data_csp))
+  if(!formula.validate(isotopic_data_csp, &error_list))
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
+                           tr("MassXpert3 - Monomer definition"),
                            tr("The formula failed to validate."),
                            QMessageBox::Ok);
       return;
@@ -413,14 +421,14 @@ MonomerDefDlg::applyMonomerPushButtonClicked()
 
   // Finally we can update the monomer's data:
 
-  monomer->setName(editName);
-  monomer->setCode(editCode);
-  monomer->setFormula(editFormula);
+  monomer_sp->setName(editName);
+  monomer_sp->setCode(editCode);
+  monomer_sp->setFormula(editFormula);
 
   // Update the list widget item.
 
   QListWidgetItem *item = m_ui.monomerListWidget->currentItem();
-  item->setData(Qt::DisplayRole, monomer->name());
+  item->setData(Qt::DisplayRole, monomer_sp->getName());
 
   setModified();
 }
@@ -436,9 +444,9 @@ MonomerDefDlg::validatePushButtonClicked()
   // everything is fine and that colinearity is perfect between the
   // monomer list and the listwidget.
 
-  int itemCount = m_ui.monomerListWidget->count();
+  std::size_t item_count = m_ui.monomerListWidget->count();
 
-  if(itemCount != mp_list->size())
+  if(item_count != mref_monomers.size())
     {
       errorList << QString(
         tr("\nThe number of monomers in the "
@@ -447,33 +455,38 @@ MonomerDefDlg::validatePushButtonClicked()
            "not identical.\n"));
 
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
+                           tr("MassXpert3 - Monomer definition"),
                            errorList.join("\n"),
                            QMessageBox::Ok);
       return false;
     }
 
-  for(int iter = 0; iter < mp_list->size(); ++iter)
+  for(std::size_t iter = 0; iter < mref_monomers.size(); ++iter)
     {
       QListWidgetItem *item = m_ui.monomerListWidget->item(iter);
 
-      libXpertMass::Monomer *monomer = mp_list->at(iter);
+      libXpertMassCore::MonomerSPtr monomer_sp = mref_monomers.at(iter);
 
-      if(item->text() != monomer->name())
+      if(item->text() != monomer_sp->getName())
         errorList << QString(tr("\nMonomer at index %1 has not the same\n"
                                 "name as the list widget item at the\n"
                                 "same index.\n")
                                .arg(iter));
 
-      if(!monomer->validate())
+      MsXpS::libXpertMassCore::ErrorList error_list;
+
+      if(!monomer_sp->validate(&error_list))
         errorList << QString(
-          tr("\nMonomer at index %1  failed to validate.\n").arg(iter));
+                       "\nMonomer at index %1  failed to validate.\n"
+                       "Errors: %2\n\n")
+                       .arg(iter)
+                       .arg(libXpertMassCore::Utils::joinErrorList(error_list));
     }
 
   if(errorList.size())
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
+                           tr("MassXpert3 - Monomer definition"),
                            errorList.join("\n"),
                            QMessageBox::Ok);
       return false;
@@ -481,7 +494,7 @@ MonomerDefDlg::validatePushButtonClicked()
   else
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
+                           tr("MassXpert3 - Monomer definition"),
                            ("Validation: success\n"),
                            QMessageBox::Ok);
     }
@@ -506,17 +519,18 @@ MonomerDefDlg::monomerListWidgetItemSelectionChanged()
   // Get the index of the current monomer.
   int index = m_ui.monomerListWidget->currentRow();
 
-  // Note that this should not occur, I gess, but one I got this crash, so
+  // Note that this should not occur, I guess, but once I got this crash, so
   // test that the returned index is not -1.
   if(index >= 0)
     {
-      libXpertMass::Monomer *monomer = mp_list->at(index);
-      Q_ASSERT(monomer);
+      libXpertMassCore::MonomerSPtr monomer_sp = mref_monomers.at(index);
+      if(monomer_sp == nullptr)
+        qFatal() << "Programming error.";
 
-      //qDebug() << "Currently selected monomer:" << monomer->name();
+      // qDebug() << "Currently selected monomer:" << monomer->name();
 
       // Set the data of the monomer to their respective widgets.
-      updateMonomerDetails(monomer);
+      updateMonomerDetails(monomer_sp);
     }
 }
 
@@ -548,13 +562,13 @@ MonomerDefDlg::codeLengthSpinBoxValueChanged(int value)
 
 
 void
-MonomerDefDlg::updateMonomerDetails(libXpertMass::Monomer *monomer)
+MonomerDefDlg::updateMonomerDetails(libXpertMassCore::MonomerSPtr monomer_sp)
 {
-  if(monomer)
+  if(monomer_sp != nullptr)
     {
-      m_ui.nameLineEdit->setText(monomer->name());
-      m_ui.codeLineEdit->setText(monomer->code());
-      m_ui.formulaLineEdit->setText(monomer->formula());
+      m_ui.nameLineEdit->setText(monomer_sp->getName());
+      m_ui.codeLineEdit->setText(monomer_sp->getCode());
+      m_ui.formulaLineEdit->setText(monomer_sp->getFormula());
     }
   else
     {
@@ -588,17 +602,21 @@ MonomerDefDlg::formulaLineEditTextChanged(const QString &string)
   // masses of the formula. If the computation failed, we set "Bad
   // formula" in the line edit widgets of the masses.
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
 
-  libXpertMass::Formula formula(string);
+  libXpertMassCore::Formula formula(string);
 
-  //qDebug() << "Formula of the monomer:" << string;
+  // qDebug() << "Formula of the monomer:" << string;
 
-  libXpertMass::Ponderable ponderable(0, 0);
+  double mono = 0;
+  double avg  = 0;
+  bool ok     = false;
 
-  if(!formula.accountMasses(isotopic_data_csp, &ponderable))
+  formula.accountMasses(ok, isotopic_data_csp, mono, avg);
+
+  if(!ok)
     {
       m_ui.monoMassLineEdit->setText(tr("Bad formula"));
       m_ui.avgMassLineEdit->setText(tr("Bad formula"));
@@ -607,9 +625,9 @@ MonomerDefDlg::formulaLineEditTextChanged(const QString &string)
     }
 
   m_ui.monoMassLineEdit->setText(
-    ponderable.monoString(libXpertMass::ATOM_DEC_PLACES));
-
-  m_ui.avgMassLineEdit->setText(ponderable.avgString(libXpertMass::ATOM_DEC_PLACES));
+    QString::number(mono, 'f', libXpertMassCore::ATOM_DEC_PLACES));
+  m_ui.avgMassLineEdit->setText(
+    QString::number(avg, 'f', libXpertMassCore::ATOM_DEC_PLACES));
 }
 
 
@@ -627,7 +645,7 @@ MonomerDefDlg::calculateMassDifferences()
   if(!threshold && !ok)
     {
       QMessageBox::warning(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
+                           tr("MassXpert3 - Monomer definition"),
                            tr("Threshold value is not valid"),
                            QMessageBox::Ok);
       return;
@@ -635,30 +653,28 @@ MonomerDefDlg::calculateMassDifferences()
 
   bool isMonoMass = m_ui.monoMassRadioButton->isChecked();
 
-  libXpertMass::MassType monoOrAvg;
+  libXpertMassCore::Enums::MassType monoOrAvg;
 
   if(isMonoMass)
-    monoOrAvg = libXpertMass::MassType::MASS_MONO;
+    monoOrAvg = libXpertMassCore::Enums::MassType::MONO;
   else
-    monoOrAvg = libXpertMass::MassType::MASS_AVG;
+    monoOrAvg = libXpertMassCore::Enums::MassType::AVG;
 
-  QStringList *diffList =
+  QStringList diffList =
     msp_polChemDef->differenceBetweenMonomers(threshold, monoOrAvg);
 
-  if(!diffList)
+  if(!diffList.size())
     return;
 
   QMessageBox::information(this,
-                           tr("massXpert - libXpertMass::Monomer definition"),
-                           diffList->join("\n"),
+                           tr("MassXpert3 - Monomer definition"),
+                           diffList.join("\n"),
                            QMessageBox::Ok);
 
-  delete diffList;
-
   return;
 }
 
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
