@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,54 +39,53 @@
 /////////////////////// Local includes
 #include "MzCalculationTreeViewSortProxyModel.hpp"
 #include "MzCalculationTreeView.hpp"
-#include <libXpertMass/Ponderable.hpp>
 #include "MzCalculationDlg.hpp"
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-  class MzCalculationTreeViewModel;
-  class MzCalculationTreeViewItem;
+class MzCalculationTreeViewModel;
+class MzCalculationTreeViewItem;
 
 
-  MzCalculationTreeView::MzCalculationTreeView(QWidget *parent)
-    : QTreeView(parent)
-  {
+MzCalculationTreeView::MzCalculationTreeView(QWidget *parent)
+  : QTreeView(parent)
+{
 
-    setAlternatingRowColors(true);
-    setAllColumnsShowFocus(true);
-    setSortingEnabled(true);
+  setAlternatingRowColors(true);
+  setAllColumnsShowFocus(true);
+  setSortingEnabled(true);
 
-    QHeaderView *headerView = header();
+  QHeaderView *headerView = header();
 
-    headerView->setSectionsClickable(true);
-    headerView->setSectionsMovable(true);
-  }
+  headerView->setSectionsClickable(true);
+  headerView->setSectionsMovable(true);
+}
 
 
-  MzCalculationTreeView::~MzCalculationTreeView()
-  {
-  }
+MzCalculationTreeView::~MzCalculationTreeView()
+{
+}
 
 
-  MzCalculationDlg *
-  MzCalculationTreeView::getParentDlg()
-  {
-    return mp_parentDlg;
-  }
+MzCalculationDlg *
+MzCalculationTreeView::getParentDlg()
+{
+  return mp_parentDlg;
+}
 
-  void
-  MzCalculationTreeView::setParentDlg(MzCalculationDlg *dlg)
-  {
-    Q_ASSERT(dlg);
-    mp_parentDlg = dlg;
-  }
+void
+MzCalculationTreeView::setParentDlg(MzCalculationDlg *dlg)
+{
+  Q_ASSERT(dlg);
+  mp_parentDlg = dlg;
+}
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
