@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,27 +31,35 @@
  */
 
 
-#ifndef MZ_CALCULATION_TREEVIEW_ITEM_HPP
-#define MZ_CALCULATION_TREEVIEW_ITEM_HPP
+#pragma once
+
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
-#include <QList>
 #include <QVariant>
 #include <QTreeView>
 #include <QMainWindow>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
-#include <libXpertMass/Ionizable.hpp>
+#include "../nongui/globals.hpp"
 
 
 namespace MsXpS
 {
 
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 enum
@@ -65,12 +73,6 @@ enum
 
 class MzCalculationTreeViewItem
 {
-  private:
-  QList<MzCalculationTreeViewItem *> m_childItemsList;
-  QList<QVariant> m_itemData;
-  MzCalculationTreeViewItem *mp_parentItem;
-  libXpertMass::Ionizable *mp_ionizable;
-
   public:
   MzCalculationTreeViewItem(const QList<QVariant> &data,
                             MzCalculationTreeViewItem *parent = 0);
@@ -94,13 +96,16 @@ class MzCalculationTreeViewItem
 
   int row() const;
 
-  void setIonizable(libXpertMass::Ionizable *);
-  libXpertMass::Ionizable *ionizable();
+  void setIon(IonSPtr ion_sp);
+  IonSPtr getIon();
+
+  private:
+  QList<MzCalculationTreeViewItem *> m_childItemsList;
+  QList<QVariant> m_itemData;
+  MzCalculationTreeViewItem *mp_parentItem;
+  IonSPtr msp_ion = nullptr;
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // MZ_CALCULATION_TREEVIEW_ITEM_HPP
