@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,26 +30,34 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef MZ_CALCULATION_TREEVIEW_MODEL_HPP
-#define MZ_CALCULATION_TREEVIEW_MODEL_HPP
+/////////////////////// stdlib includes
 
 
 /////////////////////// Qt includes
 #include <QTreeView>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Ionizer.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+
+
 /////////////////////// Local includes
 #include "MzCalculationTreeViewItem.hpp"
 #include "MzCalculationDlg.hpp"
-#include <libXpertMass/Ionizable.hpp>
+
 
 namespace MsXpS
 {
-
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class MzCalculationTreeViewItem;
@@ -59,15 +67,8 @@ class MzCalculationDlg;
 class MzCalculationTreeViewModel : public QAbstractItemModel
 {
   Q_OBJECT
-
-  private:
-  QList<libXpertMass::Ionizable *> *mp_list;
-  MzCalculationTreeViewItem *mpa_rootItem;
-  QTreeView *mp_treeView;
-  MzCalculationDlg *mp_parentDlg;
-
   public:
-  MzCalculationTreeViewModel(QList<libXpertMass::Ionizable *> *, QObject *);
+  MzCalculationTreeViewModel(std::vector<IonSPtr> *ions_p, QObject *parent);
 
   ~MzCalculationTreeViewModel();
 
@@ -91,13 +92,16 @@ class MzCalculationTreeViewModel : public QAbstractItemModel
 
   void removeAll();
 
-  void addIonizable(libXpertMass::Ionizable *);
+  void addIon(IonSPtr ion_sp);
   bool setupModelData(MzCalculationTreeViewItem *);
+
+  protected:
+  std::vector<IonSPtr> *mp_ions;
+  MzCalculationTreeViewItem *mpa_rootItem;
+  QTreeView *mp_treeView;
+  MzCalculationDlg *mp_parentDlg;
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // MZ_CALCULATION_TREEVIEW_MODEL_HPP
