@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,36 +37,40 @@
 #include <QCloseEvent>
 
 
-/////////////////////// libmass includes
-#include <libXpertMass/IsotopicDataUserConfigHandler.hpp>
-#include <libXpertMass/PolChemDef.hpp>
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/IsotopicDataUserConfigHandler.hpp>
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+#include <MsXpS/libXpertMassCore/Monomer.hpp>
+#include <MsXpS/libXpertMassCore/Modif.hpp>
+#include <MsXpS/libXpertMassCore/CrossLinker.hpp>
 
 
 /////////////////////// Local includes
 #include "PolChemDefWnd.hpp"
 #include "ProgramWindow.hpp"
-//#include "AtomDefDlg.hpp"
+#include "Application.hpp"
+// #include "AtomDefDlg.hpp"
 #include "IsotopeDefDlg.hpp"
 #include "MonomerDefDlg.hpp"
 #include "ModifDefDlg.hpp"
-#include "CleaveSpecDefDlg.hpp"
-#include "FragSpecDefDlg.hpp"
+#include "CleavageAgentDefDlg.hpp"
+#include "FragmentationPathwayDefDlg.hpp"
 #include "CrossLinkerDefDlg.hpp"
 
 
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
 PolChemDefWnd::PolChemDefWnd(ProgramWindow *parent,
                              const QString &polChemDefFilePath,
-                             const QString &applicationName,
+                             const QString &application_name,
                              const QString &description)
   : AbstractMainTaskWindow(
-      parent, "PolChemDefWnd", applicationName, description)
+      parent, "PolChemDefWnd", application_name, description)
 {
   // Start by creating the polymer chemistry definition.
   // Note that it is perfectly possible to create a window with not polymer
@@ -74,7 +78,7 @@ PolChemDefWnd::PolChemDefWnd(ProgramWindow *parent,
   // polymer chemistry definition.
 
   // Allocate a brand new empty polymer chemistry definition.
-  msp_polChemDef = std::make_shared<libXpertMass::PolChemDef>();
+  msp_polChemDef = std::make_shared<libXpertMassCore::PolChemDef>();
 
   if(!polChemDefFilePath.isEmpty())
     msp_polChemDef->setXmlDataFilePath(polChemDefFilePath);
@@ -82,7 +86,7 @@ PolChemDefWnd::PolChemDefWnd(ProgramWindow *parent,
   if(!initialize())
     {
       QMessageBox::warning(0,
-                           QString("massXpert - %1").arg(m_wndTypeName),
+                           QString("MassXpert - %1").arg(m_wndTypeName),
                            "Failed to initialize the polymer "
                            "chemistry definition window.",
                            QMessageBox::Ok);
@@ -129,6 +133,8 @@ PolChemDefWnd::initialize()
 {
   m_ui.setupUi(this);
 
+  setWindowIcon(qApp->windowIcon());
+
   setAttribute(Qt::WA_DeleteOnClose);
   statusBar()->setSizeGripEnabled(true);
 
@@ -208,13 +214,28 @@ PolChemDefWnd::initialize()
 
 
   // SAVE/SAVE AS
-  connect(m_ui.savePushButton, SIGNAL(clicked()), this, SLOT(save()));
-
-  connect(m_ui.saveAsPushButton, SIGNAL(clicked()), this, SLOT(saveAs()));
+  connect(
+    m_ui.savePushButton, &QPushButton::clicked, this, &PolChemDefWnd::save);
+  connect(
+    m_ui.saveAsPushButton, &QPushButton::clicked, this, &PolChemDefWnd::saveAs);
 
 
   // VALIDATION
-  connect(m_ui.validatePushButton, SIGNAL(clicked()), this, SLOT(validate()));
+  connect(m_ui.validatePushButton, &QPushButton::clicked, this, [&]() {
+    libXpertMassCore::ErrorList error_list;
+    validate(&error_list);
+    if(error_list.size())
+      {
+        QMessageBox::warning(
+          this,
+          QString("%1 - %2").arg(m_applicationName).arg(m_windowDescription),
+          tr("%1@%2\n"
+             "Failed to validate the polymer chemistry definition.")
+            .arg(__FILE__)
+            .arg(__LINE__),
+          QMessageBox::Ok);
+      }
+  });
 
 
   // We have to set the name with the [*] placeholder right now
@@ -228,7 +249,7 @@ PolChemDefWnd::initialize()
     {
       // We first need to load the isotopic data.
 
-      if(!libXpertMass::PolChemDef::renderXmlPolChemDefFile(msp_polChemDef))
+      if(!libXpertMassCore::PolChemDef::renderXmlPolChemDefFile(msp_polChemDef))
         {
           QMessageBox::warning(
             this,
@@ -242,7 +263,7 @@ PolChemDefWnd::initialize()
           return false;
         }
 
-      m_ui.nameLineEdit->setText(msp_polChemDef->name());
+      m_ui.nameLineEdit->setText(msp_polChemDef->getName());
     }
   else
     {
@@ -255,8 +276,8 @@ PolChemDefWnd::initialize()
   if(!setSingularData())
     return false;
 
-  if(!msp_polChemDef->name().isEmpty())
-    statusBar()->showMessage(tr("Correctly opened file."));
+  if(!msp_polChemDef->getName().isEmpty())
+    statusBar()->showMessage("Correctly opened file.");
 
   setWindowModified(false);
 
@@ -269,8 +290,8 @@ PolChemDefWnd::nameEditFinished()
 {
   if(m_ui.nameLineEdit->text().isEmpty())
     statusBar()->showMessage(
-      tr("Name of the polymer chemistry "
-         "definition cannot be empty."));
+      "Name of the polymer chemistry "
+      "definition cannot be empty.");
   else
     {
       if(m_ui.nameLineEdit->isModified())
@@ -296,17 +317,21 @@ PolChemDefWnd::leftCapEditFinished()
       return;
     }
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
-  libXpertMass::Formula formula = libXpertMass::Formula(m_ui.leftCapLineEdit->text());
+  libXpertMassCore::Formula formula =
+    libXpertMassCore::Formula(m_ui.leftCapLineEdit->text());
 
-  if(!formula.validate(isotopic_data_csp))
+  libXpertMassCore::ErrorList error_list;
+
+  if(!formula.validate(isotopic_data_csp, &error_list))
     {
       statusBar()->showMessage(
-        tr("Formula(%1) is invalid.").arg(formula.toString()));
+        tr("Formula(%1) is invalid.").arg(formula.getActionFormula()));
 
-      m_ui.leftCapLineEdit->setText(msp_polChemDef->leftCap().toString());
+      m_ui.leftCapLineEdit->setText(
+        msp_polChemDef->getLeftCap().getActionFormula());
     }
   else
     {
@@ -332,17 +357,21 @@ PolChemDefWnd::rightCapEditFinished()
       return;
     }
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
-  libXpertMass::Formula formula = libXpertMass::Formula(m_ui.rightCapLineEdit->text());
+  libXpertMassCore::Formula formula =
+    libXpertMassCore::Formula(m_ui.rightCapLineEdit->text());
 
-  if(!formula.validate(isotopic_data_csp))
+  libXpertMassCore::ErrorList error_list;
+
+  if(!formula.validate(isotopic_data_csp, &error_list))
     {
       statusBar()->showMessage(
-        tr("Formula(%1) is invalid.").arg(formula.toString()));
+        tr("Formula(%1) is invalid.").arg(formula.getActionFormula()));
 
-      m_ui.rightCapLineEdit->setText(msp_polChemDef->rightCap().toString());
+      m_ui.rightCapLineEdit->setText(
+        msp_polChemDef->getRightCap().getActionFormula());
     }
   else
     {
@@ -366,23 +395,26 @@ PolChemDefWnd::ionizeFormulaEditFinished()
   if(!checkIsotopicData())
     return;
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
-  libXpertMass::Formula formula =
-    libXpertMass::Formula(m_ui.ionizeFormulaLineEdit->text());
+  libXpertMassCore::Formula formula =
+    libXpertMassCore::Formula(m_ui.ionizeFormulaLineEdit->text());
 
-  if(!formula.validate(isotopic_data_csp))
+  libXpertMassCore::ErrorList error_list;
+
+  if(!formula.validate(isotopic_data_csp, &error_list))
     {
       statusBar()->showMessage(
-        tr("Formula(%1) is invalid.").arg(formula.toString()));
+        tr("Formula(%1) is invalid.").arg(formula.getActionFormula()));
 
-      m_ui.ionizeFormulaLineEdit->setText(
-        msp_polChemDef->ionizeRule().formula());
+      m_ui.ionizeFormulaLineEdit->setText(msp_polChemDef->getIonizerCstRef()
+                                            .getFormulaCstRef()
+                                            .getActionFormula());
     }
   else
     {
-      msp_polChemDef->ionizeRulePtr()->setFormula(formula.toString());
+      msp_polChemDef->getIonizerRef().setFormula(formula.getActionFormula());
 
       m_ui.ionizeFormulaLineEdit->setModified(false);
 
@@ -398,7 +430,7 @@ PolChemDefWnd::ionizeChargeChanged()
 {
   int charge = m_ui.ionizeChargeSpinBox->value();
 
-  msp_polChemDef->ionizeRulePtr()->setCharge(charge);
+  msp_polChemDef->getIonizerRef().setNominalCharge(charge);
 
   setWindowModified(true);
 
@@ -411,7 +443,7 @@ PolChemDefWnd::ionizeLevelChanged()
 {
   int level = m_ui.ionizeLevelSpinBox->value();
 
-  msp_polChemDef->ionizeRulePtr()->setLevel(level);
+  msp_polChemDef->getIonizerRef().setLevel(level);
 
   setWindowModified(true);
 
@@ -422,18 +454,18 @@ PolChemDefWnd::ionizeLevelChanged()
 bool
 PolChemDefWnd::setSingularData()
 {
-  libXpertMass::Formula formula = msp_polChemDef->leftCap();
-  m_ui.leftCapLineEdit->setText(formula.toString());
+  m_ui.leftCapLineEdit->setText(
+    msp_polChemDef->getLeftCap().getActionFormula());
+  m_ui.rightCapLineEdit->setText(
+    msp_polChemDef->getRightCap().getActionFormula());
 
-  formula = msp_polChemDef->rightCap();
-  m_ui.rightCapLineEdit->setText(formula.toString());
+  m_ui.ionizeFormulaLineEdit->setText(
+    msp_polChemDef->getIonizerCstRef().getFormulaCstRef().getActionFormula());
 
-  libXpertMass::IonizeRule ionizeRule = msp_polChemDef->ionizeRule();
-  formula                        = ionizeRule.formula();
-  m_ui.ionizeFormulaLineEdit->setText(formula.toString());
-
-  m_ui.ionizeChargeSpinBox->setValue(ionizeRule.charge());
-  m_ui.ionizeLevelSpinBox->setValue(ionizeRule.level());
+  m_ui.ionizeChargeSpinBox->setValue(
+    msp_polChemDef->getIonizerCstRef().getNominalCharge());
+  m_ui.ionizeLevelSpinBox->setValue(
+    msp_polChemDef->getIonizerCstRef().getLevel());
 
   return true;
 }
@@ -442,7 +474,7 @@ PolChemDefWnd::setSingularData()
 bool
 PolChemDefWnd::checkIsotopicData()
 {
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
   if(isotopic_data_csp->size() < 1)
@@ -490,7 +522,7 @@ PolChemDefWnd::isotopePushButtonClicked()
       mpa_isotopeDefDlg =
         new IsotopeDefDlg(msp_polChemDef,
                           this,
-                          mp_parentWnd->configSettingsFilePath(),
+                          dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
                           m_applicationName,
                           "Isotope definitions");
 
@@ -528,7 +560,7 @@ PolChemDefWnd::monomerPushButtonClicked()
       mpa_monomerDefDlg =
         new MonomerDefDlg(msp_polChemDef,
                           this,
-                          mp_parentWnd->configSettingsFilePath(),
+                          dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
                           m_applicationName,
                           "Monomer definitions");
       mpa_monomerDefDlg->show();
@@ -562,11 +594,12 @@ PolChemDefWnd::modifPushButtonClicked()
 
   if(!mpa_modifDefDlg)
     {
-      mpa_modifDefDlg = new ModifDefDlg(msp_polChemDef,
-                                        this,
-                                        mp_parentWnd->configSettingsFilePath(),
-                                        m_applicationName,
-                                        "Modification definitions");
+      mpa_modifDefDlg =
+        new ModifDefDlg(msp_polChemDef,
+                        this,
+                        dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
+                        m_applicationName,
+                        "Modification definitions");
       mpa_modifDefDlg->show();
       m_ui.modifPushButton->setChecked(true);
 
@@ -601,7 +634,7 @@ PolChemDefWnd::crossLinkerPushButtonClicked()
       mpa_crossLinkerDefDlg =
         new CrossLinkerDefDlg(msp_polChemDef,
                               this,
-                              mp_parentWnd->configSettingsFilePath(),
+                              dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
                               m_applicationName,
                               "Cross-linker definitions");
       mpa_crossLinkerDefDlg->show();
@@ -636,11 +669,11 @@ PolChemDefWnd::cleavagePushButtonClicked()
   if(!mpa_cleaveSpecDefDlg)
     {
       mpa_cleaveSpecDefDlg =
-        new CleaveSpecDefDlg(msp_polChemDef,
-                             this,
-                             mp_parentWnd->configSettingsFilePath(),
-                             m_applicationName,
-                             "Cleavage specification definitions");
+        new CleavageAgentDefDlg(msp_polChemDef,
+                                this,
+                                dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
+                                m_applicationName,
+                                "Cleavage specification definitions");
       mpa_cleaveSpecDefDlg->show();
       m_ui.cleavagePushButton->setChecked(true);
 
@@ -672,12 +705,12 @@ PolChemDefWnd::fragmentationPushButtonClicked()
 
   if(!mpa_fragSpecDefDlg)
     {
-      mpa_fragSpecDefDlg =
-        new FragSpecDefDlg(msp_polChemDef,
-                           this,
-                           mp_parentWnd->configSettingsFilePath(),
-                           m_applicationName,
-                           "Fragmentation specification definitions");
+      mpa_fragSpecDefDlg = new FragmentationPathwayDefDlg(
+        msp_polChemDef,
+        this,
+        dynamic_cast<Application *>(qApp)->getUserConfigSettingsFilePath(),
+        m_applicationName,
+        "Fragmentation specification definitions");
       mpa_fragSpecDefDlg->show();
       m_ui.fragmentationPushButton->setChecked(true);
 
@@ -701,152 +734,123 @@ PolChemDefWnd::fragmentationPushButtonClicked()
 }
 
 
-int
-PolChemDefWnd::validate()
+bool
+PolChemDefWnd::validate(libXpertMassCore::ErrorList *error_list_p)
 {
-  QStringList errorList;
-
-  int total_error_count = 0;
-
   if(!checkIsotopicData())
     {
-      errorList << QString(
-        tr("The isotopic data contain not a single isotope."));
+      error_list_p->push_back(
+        "The isotopic data contain not a single isotope.");
 
       // We cannot even think of going on with the validation.
 
-      return 1;
+      return false;
     }
 
-  // We know we have isotopic data, check them before anything else because we'll
-  // need them for almost all the other validations below.
+  // We know we have isotopic data, check them before anything else because
+  // we'll need them for almost all the other validations below.
 
-  libXpertMass::IsotopicDataCstSPtr isotopic_data_csp =
+  libXpertMassCore::IsotopicDataCstSPtr isotopic_data_csp =
     msp_polChemDef->getIsotopicDataCstSPtr();
 
   // The validation of the isotopic data takes a vector of strings to store all
   // the errors.
-  std::vector<QString> errors;
-  total_error_count += isotopic_data_csp->validate(errors);
 
-  if(total_error_count)
+  if(!isotopic_data_csp->validate(error_list_p))
     {
-      for(auto error : errors)
-        errorList.append(error + "\n");
+      error_list_p->push_back("The isotopic data failed to validate.");
+
+      // We cannot even think of going on with the validation.
+
+      return false;
     }
 
   QString name = m_ui.nameLineEdit->text();
 
   if(name.isEmpty())
     {
-      errorList << QString(tr("Name cannot be empty."));
-      ++total_error_count;
+      error_list_p->push_back("Name cannot be empty.");
+      return false;
     }
 
-  libXpertMass::Formula formula = msp_polChemDef->leftCap();
-  if(!formula.validate(isotopic_data_csp))
+  const libXpertMassCore::Formula &lc_formula(msp_polChemDef->getLeftCap());
+
+  if(!lc_formula.validate(isotopic_data_csp, error_list_p))
     {
-      errorList << QString(tr("Validation of left cap failed."));
-      ++total_error_count;
+      error_list_p->push_back("Validation of left cap failed.");
+      return false;
     }
 
-  formula = msp_polChemDef->rightCap();
-  if(!formula.validate(isotopic_data_csp))
+  const libXpertMassCore::Formula &rc_formula(msp_polChemDef->getLeftCap());
+  if(!rc_formula.validate(isotopic_data_csp, error_list_p))
     {
-      errorList << QString(tr("Validation of right cap failed."));
-      ++total_error_count;
+      error_list_p->push_back("Validation of right cap failed.");
+      return false;
     }
 
-  libXpertMass::IonizeRule ionizeRule = msp_polChemDef->ionizeRule();
-  if(!ionizeRule.validate(isotopic_data_csp))
+  libXpertMassCore::Ionizer ionizer = msp_polChemDef->getIonizerCstRef();
+  if(!ionizer.validate(error_list_p))
     {
-      errorList << QString(tr("Validation of ionization rule failed."));
-      ++total_error_count;
+      error_list_p->push_back("Validation of ionizer failed.");
+      return false;
     }
 
-  for(int iter = 0; iter < msp_polChemDef->monomerList().size(); ++iter)
+  for(const libXpertMassCore::MonomerCstSPtr monomer_csp :
+      msp_polChemDef->getMonomersRef())
     {
-      libXpertMass::Monomer *monomer = msp_polChemDef->monomerList().at(iter);
-
-      if(!monomer->validate())
+      if(!monomer_csp->validate(error_list_p))
         {
-          errorList << QString(tr("Validation of the monomers failed."));
-
-          total_error_count += 1;
+          error_list_p->push_back("Validation of the monomers failed.");
+          return false;
         }
     }
 
-  for(int iter = 0; iter < msp_polChemDef->modifList().size(); ++iter)
+  for(libXpertMassCore::ModifCstSPtr modif_csp : msp_polChemDef->getModifsCstRef())
     {
-      libXpertMass::Modif *modif = msp_polChemDef->modifList().at(iter);
-
-      if(!modif->validate())
+      if(!modif_csp->validate(error_list_p))
         {
-          errorList << QString(tr("Validation of the modifications failed."));
-
-          total_error_count += 1;
+          error_list_p->push_back("Validation of the modifications failed.");
+          return false;
         }
     }
 
-  for(int iter = 0; iter < msp_polChemDef->crossLinkerList().size(); ++iter)
+  for(libXpertMassCore::CrossLinkerCstSPtr cross_linker_csp :
+      msp_polChemDef->getCrossLinkersCstRef())
     {
-      libXpertMass::CrossLinker *crossLinker =
-        msp_polChemDef->crossLinkerList().at(iter);
-
-      if(!crossLinker->validate())
+      if(!cross_linker_csp->validate(error_list_p))
         {
-          errorList << QString(tr("Validation of the cross-linkers failed."));
-
-          total_error_count += 1;
+          error_list_p->push_back("Validation of the cross-linkers failed.");
+          return false;
         }
     }
 
-  for(int iter = 0; iter < msp_polChemDef->cleaveSpecList().size(); ++iter)
+  for(libXpertMassCore::CleavageAgentCstSPtr cleavage_agent_csp :
+      msp_polChemDef->getCleavageAgentsCstRef())
     {
-      libXpertMass::CleaveSpec *cleaveSpec =
-        msp_polChemDef->cleaveSpecList().at(iter);
-
-      if(!cleaveSpec->validate())
+      if(!cleavage_agent_csp->validate(error_list_p))
         {
-          errorList << QString(
-            tr("Validation of the cleavage specifications failed."));
-
-          total_error_count += 1;
+          error_list_p->push_back(
+            "Validation of the cleavage specifications failed.");
+          return false;
         }
     }
 
-  for(int iter = 0; iter < msp_polChemDef->fragSpecList().size(); ++iter)
+  for(libXpertMassCore::FragmentationPathwayCstSPtr fragmentation_pathway_csp :
+      msp_polChemDef->getFragmentationPathwaysCstRef())
     {
-      libXpertMass::FragSpec *fragSpec = msp_polChemDef->fragSpecList().at(iter);
-
-      if(!fragSpec->validate())
+      if(!fragmentation_pathway_csp->validate(error_list_p))
         {
-          errorList << QString(
-            tr("Validation of the fragmentation specifications failed."));
-
-          total_error_count += 1;
+          error_list_p->push_back(
+            "Validation of the fragmentation pathways failed.");
+          return false;
         }
     }
 
-  if(total_error_count > 0)
-    {
-      QString errorMessage =
-        tr("Total number of errors: %1\n").arg(total_error_count);
+  statusBar()->showMessage(
+    "Polymer chemistry definition "
+    "data validation succeeded.");
 
-      errorMessage += errorList.join(QString(""));
-
-      QMessageBox::warning(
-        this,
-        QString("%1 - %2").arg(m_applicationName).arg(m_windowDescription),
-        errorMessage,
-        QMessageBox::Ok);
-    }
-  else
-    statusBar()->showMessage(
-      tr("Polymer chemistry definition "
-         "data validation succeeded."));
-
-  return total_error_count;
+  return true;
 }
 
 
@@ -903,9 +907,10 @@ PolChemDefWnd::save()
   // case we ask the saveAs() to do the job.
 
   // We only save if the validation is ok.
-  int error_count = validate();
+  libXpertMassCore::ErrorList error_list;
+  bool ok = validate(&error_list);
 
-  if(error_count)
+  if(!ok)
     {
       // Errors encountered.
 
@@ -913,8 +918,8 @@ PolChemDefWnd::save()
 
       msgBox.setText("Failed to save the document.\n\n");
       msgBox.setInformativeText(
-        "Please, validate the definition "
-        "prior to saving to file.");
+        QString("The encountered errors:\n%1")
+          .arg(libXpertMassCore::Utils::joinErrorList(error_list)));
       msgBox.setStandardButtons(QMessageBox::Ok);
 
       msgBox.exec();
@@ -984,7 +989,7 @@ PolChemDefWnd::save()
   // Now use the polymer chemistry definition save function.
 
   std::size_t isotope_count =
-    libXpertMass::PolChemDef::writeIsotopicData(msp_polChemDef);
+    libXpertMassCore::PolChemDef::writeIsotopicData(msp_polChemDef);
 
   if(!isotope_count)
     {
@@ -1007,7 +1012,7 @@ PolChemDefWnd::save()
   // Now save the isotopic data using the right isotopic data handler, that is,
   // the user config handler that mimicks the library handler.
 
-  libXpertMass::IsotopicDataUserConfigHandler isotopic_data_handler(
+  libXpertMassCore::IsotopicDataUserConfigHandler isotopic_data_handler(
     msp_polChemDef->getIsotopicDataSPtr());
 
   isotopic_data_handler.setFileName(isotopic_data_file_path);
@@ -1043,25 +1048,19 @@ PolChemDefWnd::save()
 bool
 PolChemDefWnd::saveAs()
 {
-  int ret = 0;
+  libXpertMassCore::ErrorList error_list;
+  bool ok = validate(&error_list);
 
-  // We must save the new contents of the polymer chemistry definition
-  // to an xml file that is not the file from which this definition
-  // might have been saved.
-
-  // We only save if the validation is ok.
-  ret += validate();
-
-  if(ret != 0)
+  if(!ok)
     {
       // Errors encountered.
 
       QMessageBox msgBox;
 
-      msgBox.setText("Failed to save the document.");
+      msgBox.setText("Failed to save the document.\n\n");
       msgBox.setInformativeText(
-        "Please, validate the definition "
-        "prior to saving to file.");
+        QString("The encountered errors:\n%1")
+          .arg(libXpertMassCore::Utils::joinErrorList(error_list)));
       msgBox.setStandardButtons(QMessageBox::Ok);
 
       msgBox.exec();
@@ -1109,6 +1108,6 @@ PolChemDefWnd::saveAs()
   return true;
 }
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
