@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,21 +30,32 @@
  * END software license
  */
 
+#pragma once
 
-#ifndef POL_CHEM_DEF_WND_HPP
-#define POL_CHEM_DEF_WND_HPP
+/////////////////////// stdlib includes
+
+
+/////////////////////// Qt includes
+
+
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+
+
+/////////////////////// libXpertMassGui includes
 
 
 /////////////////////// Local includes
 #include "ui_PolChemDefWnd.h"
 #include "AbstractMainTaskWindow.hpp"
-#include <libXpertMass/PolChemDef.hpp>
 
 
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
@@ -52,8 +63,8 @@ namespace massxpert
 class IsotopeDefDlg;
 class MonomerDefDlg;
 class ModifDefDlg;
-class CleaveSpecDefDlg;
-class FragSpecDefDlg;
+class CleavageAgentDefDlg;
+class FragmentationPathwayDefDlg;
 class CrossLinkerDefDlg;
 
 
@@ -65,21 +76,21 @@ class PolChemDefWnd : public AbstractMainTaskWindow
   friend class IsotopeDefDlg;
   friend class MonomerDefDlg;
   friend class ModifDefDlg;
-  friend class CleaveSpecDefDlg;
-  friend class FragSpecDefDlg;
+  friend class CleavageAgentDefDlg;
+  friend class FragmentationPathwayDefDlg;
   friend class CrossLinkerDefDlg;
 
   public:
   PolChemDefWnd(ProgramWindow *parent,
                 const QString &polChemDefFilePath,
-                const QString &applicationName,
+                const QString &application_name,
                 const QString &description);
 
   ~PolChemDefWnd();
 
   bool m_forciblyClose = false;
 
-  libXpertMass::PolChemDefSPtr msp_polChemDef;
+  libXpertMassCore::PolChemDefSPtr msp_polChemDef;
 
 
   bool maybeSave();
@@ -117,7 +128,7 @@ class PolChemDefWnd : public AbstractMainTaskWindow
   bool saveAs();
 
   // VALIDATION
-  int validate();
+  bool validate(libXpertMassCore::ErrorList *error_list);
 
   private:
   Ui::PolChemDefWnd m_ui;
@@ -125,21 +136,18 @@ class PolChemDefWnd : public AbstractMainTaskWindow
   // Init to 0 all the window pointers, because we'll need to check
   // the construction of the these windows.
   // AtomDefDlg *mpa_atomDefDlg               = 0;
-  IsotopeDefDlg *mpa_isotopeDefDlg         = 0;
-  MonomerDefDlg *mpa_monomerDefDlg         = 0;
-  ModifDefDlg *mpa_modifDefDlg             = 0;
-  CleaveSpecDefDlg *mpa_cleaveSpecDefDlg   = 0;
-  FragSpecDefDlg *mpa_fragSpecDefDlg       = 0;
-  CrossLinkerDefDlg *mpa_crossLinkerDefDlg = 0;
+  IsotopeDefDlg *mpa_isotopeDefDlg               = 0;
+  MonomerDefDlg *mpa_monomerDefDlg               = 0;
+  ModifDefDlg *mpa_modifDefDlg                   = 0;
+  CleavageAgentDefDlg *mpa_cleaveSpecDefDlg      = 0;
+  FragmentationPathwayDefDlg *mpa_fragSpecDefDlg = 0;
+  CrossLinkerDefDlg *mpa_crossLinkerDefDlg       = 0;
 
   bool initialize();
 
   void closeEvent(QCloseEvent *event);
 };
 
-} // namespace massxpert
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif // POL_CHEM_DEF_WND_HPP
