@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,38 +34,65 @@
 #pragma once
 
 
+/////////////////////// stdlib includes
+
+
 /////////////////////// Qt includes
 #include <QMainWindow>
 #include <QGraphicsScene>
 #include <QClipboard>
 #include <QProgressBar>
+#include <QListWidgetItem>
+
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+// For MSXPS_REGISTER_JS_CLASS at the end of the class declaration
+#include <MsXpS/libXpertMassCore/CalcOptions.hpp>
+#include <MsXpS/libXpertMassCore/Ionizer.hpp>
+#include <MsXpS/libXpertMassCore/PolChemDef.hpp>
+
+
+/////////////////////// libXpertMassGui includes
+#include <MsXpS/libXpertMassGui/MassPeakShaperConfigDlg.hpp>
 
-/////////////////////// libmassgui includes
-#include <libXpertMassGui/MassPeakShaperConfigDlg.hpp>
-#include <pappsomspp/trace/trace.h>
 
 /////////////////////// Local includes
-#include "ui_SequenceEditorWnd.h"
+#include "SequenceEditorWnd.hpp"
 #include "AbstractMainTaskWindow.hpp"
-#include <libXpertMass/CalcOptions.hpp>
-#include <libXpertMass/IonizeRule.hpp>
-#include <libXpertMass/PolChemDef.hpp>
 #include "MonomerCodeEvaluator.hpp"
 #include "PolChemDefRendering.hpp"
 #include "SequenceEditorGraphicsView.hpp"
 
+namespace Ui
+{
+class SequenceEditorWnd;
+}
+
 
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
 class SequenceEditorGraphicsView;
-
-
 class SequenceEditorFindDlg;
+class CleavageDlg;
+class FragmentationDlg;
+class MassSearchDlg;
+class MzCalculationDlg;
+class CompositionsDlg;
+class PkaPhPiDlg;
+class MonomerModificationDlg;
+class PolymerModificationDlg;
+class MonomerCrossLinkDlg;
+
+/*  BEGIN CLASS JS REFERENCE
+ *  namespace: MsXpS::MassXpert
+ *  class name: SequenceEditorWnd
+ */
 
 class SequenceEditorWnd : public AbstractMainTaskWindow
 {
@@ -73,7 +100,7 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
 
   public:
   SequenceEditorWnd(ProgramWindow *parent,
-                    const QString &applicationName,
+                    const QString &application_name,
                     const QString &description);
 
   ~SequenceEditorWnd();
@@ -86,26 +113,41 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
 
   QProgressBar *progressBar();
 
-  bool openSequence(QString &);
-  bool newSequence(QString &);
-  bool maybeSave();
+  Q_INVOKABLE bool openSequence(QString &sequence_file_path);
+  Q_INVOKABLE bool newSequence(QString &pol_chem_def_file_path);
+  Q_INVOKABLE bool maybeSave();
 
-  void updateMonomerPosition(int);
+  Q_INVOKABLE void updateMonomerPosition(int);
 
-  PolChemDefRendering *preparePolChemDefRendering(const QString &);
+  bool preparePolChemDefRendering(const QString &name);
   void populateCalculationOptions();
-  bool populateMonomerCodeList(bool = false);
+  bool populateAvailableMonomerCodesList(bool = false);
 
-  bool readFile(const QString &);
+  Q_INVOKABLE bool readFile(const QString &file_path);
 
-  const PolChemDefRendering *polChemDefRendering() const;
-  PolChemDefRendering *polChemDefRendering();
+  Q_INVOKABLE int
+  setIndexRanges(const QString &text,
+                 libXpertMassCore::Enums::LocationType location_type =
+                   libXpertMassCore::Enums::LocationType::INDEX);
 
-  libXpertMass::Polymer *polymer();
-  QList<libXpertMass::Prop *> *propList();
+  PolChemDefRenderingCstRPtr getPolChemDefRenderingCstRPtr() const;
+  PolChemDefRenderingRPtr getPolChemDefRenderingRPtr();
 
-  const libXpertMass::CalcOptions &calcOptions() const;
-  const libXpertMass::IonizeRule &ionizeRule() const;
+  Q_INVOKABLE libXpertMassCore::PolymerQSPtr getPolymerSPtr();
+  Q_INVOKABLE libXpertMassCore::Polymer *getPolymer();
+
+  QList<libXpertMassCore::Prop *> *propList();
+
+  Q_INVOKABLE void
+  setCalcOptions(const libXpertMassCore::CalcOptions *calc_options_p);
+  Q_INVOKABLE void
+  setCalcOptions(const libXpertMassCore::CalcOptions &calc_options);
+  Q_INVOKABLE libXpertMassCore::CalcOptions *getCalcOptions();
+  Q_INVOKABLE const libXpertMassCore::CalcOptions &getCalcOptionsCstRef() const;
+
+  Q_INVOKABLE const libXpertMassCore::Ionizer &getIonizerCstRef() const;
+  Q_INVOKABLE libXpertMassCore::Ionizer &getIonizerRef();
+  Q_INVOKABLE libXpertMassCore::Ionizer *getIonizer();
 
   void clearCompletionsListSelection();
   void completionsListSelectAt(int);
@@ -135,12 +177,19 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
   bool save();
   bool saveAs();
 
-  void updateWholeSequenceMasses(bool = false);
-  void updateSelectedSequenceMasses(bool = false);
+  void updateMassesNewDecimalsOptions();
 
-  // libXpertMass::Sequence importers.
+  // Whole sequence
+  void initialNonIonizedWholePolymerSequenceMassCalculation(bool deep = false);
+  void updateWholeSequenceMasses(bool deep = false);
+
+  // Selected sequence
+  void
+  initialNonIonizedSelectedPolymerSequenceMassCalculation(bool deep = false);
+  void updateSelectedSequenceMasses(bool deep = false);
+
+  // libXpertMassCore::Sequence importers.
   void importRaw();
-  void importPdbProt();
 
   void exportClipboard();
   void exportFile();
@@ -162,6 +211,8 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
   void rightModifOptionsChanged();
   void forceRightModifOptionsChanged();
 
+  void setMultiRegionSelection(bool is_multi_region_selection);
+  void setMultiSelectionRegion(bool is_multi_selection_region);
   void multiRegionSelectionOptionChanged(int);
   bool isMultiRegionSelection();
 
@@ -177,20 +228,21 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
 
   void monomerCrossLinkOptionChanged(int);
 
-  void modifMonomer();
+  Q_INVOKABLE MonomerModificationDlg *modifMonomer();
 
-  void modifPolymer();
-  void modifLeftEnd();
-  void modifRightEnd();
+  Q_INVOKABLE PolymerModificationDlg *modifPolymer();
+  Q_INVOKABLE PolymerModificationDlg *modifLeftEnd();
+  Q_INVOKABLE PolymerModificationDlg *modifRightEnd();
+  Q_INVOKABLE void modifPolymerEnd(libXpertMassCore::Enums::PolymerEnd end,
+                                   const QString &modif_name);
 
-  void crossLinkMonomers();
-
-  void cleave();
-  void fragment();
-  void massSearch();
-  void mzCalculation();
-  void compositions();
-  void pkaPhPi();
+  MonomerCrossLinkDlg *crossLinkMonomers();
+  CleavageDlg *cleave();
+  FragmentationDlg *fragment();
+  MassSearchDlg *massSearch();
+  MzCalculationDlg *mzCalculation();
+  CompositionsDlg *compositions();
+  PkaPhPiDlg *pkaPhPi();
 
   void decimalPlacesOptions();
 
@@ -201,14 +253,15 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
 
   void vignetteListWidgetItemDoubleClicked(QListWidgetItem *);
 
-  QString getSequenceName() const;
+  Q_INVOKABLE QString getSequenceName() const;
 
-  private:
-  Ui::SequenceEditorWnd m_ui;
+  static void registerJsConstructor(QJSEngine *engine);
 
-  libXpertMassGui::MassPeakShaperConfigDlg *mp_massPeakShaperConfigDlg = nullptr;
+  protected:
+  libXpertMassGui::MassPeakShaperConfigDlg *mp_massPeakShaperConfigDlg =
+    nullptr;
 
-  libXpertMass::IsotopicDataSPtr msp_isotopicData = nullptr;
+  libXpertMassCore::IsotopicDataSPtr msp_isotopicData = nullptr;
 
   // The results-exporting strings. ////////////////////////////////
   QString *mpa_resultsString;
@@ -226,7 +279,6 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
   QAction *saveAct;
   QAction *saveAsAct;
   QAction *importRawAct;
-  QAction *importPdbProtAct;
   QAction *exportClipboardAct;
   QAction *exportFileAct;
   QAction *exportSelectFileAct;
@@ -254,24 +306,27 @@ class SequenceEditorWnd : public AbstractMainTaskWindow
   // Allocated but reparented to QStatusBar.
   QProgressBar *mp_progressBar;
 
-  libXpertMass::Polymer *mpa_polymer = 0;
-  PolChemDefRendering *mp_polChemDefRendering;
-  libXpertMass::CalcOptions m_calcOptions;
-  libXpertMass::IonizeRule m_ionizeRule;
+  libXpertMassCore::PolymerQSPtr msp_polymer = nullptr;
+  PolChemDefRenderingUPtr mup_polChemDefRendering;
+  libXpertMassCore::CalcOptions *mp_calcOptions = nullptr;
 
-  QList<libXpertMass::Prop *> m_propList;
+  QList<libXpertMassCore::Prop *> m_propList;
 
-libXpertMass::MassPeakShaperConfig m_massPeakShaperConfig;
+  libXpertMassCore::MassPeakShaperConfig m_massPeakShaperConfig;
 
   pappso::Trace m_syntheticMassSpectrum;
 
   // For the mass spectra that are synthesized and served.
   QByteArray m_colorByteArray;
 
+
+  Ui::SequenceEditorWnd *mp_ui;
+
   void writeSettings();
   void readSettings();
 
   // Before the creation of the polChemDef/polymer relationship.
+  Q_INVOKABLE bool initializeIonizerFromPolChemDef();
   bool initialize();
 
   // Creates the polChemDef/polymer relationship.
@@ -288,16 +343,30 @@ libXpertMass::MassPeakShaperConfig m_massPeakShaperConfig;
   void keyPressEvent(QKeyEvent *);
 
   void massSpectrumSynthesisMenuActivated(int index);
-  bool loadIsotopicDataFromFile();
+  Q_INVOKABLE bool
+  loadIsotopicDataFromFile(const QString &file_path = QString());
+
+  Q_INVOKABLE void setCumulatedProbabilities(double cumul_probs);
+  Q_INVOKABLE void setNormalizationIntensity(double intensity);
+
   void traceColorPushButtonClicked();
-  void configureMassPeakShaper();
-  void synthesizeMassSpectra();
+  Q_INVOKABLE void setTraceColor(QColor color);
+  Q_INVOKABLE void setTraceTitle(const QString &title);
+
+  Q_INVOKABLE libXpertMassGui::MassPeakShaperConfigDlg *
+  configureMassPeakShaper();
+  Q_INVOKABLE void synthesizeMassSpectra();
 
   // The results-exporting functions. ////////////////////////////////
-  void prepareResultsTxtString();
+  Q_INVOKABLE void prepareResultsTxtString();
   //////////////////////////////////// The results-exporting functions.
 };
 
-} // namespace massxpert
+/*  END CLASS JS REFERENCE
+ *  namespace: MsXpS::MassXpert
+ *  class name: SequenceEditorWnd
+ */
 
+} // namespace MassXpert
+MSXPS_REGISTER_JS_CLASS(MsXpS::MassXpert, SequenceEditorWnd)
 } // namespace MsXpS
