@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,27 +45,25 @@
 namespace MsXpS
 {
 
-namespace massxpert
+namespace MassXpert
 {
 
 
-ConfigSetting::ConfigSetting(const QString &applicationName)
-  : m_applicationName{applicationName}
+ConfigSetting::ConfigSetting(const QString &application_name)
+  : m_applicationName{application_name}
 {
 }
 
 
 ConfigSetting::ConfigSetting(const ConfigSetting &other)
+  : m_title(other.m_title),
+    m_applicationName(other.m_applicationName),
+    m_userType(other.m_userType),
+    m_key(other.m_key),
+    m_value(other.m_value),
+    m_comment(other.m_comment),
+    m_prefixDelimiter(other.m_prefixDelimiter)
 {
-  m_applicationName = other.m_applicationName;
-  m_prefixDelimiter = other.m_prefixDelimiter;
-
-  m_userType = other.m_userType;
-  m_key      = other.m_key;
-  m_value    = other.m_value;
-
-  m_title   = other.m_title;
-  m_comment = other.m_comment;
 }
 
 
@@ -75,15 +73,16 @@ ConfigSetting::operator=(const ConfigSetting &other)
   if(&other == this)
     return *this;
 
+  m_title           = other.m_title;
   m_applicationName = other.m_applicationName;
-  m_prefixDelimiter = other.m_prefixDelimiter;
 
   m_userType = other.m_userType;
-  m_key      = other.m_key;
-  m_value    = other.m_value;
 
-  m_title   = other.m_title;
-  m_comment = other.m_comment;
+  m_key   = other.m_key;
+  m_value = other.m_value;
+
+  m_comment         = other.m_comment;
+  m_prefixDelimiter = other.m_prefixDelimiter;
 
   return *this;
 }
@@ -94,27 +93,13 @@ ConfigSetting::~ConfigSetting()
 }
 
 
-void
-ConfigSetting::setApplicationName(const QString &applicationName)
-{
-  m_applicationName = applicationName;
-}
-
-
 QString
-ConfigSetting::getApplicationName()
-{
-  return m_applicationName;
-}
-
-
-QString
-ConfigSetting::text() const
+ConfigSetting::toString() const
 {
   QString text = m_title;
   text += "\n";
 
-  if(m_userType == UserType::USER_TYPE_USER)
+  if(m_userType == UserType::USER)
     text += "User type : user\n";
   else
     text += "User type : system\n";
@@ -128,6 +113,5 @@ ConfigSetting::text() const
 }
 
 
-} // namespace massxpert
-
+} // namespace MassXpert
 } // namespace MsXpS
