@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +31,7 @@
  */
 
 
-#ifndef CONFIG_SETTING_HPP
-#define CONFIG_SETTING_HPP
+#pragma once
 
 /////////////////////// Qt includes
 #include <QString>
@@ -42,23 +41,22 @@
 /////////////////////// Local includes
 #include "globals.hpp"
 
+
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
 class ConfigSetting
 {
-  private:
-  QString m_applicationName;
-  QChar m_prefixDelimiter = gConfigKeyStringPrefixDelimiter;
-
   public:
-  // Is the key for user or system config settings (USER_TYPE_USER ||
-  // USER_TYPE_SYSTEM)?
-  int m_userType;
+  QString m_title;
+  QString m_applicationName;
+
+  // Is the key for user or system config settings (UserType::USER or
+  // UserType::SYSTEM)?
+  UserType m_userType;
 
   // The key is a string like dataDir or polChemDefsDir or docDir...
   QString m_key;
@@ -66,23 +64,19 @@ class ConfigSetting
   // The value can be bool, int, double or QString.
   QVariant m_value;
 
-  QString m_title;
   QString m_comment;
+  QChar m_prefixDelimiter = gConfigKeyStringPrefixDelimiter;
 
-  ConfigSetting(const QString &applicationName);
+  ConfigSetting(const QString &application_name);
   ConfigSetting(const ConfigSetting &other);
   ConfigSetting &operator=(const ConfigSetting &other);
   ~ConfigSetting();
 
-  void setApplicationName(const QString &applicationName);
-  QString getApplicationName();
-
-  QString text() const;
+  QString toString() const;
 };
 
-} // namespace massxpert
+typedef std::shared_ptr<ConfigSetting> ConfigSettingSPtr;
+typedef std::shared_ptr<const ConfigSetting> ConfigSettingCstSPtr;
 
+} // namespace MassXpert
 } // namespace MsXpS
-
-
-#endif /* CONFIG_SETTING_HPP */
