@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,8 @@
 namespace MsXpS
 {
 
-	namespace massxpert
-	{
-
+namespace MassXpert
+{
 
 
 class UserSpec
@@ -73,7 +72,7 @@ class UserSpec
   const QString &dataDir() const;
 };
 
-} // namespace massxpert
+} // namespace MassXpert
 
 } // namespace MsXpS
 
