@@ -1,18 +1,18 @@
 /* BEGIN software license
  *
- * msXpertSuite - mass spectrometry software suite
+ * MsXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright(C) 2009,...,2026 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
- * This file is part of the msXpertSuite project.
+ * This file is part of the MsXpertSuite project.
  *
- * The msXpertSuite project is the successor of the massXpert project. This
+ * The MsXpertSuite project is the successor of the massXpert project. This
  * project now includes various independent modules:
  *
- * - massXpert, model polymer chemistries and simulate mass spectrometric data;
- * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - MassXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - MineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,14 +33,31 @@
 
 #pragma once
 
+/////////////////////// stdlib includes
 
-////////////////////////////// Qt includes
+
+/////////////////////// Qt includes
 #include <QString>
 #include <QList>
 #include <QTextStream>
 #include <QRegularExpression>
 
 
+/////////////////////// pappsomspp includes
+
+
+/////////////////////// libXpertMass includes
+#include <MsXpS/libXpertMassCore/Ionizer.hpp>
+
+/////////////////////// libXpertMassGui includes
+
+
+/////////////////////// Local includes
+
+
+////////////////////////////// Qt includes
+
+
 /*
  * Bitwise stuff (from StackOverflow)
 
@@ -65,24 +82,43 @@
 
 namespace MsXpS
 {
-
-namespace massxpert
+namespace MassXpert
 {
 
 
-  //! Type of user that makes use of resources.
-  /*!
-   * When looking for resources, this enum variables allow defining if the
-   * resources are to be searched in system directories or in the user $HOME
-   * directory.
-   */
-  enum UserType
-  {
-    USER_TYPE_USER   = 0, //!< User is a logon user
-    USER_TYPE_SYSTEM = 1  //!< User is the system
-  };
+// Macro to fake qFatal() << streaming in Qt 6.4
+#if QT_VERSION < QT_VERSION_CHECK(6, 8, 0) // If Qt < 6.8
+#define qFatalStream()                             \
+  for(QString fatalMsg; fatalMsg.isEmpty();        \
+      qFatal("%s", fatalMsg.toUtf8().constData())) \
+  QDebug(&fatalMsg).noquote().nospace()
+#else
+#define qFatalStream() qFatal()
+#endif
 
-  extern QChar gConfigKeyStringPrefixDelimiter;
+
+//! Type of user that makes use of resources.
+/*!
+ * When looking for resources, this enum variables allow defining if the
+ * resources are to be searched in system directories or in the user $HOME
+ * directory (or BOTH).
+ */
+enum class UserType
+{
+  USER   = 1 << 0, //!< User is a logon user
+  SYSTEM = 1 << 1, //!< User is the system
+  BOTH   = USER | SYSTEM
+};
+
+
+enum class ContainerItemAddMode
+{
+  PREPEND,
+  APPEND,
+};
+
+
+extern QChar gConfigKeyStringPrefixDelimiter;
 
 
 enum class MassSpectrumSynthesisActions
@@ -95,6 +131,16 @@ enum class MassSpectrumSynthesisActions
 };
 
 
-} // namespace massxpert
+struct Ion
+{
+  double mono     = 0.0;
+  double avg      = 0.0;
+  QString formula = "";
+  libXpertMassCore::IonizerSPtr ionizer_sp;
+};
 
+typedef std::shared_ptr<Ion> IonSPtr;
+
+
+} // namespace MassXpert
 } // namespace MsXpS
